/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


static struct sock *			nlk_sock = NULL;


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void
_handle_rx_for_tty(
	unsigned char *				buff,
	unsigned int				length
	)
{
	PNLK_HEADER					h_ptr = NULL;
	struct snxdp_ser_port *		sp = NULL;
	struct sdp_ser_port *		port = NULL;
	unsigned int				event_length = 0;
	unsigned int				data_length = 0;


	do
	{
		h_ptr = (PNLK_HEADER)buff;
		sp = &snxdp_ser_table[h_ptr->MinorNumber];
		port = &snxdp_ser_table[h_ptr->MinorNumber].port;
		if ((sp == NULL) || (port == NULL))
		{
			printk("SDDVIRTUAL : (M:%d) %s, Can't get port.\n", h_ptr->MinorNumber, __FUNCTION__);
			break;
		}

		switch (h_ptr->CmdResponseEventData)
		{
			case NLK_TTY_RESPONSE_OPEN :
				if (port->rx_response_buff != NULL)
				{
					memset(port->rx_response_buff, 0, NLK_MSG_MAX_PAYLOAD);
					memcpy(port->rx_response_buff, buff, length);
					port->rx_response_length = length;
					complete(&port->open_comp);
				}
				break;

			case NLK_TTY_RESPONSE_CLOSE :
				if (port->rx_response_buff != NULL)
				{
					memset(port->rx_response_buff, 0, NLK_MSG_MAX_PAYLOAD);
					memcpy(port->rx_response_buff, buff, length);
					port->rx_response_length = length;
					complete(&port->close_comp);
				}
				break;

			case NLK_TTY_RESPONSE_SET_DLL_DLM :
				if (port->rx_response_buff != NULL)
				{
					memset(port->rx_response_buff, 0, NLK_MSG_MAX_PAYLOAD);
					memcpy(port->rx_response_buff, buff, length);
					port->rx_response_length = length;
#if (DRIVERTARGET == 1)
					complete(&port->set_dll_dlm_comp);
#endif
				}
				break;

			case NLK_TTY_RESPONSE_SET_LCR :
				if (port->rx_response_buff != NULL)
				{
					memset(port->rx_response_buff, 0, NLK_MSG_MAX_PAYLOAD);
					memcpy(port->rx_response_buff, buff, length);
					port->rx_response_length = length;
#if (DRIVERTARGET == 1)
					complete(&port->set_lcr_comp);
#endif
				}
				break;

			case NLK_TTY_RESPONSE_SET_FIFO_CTRL :
				if (port->rx_response_buff != NULL)
				{
					memset(port->rx_response_buff, 0, NLK_MSG_MAX_PAYLOAD);
					memcpy(port->rx_response_buff, buff, length);
					port->rx_response_length = length;
#if (DRIVERTARGET == 1)
					complete(&port->set_fifo_ctrl_comp);
#endif
				}
				break;

			case NLK_TTY_RESPONSE_SET_MCR :
				if (port->rx_response_buff != NULL)
				{
					memset(port->rx_response_buff, 0, NLK_MSG_MAX_PAYLOAD);
					memcpy(port->rx_response_buff, buff, length);
					port->rx_response_length = length;
#if (DRIVERTARGET == 1)
					complete(&port->set_mcr_comp);
#endif
				}
				break;

			case NLK_TTY_RESPONSE_SET_EVENT_CTRL :
				if (port->rx_response_buff != NULL)
				{
					memset(port->rx_response_buff, 0, NLK_MSG_MAX_PAYLOAD);
					memcpy(port->rx_response_buff, buff, length);
					port->rx_response_length = length;
#if (DRIVERTARGET == 1)
					complete(&port->set_event_ctrl_comp);
#endif
				}
				break;

			case NLK_TTY_RESPONSE_GET_MSR :
				if (port->rx_response_buff != NULL)
				{
					memset(port->rx_response_buff, 0, NLK_MSG_MAX_PAYLOAD);
					memcpy(port->rx_response_buff, buff, length);
					port->rx_response_length = length;
					complete(&port->get_msr_comp);
				}
				break;

			case NLK_TTY_RESPONSE_SET_ON_CHIP_HARDWARE_FLOW_CTRL :
				if (port->rx_response_buff != NULL)
				{
					memset(port->rx_response_buff, 0, NLK_MSG_MAX_PAYLOAD);
					memcpy(port->rx_response_buff, buff, length);
					port->rx_response_length = length;
#if (DRIVERTARGET == 1)
					complete(&port->set_on_chip_hw_flow_ctrl_comp);
#endif
				}
				break;

			case NLK_TTY_RESPONSE_SET_TX_TRIGGER_LEVEL :
				if (port->rx_response_buff != NULL)
				{
					memset(port->rx_response_buff, 0, NLK_MSG_MAX_PAYLOAD);
					memcpy(port->rx_response_buff, buff, length);
					port->rx_response_length = length;
#if (DRIVERTARGET == 1)
					complete(&port->set_tx_trigger_level_comp);
#endif
				}
				break;

			case NLK_TTY_RESPONSE_SET_RX_TRIGGER_LEVEL :
				if (port->rx_response_buff != NULL)
				{
					memset(port->rx_response_buff, 0, NLK_MSG_MAX_PAYLOAD);
					memcpy(port->rx_response_buff, buff, length);
					port->rx_response_length = length;
#if (DRIVERTARGET == 1)
					complete(&port->set_rx_trigger_level_comp);
#endif
				}
				break;

			case NLK_TTY_EVENT_MSR :
				if (port->opened == true)
				{
					event_length = h_ptr->Length;
					if ((port->rx_event_buff != NULL) && (event_length > 0))
					{
						memset(port->rx_event_buff, 0, NLK_MSG_MAX_PAYLOAD);
						memcpy(port->rx_event_buff, buff + sizeof(NLK_HEADER), event_length);
						port->rx_event_length = event_length;
						port->rx_event_type = NLK_TTY_EVENT_MSR;
						snxdp_ser_handle_port(sp);
					}
				}
				break;

			case NLK_TTY_EVENT_LSR :
				if (port->opened == true)
				{
					event_length = h_ptr->Length;
					if ((port->rx_event_buff != NULL) && (event_length > 0))
					{
						memset(port->rx_event_buff, 0, NLK_MSG_MAX_PAYLOAD);
						memcpy(port->rx_event_buff, buff + sizeof(NLK_HEADER), event_length);
						port->rx_event_length = event_length;
						port->rx_event_type = NLK_TTY_EVENT_LSR;
						snxdp_ser_handle_port(sp);
					}
				}
				break;

			case NLK_TTY_EVENT_DATA_SERVICE_TO_KERNEL :
				if (port->opened == true)
				{
					event_length = h_ptr->Length;
					if ((port->rx_event_buff != NULL) && (event_length > 0))
					{
						memset(port->rx_event_buff, 0, NLK_MSG_MAX_PAYLOAD);
						memcpy(port->rx_event_buff, buff + sizeof(NLK_HEADER), event_length);
						port->rx_event_length = event_length;
						port->rx_event_type = NLK_TTY_EVENT_DATA_SERVICE_TO_KERNEL;
						snxdp_ser_handle_port(sp);
					}
				}
				break;

			case NLK_TTY_DATA_SERVICE_TO_KERNEL :
				if (port->opened == true)
				{
					data_length = h_ptr->Length;
					if ((port->rx_data_buff != NULL) && (data_length > 0))
					{
						memset(port->rx_data_buff, 0, NLK_MSG_MAX_PAYLOAD);
						memcpy(port->rx_data_buff, buff + sizeof(NLK_HEADER), data_length);
						port->rx_data_length = data_length;
						snxdp_ser_handle_port(sp);
					}
				}
				break;

			default :
				printk("SDDVIRTUAL : (M:%d) %s, Unknow packet type.\n", h_ptr->MinorNumber, __FUNCTION__);
				break;
		}


	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_nlk_rx(
	struct sk_buff *			skb_rx
	)
{
	struct nlmsghdr *			nlh_rx = NULL;
	unsigned int				length = 0;
	PNLK_HEADER					h_ptr = NULL;


	do
	{
		nlh_rx = (struct nlmsghdr *)skb_rx->data;
		length = (unsigned int)NLMSG_PAYLOAD(nlh_rx, 0);
		if (length < sizeof(NLK_HEADER))
		{
			printk("SDDVIRTUAL : %s, Receive length(%d) too small.\n", __FUNCTION__, length);
			break;
		}

		h_ptr = (PNLK_HEADER)NLMSG_DATA(nlh_rx);

		switch (h_ptr->DeviceType)
		{
			case NLK_DEVICE_TYPE_TTY :
//				printk("SDDVIRTUAL : %s, TTY length:%d\n", __FUNCTION__, length);
				_handle_rx_for_tty((unsigned char *)NLMSG_DATA(nlh_rx), length);
				break;

			default :
				printk("SDDVIRTUAL : %s, Unknow device type.\n", __FUNCTION__);
				break;
		}

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
snxdp_nlk_tx(
	unsigned char *				data,
	int							length
	)
{
	struct nlmsghdr *			nlh_tx = NULL;
	struct sk_buff *			skb_tx = NULL;
	int 						msg_tx_size = 0;
	int 						retval = 0;


	do
	{
		if (length > NLK_MSG_MAX_PAYLOAD)
		{
			printk("SDDVIRTUAL : %s, Transmit length(%d) too big.\n", __FUNCTION__, length);
			retval = -ENOMEM;
			break;
		}

		msg_tx_size = length;
		skb_tx = nlmsg_new(msg_tx_size, 0);
		if (skb_tx == NULL)
		{
			printk("SDDVIRTUAL : %s, Create message fail.\n", __FUNCTION__);
			retval = -ENOMEM;
			break;
		}

		nlh_tx = nlmsg_put(skb_tx, 0, 1, NLMSG_DONE, msg_tx_size, 0);
		if (nlh_tx == NULL)
		{
			printk("SDDVIRTUAL : %s, Copy data to message fail.\n", __FUNCTION__);
			retval = -ECOMM;
			break;
		}

		memcpy(nlmsg_data(nlh_tx), data, msg_tx_size);

		retval = nlmsg_multicast(nlk_sock, skb_tx, 0, NLK_GROUP, GFP_KERNEL);
		if (retval < 0)
		{
			printk("SDDVIRTUAL : %s, Sending message to user fail(retval:%d, msg_tx_size:%d).\n", __FUNCTION__, retval, msg_tx_size);
			retval = -ECOMM;
			break;
		}

		retval = 0;

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
snxdp_nlk_init(
	void
	)
{
	int							retval = 0;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,5,8)) 
	struct netlink_kernel_cfg 	nlk_cfg = 
	{
		.input = snxdp_nlk_rx,
		.groups = NLK_GROUP,
	};


	do
	{
		nlk_sock = netlink_kernel_create(&init_net, NLK_PROTO, &nlk_cfg);
		if (nlk_sock == NULL)
		{
			printk("SDDVIRTUAL : %s, Create socket fail.\n", __FUNCTION__);
			retval = -ESOCKTNOSUPPORT;
			break;
		}

	} while (false);
#else
	do
	{
		nlk_sock = netlink_kernel_create(&init_net, NLK_PROTO, NLK_GROUP, snxdp_nlk_rx, NULL, THIS_MODULE);
		if (nlk_sock == NULL)
		{
			printk("SDDVIRTUAL : %s, Create socket fail.\n", __FUNCTION__);
			retval = -ESOCKTNOSUPPORT;
			break;
		}

	} while (false);
#endif
	if (retval != 0)
	{

	}

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void 
snxdp_nlk_exit(
	void
	)
{
	do
	{
		netlink_kernel_release(nlk_sock);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


