/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
ser_cmd_set_dll_dlm(
	struct snxdp_ser_port *		sp,
	unsigned char				dll,
	unsigned char				dlm
	)
{
	int							retval = 0;
	struct sdp_ser_port *		port = &sp->port;
	unsigned char				data[256];
	int							length = 0;
	PNLK_HEADER					h_ptr = NULL;
	

	do
	{
		sp->dll = dll;
		sp->dlm = dlm;

		if (port->opened == false)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Port not opened.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}


		memset(data, 0, sizeof(data));
		h_ptr = (PNLK_HEADER)data;
		h_ptr->Version = NLK_PROTOCOL_VERSION;
		h_ptr->DeviceType = NLK_DEVICE_TYPE_TTY;
		h_ptr->MinorNumber = (unsigned char)port->line;
		h_ptr->CmdResponseEventData = NLK_TTY_CMD_SET_DLL_DLM;
		h_ptr->ResponseStatus = 0;
		h_ptr->Length = 2;
		length = sizeof(NLK_HEADER);
		// data
		data[length++] = sp->dll;
		data[length++] = sp->dlm;

		retval = snxdp_nlk_tx(data, length);
		if (retval < 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, NLK tx fail(retval:%d).\n", port->line, __FUNCTION__, retval);
			break;
		}

#if (DRIVERTARGET == 1)
		retval = wait_for_completion_timeout(&port->set_dll_dlm_comp, msecs_to_jiffies(1000));
		if (retval == 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Wait command complete timeout.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (port->rx_response_length < sizeof(NLK_HEADER))
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response length(%d) too small.\n", port->line, __FUNCTION__, port->rx_response_length);
			retval = -ENODEV;
			break;
		}

		h_ptr = (PNLK_HEADER)port->rx_response_buff;

		if (h_ptr->ResponseStatus != NLK_STATUS_SUCCESS)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response status failure.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (h_ptr->Length != 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response data length(%d) invalid.\n", port->line, __FUNCTION__, h_ptr->Length);
			retval = -ENODEV;
			break;
		}
#else
		mdelay(CMD_DELAY_MSEC);
#endif

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
ser_cmd_set_lcr(
	struct snxdp_ser_port *		sp,
	unsigned char				lcr
	)
{
	int							retval = 0;
	struct sdp_ser_port *		port = &sp->port;
	unsigned char				data[256];
	int							length = 0;
	PNLK_HEADER					h_ptr = NULL;
	

	do
	{
		sp->lcr = lcr;

		if (port->opened == false)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Port not opened.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}


		memset(data, 0, sizeof(data));
		h_ptr = (PNLK_HEADER)data;
		h_ptr->Version = NLK_PROTOCOL_VERSION;
		h_ptr->DeviceType = NLK_DEVICE_TYPE_TTY;
		h_ptr->MinorNumber = (unsigned char)port->line;
		h_ptr->CmdResponseEventData = NLK_TTY_CMD_SET_LCR;
		h_ptr->ResponseStatus = 0;
		h_ptr->Length = 1;
		length = sizeof(NLK_HEADER);
		// data
		data[length++] = sp->lcr;

		retval = snxdp_nlk_tx(data, length);
		if (retval < 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, NLK tx fail(retval:%d).\n", port->line, __FUNCTION__, retval);
			break;
		}

#if (DRIVERTARGET == 1)
		retval = wait_for_completion_timeout(&port->set_lcr_comp, msecs_to_jiffies(1000));
		if (retval == 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Wait command complete timeout.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}
		
		if (port->rx_response_length < sizeof(NLK_HEADER))
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response length(%d) too small.\n", port->line, __FUNCTION__, port->rx_response_length);
			retval = -ENODEV;
			break;
		}

		h_ptr = (PNLK_HEADER)port->rx_response_buff;

		if (h_ptr->ResponseStatus != NLK_STATUS_SUCCESS)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response status failure.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (h_ptr->Length != 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response data length(%d) invalid.\n", port->line, __FUNCTION__, h_ptr->Length);
			retval = -ENODEV;
			break;
		}
#else
		mdelay(CMD_DELAY_MSEC);
#endif

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
ser_cmd_set_fifo_ctrl(
	struct snxdp_ser_port *		sp,
	unsigned char				fifo_ctrl
	)
{
	int							retval = 0;
	struct sdp_ser_port *		port = &sp->port;
	unsigned char				data[256];
	int							length = 0;
	PNLK_HEADER					h_ptr = NULL;
	

	do
	{
		sp->fifo_ctrl = fifo_ctrl;

		if (port->opened == false)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Port not opened.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}


		memset(data, 0, sizeof(data));
		h_ptr = (PNLK_HEADER)data;
		h_ptr->Version = NLK_PROTOCOL_VERSION;
		h_ptr->DeviceType = NLK_DEVICE_TYPE_TTY;
		h_ptr->MinorNumber = (unsigned char)port->line;
		h_ptr->CmdResponseEventData = NLK_TTY_CMD_SET_FIFO_CTRL;
		h_ptr->ResponseStatus = 0;
		h_ptr->Length = 1;
		length = sizeof(NLK_HEADER);
		// data
		data[length++] = sp->fifo_ctrl;

		retval = snxdp_nlk_tx(data, length);
		if (retval < 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, NLK tx fail(retval:%d).\n", port->line, __FUNCTION__, retval);
			break;
		}

#if (DRIVERTARGET == 1)
		retval = wait_for_completion_timeout(&port->set_fifo_ctrl_comp, msecs_to_jiffies(1000));
		if (retval == 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Wait command complete timeout.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (port->rx_response_length < sizeof(NLK_HEADER))
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response length(%d) too small.\n", port->line, __FUNCTION__, port->rx_response_length);
			retval = -ENODEV;
			break;
		}

		h_ptr = (PNLK_HEADER)port->rx_response_buff;

		if (h_ptr->ResponseStatus != NLK_STATUS_SUCCESS)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response status failure.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (h_ptr->Length != 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response data length(%d) invalid.\n", port->line, __FUNCTION__, h_ptr->Length);
			retval = -ENODEV;
			break;
		}
#else
		mdelay(CMD_DELAY_MSEC);
#endif

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
ser_cmd_set_mcr(
	struct snxdp_ser_port *		sp,
	unsigned char				mcr
	)
{
	int							retval = 0;
	struct sdp_ser_port *		port = &sp->port;
	unsigned char				data[256];
	int							length = 0;
	PNLK_HEADER					h_ptr = NULL;
	

	do
	{
		sp->mcr = mcr;

		if (port->opened == false)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Port not opened.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}


		memset(data, 0, sizeof(data));
		h_ptr = (PNLK_HEADER)data;
		h_ptr->Version = NLK_PROTOCOL_VERSION;
		h_ptr->DeviceType = NLK_DEVICE_TYPE_TTY;
		h_ptr->MinorNumber = (unsigned char)port->line;
		h_ptr->CmdResponseEventData = NLK_TTY_CMD_SET_MCR;
		h_ptr->ResponseStatus = 0;
		h_ptr->Length = 1;
		length = sizeof(NLK_HEADER);
		// data
		data[length++] = sp->mcr;

		retval = snxdp_nlk_tx(data, length);
		if (retval < 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, NLK tx fail(retval:%d).\n", port->line, __FUNCTION__, retval);
			break;
		}

#if (DRIVERTARGET == 1)
		retval = wait_for_completion_timeout(&port->set_mcr_comp, msecs_to_jiffies(1000));
		if (retval == 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Wait command complete timeout.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (port->rx_response_length < sizeof(NLK_HEADER))
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response length(%d) too small.\n", port->line, __FUNCTION__, port->rx_response_length);
			retval = -ENODEV;
			break;
		}

		h_ptr = (PNLK_HEADER)port->rx_response_buff;

		if (h_ptr->ResponseStatus != NLK_STATUS_SUCCESS)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response status failure.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (h_ptr->Length != 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response data length(%d) invalid.\n", port->line, __FUNCTION__, h_ptr->Length);
			retval = -ENODEV;
			break;
		}
#else
		mdelay(CMD_DELAY_MSEC);
#endif

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
ser_cmd_set_event_ctrl(
	struct snxdp_ser_port *		sp,
	unsigned char				event_ctrl
	)
{
	int							retval = 0;
	struct sdp_ser_port *		port = &sp->port;
	unsigned char				data[256];
	int							length = 0;
	PNLK_HEADER					h_ptr = NULL;
	

	do
	{
		sp->event_ctrl = event_ctrl;

		if (port->opened == false)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Port not opened.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}


		memset(data, 0, sizeof(data));
		h_ptr = (PNLK_HEADER)data;
		h_ptr->Version = NLK_PROTOCOL_VERSION;
		h_ptr->DeviceType = NLK_DEVICE_TYPE_TTY;
		h_ptr->MinorNumber = (unsigned char)port->line;
		h_ptr->CmdResponseEventData = NLK_TTY_CMD_SET_EVENT_CTRL;
		h_ptr->ResponseStatus = 0;
		h_ptr->Length = 1;
		length = sizeof(NLK_HEADER);
		// data
		data[length++] = sp->event_ctrl;

		retval = snxdp_nlk_tx(data, length);
		if (retval < 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, NLK tx fail(retval:%d).\n", port->line, __FUNCTION__, retval);
			break;
		}

#if (DRIVERTARGET == 1)
		retval = wait_for_completion_timeout(&port->set_event_ctrl_comp, msecs_to_jiffies(1000));
		if (retval == 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Wait command complete timeout.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (port->rx_response_length < sizeof(NLK_HEADER))
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response length(%d) too small.\n", port->line, __FUNCTION__, port->rx_response_length);
			retval = -ENODEV;
			break;
		}

		h_ptr = (PNLK_HEADER)port->rx_response_buff;

		if (h_ptr->ResponseStatus != NLK_STATUS_SUCCESS)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response status failure.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (h_ptr->Length != 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response data length(%d) invalid.\n", port->line, __FUNCTION__, h_ptr->Length);
			retval = -ENODEV;
			break;
		}
#else
		mdelay(CMD_DELAY_MSEC);
#endif

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
ser_cmd_get_msr(
	struct snxdp_ser_port *		sp
	)
{
	int							retval = 0;
	struct sdp_ser_port *		port = &sp->port;
	unsigned char				data[256];
	int							length = 0;
	PNLK_HEADER					h_ptr = NULL;
	

	do
	{
		if (port->opened == false)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Port not opened.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}


		memset(data, 0, sizeof(data));
		h_ptr = (PNLK_HEADER)data;
		h_ptr->Version = NLK_PROTOCOL_VERSION;
		h_ptr->DeviceType = NLK_DEVICE_TYPE_TTY;
		h_ptr->MinorNumber = (unsigned char)port->line;
		h_ptr->CmdResponseEventData = NLK_TTY_CMD_GET_MSR;
		h_ptr->ResponseStatus = 0;
		h_ptr->Length = 0;
		length = sizeof(NLK_HEADER);

		retval = snxdp_nlk_tx(data, length);
		if (retval < 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, NLK tx fail(retval:%d).\n", port->line, __FUNCTION__, retval);
			break;
		}

		retval = wait_for_completion_timeout(&port->get_msr_comp, msecs_to_jiffies(1000));
		if (retval == 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Wait command complete timeout.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (port->rx_response_length < sizeof(NLK_HEADER))
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response length(%d) too small.\n", port->line, __FUNCTION__, port->rx_response_length);
			retval = -ENODEV;
			break;
		}

		h_ptr = (PNLK_HEADER)port->rx_response_buff;

		if (h_ptr->ResponseStatus != NLK_STATUS_SUCCESS)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response status failure.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (h_ptr->Length != 1)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response data length(%d) invalid.\n", port->line, __FUNCTION__, h_ptr->Length);
			retval = -ENODEV;
			break;
		}

		sp->msr = *(port->rx_response_buff + sizeof(NLK_HEADER) + 0);

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
ser_cmd_set_on_chip_hw_flow_ctrl(
	struct snxdp_ser_port *		sp,
	unsigned char				on_chip_hw_flow_ctrl
	)
{
	int							retval = 0;
	struct sdp_ser_port *		port = &sp->port;
	unsigned char				data[256];
	int							length = 0;
	PNLK_HEADER					h_ptr = NULL;
	

	do
	{
		sp->on_chip_hw_flow_ctrl = on_chip_hw_flow_ctrl;

		if (port->opened == false)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Port not opened.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}


		memset(data, 0, sizeof(data));
		h_ptr = (PNLK_HEADER)data;
		h_ptr->Version = NLK_PROTOCOL_VERSION;
		h_ptr->DeviceType = NLK_DEVICE_TYPE_TTY;
		h_ptr->MinorNumber = (unsigned char)port->line;
		h_ptr->CmdResponseEventData = NLK_TTY_CMD_SET_ON_CHIP_HARDWARE_FLOW_CTRL;
		h_ptr->ResponseStatus = 0;
		h_ptr->Length = 1;
		length = sizeof(NLK_HEADER);
		// data
		data[length++] = sp->on_chip_hw_flow_ctrl;

		retval = snxdp_nlk_tx(data, length);
		if (retval < 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, NLK tx fail(retval:%d).\n", port->line, __FUNCTION__, retval);
			break;
		}

#if (DRIVERTARGET == 1)
		retval = wait_for_completion_timeout(&port->set_on_chip_hw_flow_ctrl_comp, msecs_to_jiffies(1000));
		if (retval == 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Wait command complete timeout.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (port->rx_response_length < sizeof(NLK_HEADER))
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response length(%d) too small.\n", port->line, __FUNCTION__, port->rx_response_length);
			retval = -ENODEV;
			break;
		}

		h_ptr = (PNLK_HEADER)port->rx_response_buff;

		if (h_ptr->ResponseStatus != NLK_STATUS_SUCCESS)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response status failure.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (h_ptr->Length != 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response data length(%d) invalid.\n", port->line, __FUNCTION__, h_ptr->Length);
			retval = -ENODEV;
			break;
		}
#else
		mdelay(CMD_DELAY_MSEC);
#endif

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
ser_cmd_set_tx_trigger_level(
	struct snxdp_ser_port *		sp,
	unsigned char				tx_trigger_level
	)
{
	int							retval = 0;
	struct sdp_ser_port *		port = &sp->port;
	unsigned char				data[256];
	int							length = 0;
	PNLK_HEADER					h_ptr = NULL;
	

	do
	{
		sp->tx_trigger_level = tx_trigger_level;

		if (port->opened == false)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Port not opened.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}


		memset(data, 0, sizeof(data));
		h_ptr = (PNLK_HEADER)data;
		h_ptr->Version = NLK_PROTOCOL_VERSION;
		h_ptr->DeviceType = NLK_DEVICE_TYPE_TTY;
		h_ptr->MinorNumber = (unsigned char)port->line;
		h_ptr->CmdResponseEventData = NLK_TTY_CMD_SET_TX_TRIGGER_LEVEL;
		h_ptr->ResponseStatus = 0;
		h_ptr->Length = 1;
		length = sizeof(NLK_HEADER);
		// data
		data[length++] = sp->tx_trigger_level;

		retval = snxdp_nlk_tx(data, length);
		if (retval < 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, NLK tx fail(retval:%d).\n", port->line, __FUNCTION__, retval);
			break;
		}

#if (DRIVERTARGET == 1)
		retval = wait_for_completion_timeout(&port->set_tx_trigger_level_comp, msecs_to_jiffies(1000));
		if (retval == 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Wait command complete timeout.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (port->rx_response_length < sizeof(NLK_HEADER))
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response length(%d) too small.\n", port->line, __FUNCTION__, port->rx_response_length);
			retval = -ENODEV;
			break;
		}

		h_ptr = (PNLK_HEADER)port->rx_response_buff;

		if (h_ptr->ResponseStatus != NLK_STATUS_SUCCESS)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response status failure.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (h_ptr->Length != 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response data length(%d) invalid.\n", port->line, __FUNCTION__, h_ptr->Length);
			retval = -ENODEV;
			break;
		}
#else
		mdelay(CMD_DELAY_MSEC);
#endif

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
ser_cmd_set_rx_trigger_level(
	struct snxdp_ser_port *		sp,
	unsigned char				rx_trigger_level
	)
{
	int							retval = 0;
	struct sdp_ser_port *		port = &sp->port;
	unsigned char				data[256];
	int							length = 0;
	PNLK_HEADER					h_ptr = NULL;
	

	do
	{
		sp->rx_trigger_level = rx_trigger_level;

		if (port->opened == false)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Port not opened.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}


		memset(data, 0, sizeof(data));
		h_ptr = (PNLK_HEADER)data;
		h_ptr->Version = NLK_PROTOCOL_VERSION;
		h_ptr->DeviceType = NLK_DEVICE_TYPE_TTY;
		h_ptr->MinorNumber = (unsigned char)port->line;
		h_ptr->CmdResponseEventData = NLK_TTY_CMD_SET_RX_TRIGGER_LEVEL;
		h_ptr->ResponseStatus = 0;
		h_ptr->Length = 1;
		length = sizeof(NLK_HEADER);
		// data
		data[length++] = sp->rx_trigger_level;

		retval = snxdp_nlk_tx(data, length);
		if (retval < 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, NLK tx fail(retval:%d).\n", port->line, __FUNCTION__, retval);
			break;
		}

#if (DRIVERTARGET == 1)
		retval = wait_for_completion_timeout(&port->set_rx_trigger_level_comp, msecs_to_jiffies(1000));
		if (retval == 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Wait command complete timeout.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		mdelay(100);

		if (port->rx_response_length < sizeof(NLK_HEADER))
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response length(%d) too small.\n", port->line, __FUNCTION__, port->rx_response_length);
			retval = -ENODEV;
			break;
		}

		h_ptr = (PNLK_HEADER)port->rx_response_buff;

		if (h_ptr->ResponseStatus != NLK_STATUS_SUCCESS)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response status failure.\n", port->line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (h_ptr->Length != 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response data length(%d) invalid.\n", port->line, __FUNCTION__, h_ptr->Length);
			retval = -ENODEV;
			break;
		}
#else
		mdelay(CMD_DELAY_MSEC);
#endif

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


