/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#ifndef _SDDVIRTUAL_DRVR_SXXC_H_
#define _SDDVIRTUAL_DRVR_SXXC_H_


/**********************************************************************/


#define SDP_SER_TOTAL_MAX					NLK_TTY_MAX


/***********************************************************************
					miscellaneous define
***********************************************************************/


#define UART_DEFAULT_FCR					0x00

#define DEFAULT_FIFOSIZE					1
#define DEFAULT_TRIGGER_LEVEL				1
#define SUNIXDP_FIFOSIZE  					200
#define SUNIXDP_TRIGGER_LEVEL	  			32

#define UART_LSR_ERR_IN_RFIFO				0x80
#define UART_MCR_AFE						0x20
#define UART_IIR_CTO						0x0C

#define INTERRUPT_COUNT						128
#define WAKEUP_CHARS						256

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,6,19))
#define SDPTERMIOS							ktermios
#else
#define SDPTERMIOS							termios
#endif

// for sdp_ser_port->setserial_flag
#define SDP_SER_BAUD_SETSERIAL				1
#define SDP_SER_BAUD_NOTSETSER				0



/***********************************************************************
					serial define
***********************************************************************/


#define PORT_SER_UNKNOWN					0
#define PORT_SER_8250						1
#define PORT_SER_16450						2
#define PORT_SER_16550						3
#define PORT_SER_16550A						4
#define PORT_SER_CIRRUS						5
#define PORT_SER_16650						6
#define PORT_SER_16650V2					7
#define PORT_SER_16750						8
#define PORT_SER_MAX_UART					8


#define SDP_USF_CLOSING_WAIT_INF			(0)
#define SDP_USF_CLOSING_WAIT_NONE			(65535)
#define SDP_UART_CONFIG_TYPE				(1 << 0)
#define SDP_UART_CONFIG_IRQ					(1 << 1)

#define SDP_UART_XMIT_SIZE					4096

#define sdp_ser_circ_empty(circ)			((circ)->head == (circ)->tail)
#define sdp_ser_circ_clear(circ)			((circ)->head = (circ)->tail = 0)
#define sdp_ser_circ_chars_pending(circ)	(CIRC_CNT((circ)->head, (circ)->tail, SDP_UART_XMIT_SIZE))
#define sdp_ser_circ_chars_free(circ)		(CIRC_SPACE((circ)->head, (circ)->tail, SDP_UART_XMIT_SIZE))
#define sdp_ser_tx_stopped(port)			((port)->info->tty->stopped || (port)->info->tty->hw_stopped)


#if defined(__i386__) && (defined(CONFIG_M386) || defined(CONFIG_M486))
#define SDP_SERIAL_INLINE
#endif

#ifdef SDP_SERIAL_INLINE
#define _INLINE_ inline
#else
#define _INLINE_
#endif


#define SDP_UPIO_PORT						(0)
#define SDP_UPIO_MEM						(1)

#define SDP_UPF_SAK							(1 << 2)
#define SDP_UPF_SPD_MASK					(0x1030)
#define SDP_UPF_SPD_HI						(0x0010)
#define SDP_UPF_SPD_VHI						(0x0020)
#define SDP_UPF_SPD_CUST					(0x0030)
#define SDP_UPF_SPD_SHI						(0x1000)
#define SDP_UPF_SPD_WARP					(0x1010)
#define SDP_UPF_SKIP_TEST					(1 << 6)
#define SDP_UPF_HARDPPS_CD					(1 << 11)
#define SDP_UPF_LOW_LATENCY					(1 << 13)
#define SDP_UPF_BUGGY_UART					(1 << 14)
#define SDP_UPF_MAGIC_MULTIPLIER			(1 << 16)
#define SDP_UPF_CHANGE_MASK					(0x17fff)
#define SDP_UPF_USR_MASK					(SDP_UPF_SPD_MASK | SDP_UPF_LOW_LATENCY)
#define SDP_UIF_CHECK_CD					(1 << 25)
#define SDP_UIF_CTS_FLOW					(1 << 26)
#define SDP_UIF_NORMAL_ACTIVE				(1 << 29)
#define SDP_UIF_INITIALIZED					(1 << 31)
#define SDP_ENABLE_MS(port, cflag)			((port)->flags & SDP_UPF_HARDPPS_CD || (cflag) & CRTSCTS || !((cflag) & CLOCAL))


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
#define SDP_SER_DEVNUM(x)					((x)->index)
#else
#define SDP_SER_DEVNUM(x)					(MINOR((x)->device) - (x)->driver.minor_start)
#endif


struct sdp_ser_info;
struct sdp_ser_port;


struct sdp_ser_icount
{
	__u32						cts;
	__u32						dsr;
	__u32						rng;
	__u32						dcd;
	__u32						rx;
	__u32						tx;
	__u32						frame;
	__u32						overrun;
	__u32						parity;
	__u32						brk;
	__u32						buf_overrun;
};


struct sdp_ser_info
{
	struct tty_struct *			tty;
	struct circ_buf				xmit;
	unsigned int				flags;
	unsigned char *				tmpbuf;
	struct semaphore			tmpbuf_sem;
	int							blocked_open;
	struct tasklet_struct		tlet;

	wait_queue_head_t			open_wait;
	wait_queue_head_t			delta_msr_wait;
};


struct sdp_ser_driver
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))	
	struct module *				owner;
	const char *				driver_name;
#endif
	const char *				dev_name;
	int							major;
	int							minor;
	int							nr;
	struct sdp_ser_state *		state;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
	struct tty_driver *			tty_driver;
#else
	struct tty_driver			tty_driver;
#endif
};


struct sdp_ser_port
{
	spinlock_t					lock;
	//--- DP
	/*
	unsigned int				iobase;
	unsigned int				irq;
	*/
	//---
	unsigned int				uartclk;
	unsigned char				fifosize;
	unsigned char				x_char;
	unsigned char				iotype;

	unsigned int				read_status_mask;
	unsigned int				ignore_status_mask;
	struct sdp_ser_info	*		info;
	struct sdp_ser_icount		icount;

	unsigned int				flags;
	unsigned int				mctrl;
	unsigned int				timeout;
	unsigned int				type;
	unsigned int				custom_divisor;
	unsigned int				line;

	unsigned int				baud_base;
	int							rx_trigger;
	unsigned char				ldisc_stop_rx;

	unsigned int				setserial_flag;

	unsigned char				suspended;
	struct device *				dev;

	//--- DP
	bool						opened;
	unsigned char *				rx_response_buff;
	unsigned int				rx_response_length;
	unsigned char *				rx_event_buff;
	unsigned int				rx_event_length;
	unsigned int				rx_event_type;
	unsigned char *				rx_data_buff;
	unsigned int				rx_data_length;

	unsigned char *				tx_data_buff;
	unsigned int				tx_data_length;
	unsigned char *				tx_data_msg;

	struct mutex	 			xmit_buff_mutex;
	unsigned int				wpt;
	unsigned int				rpt;

	struct completion			open_comp;
	struct completion			close_comp;
#if (DRIVERTARGET == 1)
	struct completion			set_dll_dlm_comp;
	struct completion			set_lcr_comp;
	struct completion			set_fifo_ctrl_comp;
	struct completion			set_mcr_comp;
	struct completion			set_event_ctrl_comp;
	struct completion			set_on_chip_hw_flow_ctrl_comp;
	struct completion			set_tx_trigger_level_comp;
	struct completion			set_rx_trigger_level_comp;
#endif
	struct completion			get_msr_comp;
	//---
};


struct sdp_ser_state
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))	
	struct tty_port				tport;	
#endif		
	unsigned int				close_delay;
	unsigned int				closing_wait;
	int							count;
	struct sdp_ser_info	*		info;
	struct sdp_ser_port	*		port;
	struct semaphore			sem;
};


static inline int sdp_ser_handle_break(struct sdp_ser_port *port)
{
	struct sdp_ser_info *info = port->info;

	if (info->flags & SDP_UPF_SAK)
	{
		do_SAK(info->tty);
	}
	return 0;
}


static inline void sdp_ser_handle_dcd_change(struct sdp_ser_port *port, unsigned int status)
{
	struct sdp_ser_info *info = port->info;

	port->icount.dcd++;

	if (info->flags & SDP_UIF_CHECK_CD)
	{
		if (status)
		{
			wake_up_interruptible(&info->open_wait);
		}
		else if (info->tty)
		{
			tty_hangup(info->tty);
		}
	}
}


#include <linux/tty_flip.h>

static inline void sdp_ser_insert_char(
	struct sdp_ser_port *port,
	unsigned int status,
	unsigned int overrun,
	unsigned int ch,
	unsigned int flag
	)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0))
	struct sdp_ser_info *info = port->info;
	struct tty_struct *tty = info->tty;
	struct sdp_ser_state *state = NULL;
	struct tty_port *tport = NULL;
	state = tty->driver_data;
	tport = &state->tport;
#else
	struct tty_struct *tty = port->info->tty;
#endif


	if ((status & port->ignore_status_mask & ~overrun) == 0)
	{
#if	(LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0))
		if (tty_insert_flip_char(tport, ch, flag) == 0)
			++port->icount.buf_overrun;	
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(3,6,0))	
		if (tty_insert_flip_char(tty, ch, flag) == 0)
			++port->icount.buf_overrun;
#else
		tty_insert_flip_char(tty, ch, flag);
#endif
	}


	if (status & ~port->ignore_status_mask & overrun)
	{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0))
		if (tty_insert_flip_char(tport, 0, TTY_OVERRUN) == 0)
			++port->icount.buf_overrun;	
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(3,6,0))
		if (tty_insert_flip_char(tty, 0, TTY_OVERRUN) == 0)
			++port->icount.buf_overrun;
#else
		tty_insert_flip_char(tty, 0, TTY_OVERRUN);
#endif
    }
}


/***********************************************************************
					SUNIX serial port struct
***********************************************************************/


#define SDP_SER_TIMEOUT						HZ/2
#define SDP_SER_TX_IDLE_CNT_MAX				4

struct snxdp_ser_port
{
	struct sdp_ser_port			port;
	struct list_head			list;
	unsigned int				capabilities;

	//--- DP
	//struct timer_list			timer;
	unsigned char				uart_version;
	unsigned char				dll;
	unsigned char				dlm;
	unsigned char				lcr;
	unsigned char				fifo_ctrl;
	unsigned char				mcr;
	unsigned char				mcr_mask;
	unsigned char				mcr_force;
	unsigned char				event_ctrl;
	unsigned char				on_chip_hw_flow_ctrl;
	unsigned char				msr;
	unsigned char				msr_for_timer;
	unsigned short				tx_current_credit;
	unsigned short				tx_fifo_data_cnt;
	unsigned char				tx_trigger_level;
	unsigned char				rx_trigger_level;
	unsigned char				mode_1;
	unsigned char				mode_2;
	//---
};


extern struct snxdp_ser_port	    snxdp_ser_table[SDP_SER_TOTAL_MAX];


/**********************************************************************/


#endif


