/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,37)) 
static DEFINE_SEMAPHORE(ser_port_sem);
#else
static DECLARE_MUTEX(ser_port_sem);
#endif


#define SDP_HIGH_BITS_OFFSET		((sizeof(long)-sizeof(int))*8)
#define snxdp_ser_users(state)		((state)->count + ((state)->info ? (state)->info->blocked_open : 0))


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
static struct tty_port				sdp_service_port;
#endif


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
struct serial_uart_config
{
	char						*name;
	int							dfl_xmit_fifo_size;
	int							flags;
};
#endif


static const struct serial_uart_config sdp_uart_config[PORT_SER_MAX_UART + 1] = 
{
	{ "unknown",	1,		0 },
	{ "8250",       1,		0 },
	{ "16450",      1,		0 },
	{ "16550",      1,		0 },
	{ "16550A",     16,		UART_CLEAR_FIFO | UART_USE_FIFO },
	{ "Cirrus",     1,		0 },
	{ "ST16650",    1,		0 },
	{ "ST16650V2",  32,		UART_CLEAR_FIFO | UART_USE_FIFO },
	{ "TI16750",    64,		UART_CLEAR_FIFO | UART_USE_FIFO },
};


#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)) 
static int							snxdp_ser_refcount;
static struct tty_struct *			snxdp_ser_tty[SDP_SER_TOTAL_MAX + 1];
static struct termios *				snxdp_ser_termios[SDP_SER_TOTAL_MAX + 1];
static struct termios *				snxdp_ser_termios_locked[SDP_SER_TOTAL_MAX + 1];
#endif


static _INLINE_ void sdp_ser_handle_cts_change(struct sdp_ser_port *, unsigned int);
static _INLINE_ void sdp_ser_update_mctrl(struct sdp_ser_port *, unsigned int, unsigned int);
static void sdp_ser_write_wakeup(struct sdp_ser_port *);
static void sdp_ser_stop(struct tty_struct *);
static void __sdp_ser_start(struct tty_struct *);
static void sdp_ser_start(struct tty_struct *);
static void sdp_ser_tasklet_action(unsigned long);
static int sdp_ser_startup(struct sdp_ser_state *, int);
static void sdp_ser_shutdown(struct sdp_ser_state *);
static _INLINE_ void __sdp_ser_put_char(struct sdp_ser_port *, struct circ_buf *, unsigned char);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26))
static int sdp_ser_put_char(struct tty_struct *, unsigned char);
#else
static void sdp_ser_put_char(struct tty_struct *, unsigned char);
#endif
static void sdp_ser_flush_chars(struct tty_struct *);
static int sdp_ser_chars_in_buffer(struct tty_struct *);
static void sdp_ser_flush_buffer(struct tty_struct *);
static void sdp_ser_send_xchar(struct tty_struct *, char);
static void sdp_ser_throttle(struct tty_struct *);
static void sdp_ser_unthrottle(struct tty_struct *);
static int sdp_ser_get_info(struct sdp_ser_state *, struct serial_struct *);
static int sdp_ser_set_info(struct sdp_ser_state *, struct serial_struct *);
static int sdp_ser_write_room(struct tty_struct *);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,10))
static int sdp_ser_write(struct tty_struct *, const unsigned char *, int);
#else
static int sdp_ser_write(struct tty_struct *, int, const unsigned char *, int);
#endif
static int sdp_ser_get_lsr_info(struct sdp_ser_state *, unsigned int *);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,39))
static int sdp_ser_tiocmget(struct tty_struct *);
static int sdp_ser_tiocmset(struct tty_struct *, unsigned int, unsigned int);
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
static int sdp_ser_tiocmget(struct tty_struct *, struct file *);
static int sdp_ser_tiocmset(struct tty_struct *, struct file *, unsigned int, unsigned int);
#else
static int sdp_ser_get_modem_info(struct sdp_ser_state *, unsigned int *);
static int sdp_ser_set_modem_info(struct sdp_ser_state *, unsigned int, unsigned int *);
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27))
static int sdp_ser_break_ctl(struct tty_struct *, int);
#else
static void sdp_ser_break_ctl(struct tty_struct *, int);
#endif
static int sdp_ser_wait_modem_status(struct sdp_ser_state *, unsigned long);
static int sdp_ser_get_count(struct sdp_ser_state *, struct serial_icounter_struct *);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,39))
static int sdp_ser_ioctl(struct tty_struct *, unsigned int, unsigned long);
#else
static int sdp_ser_ioctl(struct tty_struct *, struct file *, unsigned int, unsigned long);
#endif
static void sdp_ser_hangup(struct tty_struct *);
static unsigned int sdp_ser_get_divisor(struct sdp_ser_port *, unsigned int);
static unsigned int sdp_ser_get_baud_rate(struct sdp_ser_port *, struct SDPTERMIOS *, struct SDPTERMIOS *, unsigned int, unsigned int);
static void sdp_ser_change_speed(struct sdp_ser_state *, struct SDPTERMIOS *);
static void sdp_ser_set_termios(struct tty_struct *, struct SDPTERMIOS *);
static void sdp_ser_update_termios(struct sdp_ser_state *);
static void sdp_ser_update_timeout(struct sdp_ser_port *, unsigned int, unsigned int);
static struct sdp_ser_state *sdp_ser_get(struct sdp_ser_driver *, int);
static int sdp_ser_block_til_ready(struct file *, struct sdp_ser_state *);
static void sdp_ser_wait_until_sent(struct tty_struct *, int);
static int sdp_ser_open(struct tty_struct *, struct file *);
static void sdp_ser_close(struct tty_struct *, struct file *);

static void snxdp_ser_set_mctrl(struct sdp_ser_port *, unsigned int);
static unsigned int snxdp_ser_tx_empty(struct sdp_ser_port *);
static unsigned int snxdp_ser_get_mctrl(struct sdp_ser_port *);
static void snxdp_ser_stop_tx(struct sdp_ser_port *, unsigned int);
static void snxdp_ser_start_tx(struct sdp_ser_port *, unsigned int);
static void snxdp_ser_stop_rx(struct sdp_ser_port *);
static void snxdp_ser_enable_ms(struct sdp_ser_port *);
static void snxdp_ser_break_ctl(struct sdp_ser_port *, int);
static int snxdp_ser_startup(struct sdp_ser_port *);
static void snxdp_ser_shutdown(struct sdp_ser_port *);
static unsigned int snxdp_ser_get_divisor(struct sdp_ser_port *, unsigned int);
static void snxdp_ser_set_termios(struct sdp_ser_port *, struct SDPTERMIOS *, struct SDPTERMIOS *);
//--- no timer for DP TTY
//static void snxdp_ser_timeout(unsigned long);
//---
static _INLINE_ void snxdp_ser_receive_chars_for_data_pipe_normal(struct snxdp_ser_port *);
static _INLINE_ void snxdp_ser_receive_chars_for_data_pipe_error(struct snxdp_ser_port *);
static _INLINE_ void snxdp_ser_receive_chars_for_event(struct snxdp_ser_port *);
static _INLINE_ void snxdp_ser_transmit_chars(struct snxdp_ser_port *);
static _INLINE_ void snxdp_ser_check_modem_status(struct snxdp_ser_port *, unsigned char);

static void snxdp_ser_release_io(struct sdp_ser_port *);
static void snxdp_ser_request_io(struct sdp_ser_port *);
static void snxdp_ser_configure_port(struct sdp_ser_driver *, struct sdp_ser_state *, struct sdp_ser_port *);
static void snxdp_ser_unconfigure_port(struct sdp_ser_driver *, struct sdp_ser_state *);
static int snxdp_ser_add_one_port(struct sdp_ser_driver *, struct sdp_ser_port *);
static int snxdp_ser_remove_one_port(struct sdp_ser_driver *, struct sdp_ser_port *);


/*-----------------------------------------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------------------------------------*/


static _INLINE_ void 
sdp_ser_handle_cts_change(
	struct sdp_ser_port *		port,
	unsigned int 				status
	)
{
	struct sdp_ser_info	*		info = port->info;
	struct tty_struct *			tty = info->tty;


	port->icount.cts++;

	if (info->flags & SDP_UIF_CTS_FLOW)
	{
		if (tty->hw_stopped)
		{
			if (status)
			{
				tty->hw_stopped = 0;
				snxdp_ser_start_tx(port, 0);
				sdp_ser_write_wakeup(port);
			}
		}
		else
		{
			if (!status)
			{
				tty->hw_stopped = 1;
				snxdp_ser_stop_tx(port, 0);
			}
		}
	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static _INLINE_ void 
sdp_ser_update_mctrl(
	struct sdp_ser_port *		port,
	unsigned int 				set,
	unsigned int 				clear
	)
{
	unsigned int 				old;


	old = port->mctrl;
	port->mctrl = (old & ~clear) | set;

	if (old != port->mctrl)
	{
		snxdp_ser_set_mctrl(port, port->mctrl);
	}
}


#define sdp_set_mctrl(port, set)			sdp_ser_update_mctrl(port, set, 0)
#define sdp_clear_mctrl(port, clear)		sdp_ser_update_mctrl(port, 0, clear)


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_write_wakeup(
	struct sdp_ser_port *		port
	)
{	
	struct sdp_ser_info	*		info = port->info;


	tasklet_schedule(&info->tlet);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_stop(
	struct tty_struct *			tty
	)
{
	struct sdp_ser_state *		state = NULL;
	struct sdp_ser_port	*		port = NULL;
	unsigned long				flags;
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			break;
		}

		state = tty->driver_data;
		port = state->port;

		spin_lock_irqsave(&port->lock, flags);
		snxdp_ser_stop_tx(port, 1);
		spin_unlock_irqrestore(&port->lock, flags);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
__sdp_ser_start(
	struct tty_struct *			tty
	)
{
	struct sdp_ser_state *		state = tty->driver_data;
	struct sdp_ser_port	*		port = state->port;


	if (!sdp_ser_circ_empty(&state->info->xmit) && state->info->xmit.buf && !tty->stopped && !tty->hw_stopped)
	{
		snxdp_ser_start_tx(port, 1);
	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_start(
	struct tty_struct *			tty
	)
{
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			break;
		}

		__sdp_ser_start(tty);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_tasklet_action(
	unsigned long 				data
	)
{
	struct sdp_ser_state *		state = (struct sdp_ser_state *)data;
	struct tty_struct *			tty = NULL;


	tty = state->info->tty;
	if (tty)
	{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31))
		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) && tty->ldisc->ops->write_wakeup)
		{
			tty->ldisc->ops->write_wakeup(tty);
		}
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27) && LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,30))
        if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) && tty->ldisc.ops->write_wakeup)
		{
			tty->ldisc.ops->write_wakeup(tty);
		}
#else
		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) && tty->ldisc.write_wakeup)
		{
			tty->ldisc.write_wakeup(tty);
		}
#endif
		wake_up_interruptible(&tty->write_wait);
	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
sdp_ser_startup(
	struct sdp_ser_state *		state,
	int 						init_hw
	)
{
	struct sdp_ser_info	*		info = state->info;
	struct sdp_ser_port	*		port = state->port;
	unsigned long				page;
	int							retval = 0;


	do
	{
		if (info->flags & SDP_UIF_INITIALIZED)
		{
			retval = 0;
			break;
		}


		if (info->tty)
		{
			set_bit(TTY_IO_ERROR, &info->tty->flags);
		}


		if (port->type == PORT_UNKNOWN)
		{
			retval = 0;
			break;
		}


		if (!info->xmit.buf) 
		{
			page = get_zeroed_page(GFP_KERNEL);
			if (!page)
			{
				retval = -ENOMEM;
				break;
			}

			info->xmit.buf = (unsigned char *) page;
			info->tmpbuf = info->xmit.buf + SDP_UART_XMIT_SIZE;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,37))
			sema_init(&info->tmpbuf_sem,1);
#else
			init_MUTEX(&info->tmpbuf_sem);
#endif
			sdp_ser_circ_clear(&info->xmit);
		}

		retval = snxdp_ser_startup(port);
		if (retval == 0)
		{
			if (init_hw)
			{
				sdp_ser_change_speed(state, NULL);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
				if (info->tty->termios.c_cflag & CBAUD)
				{
					sdp_set_mctrl(port, TIOCM_RTS | TIOCM_DTR);
				}
#else
				if (info->tty->termios->c_cflag & CBAUD)
				{
					sdp_set_mctrl(port, TIOCM_RTS | TIOCM_DTR);
				}
#endif      
			}


			info->flags |= SDP_UIF_INITIALIZED;

			clear_bit(TTY_IO_ERROR, &info->tty->flags);
		}


		if (retval && capable(CAP_SYS_ADMIN))
		{
			retval = 0;
		}

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_shutdown(
	struct sdp_ser_state *		state
	)
{
	struct sdp_ser_info	*		info = state->info;
	struct sdp_ser_port	*		port = state->port;


	do
	{
		if (!(info->flags & SDP_UIF_INITIALIZED))
		{
			break;
		}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		if (!info->tty || (info->tty->termios.c_cflag & HUPCL))
		{
			sdp_clear_mctrl(port, TIOCM_DTR | TIOCM_RTS);
		}
#else
		if (!info->tty || (info->tty->termios->c_cflag & HUPCL))
		{
			sdp_clear_mctrl(port, TIOCM_DTR | TIOCM_RTS);
		}
#endif

		wake_up_interruptible(&info->delta_msr_wait);


		snxdp_ser_shutdown(port);

		//--- no irq for DP TTY
		/*
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
		synchronize_irq(port->irq);
#endif
		*/
		//---

		if (info->xmit.buf)
		{
			free_page((unsigned long)info->xmit.buf);
			info->xmit.buf = NULL;
			info->tmpbuf = NULL;
		}

		tasklet_kill(&info->tlet);

		if (info->tty)
		{
			set_bit(TTY_IO_ERROR, &info->tty->flags);
		}

		info->flags &= ~SDP_UIF_INITIALIZED;

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static _INLINE_ void 
__sdp_ser_put_char(
	struct sdp_ser_port *		port,
	struct circ_buf *			circ,
	unsigned char 				c
	)
{
	unsigned long				flags;


	do
	{
		if (!circ->buf)
		{
			break;
		}

		spin_lock_irqsave(&port->lock, flags);

		if (sdp_ser_circ_chars_free(circ) != 0)
		{
			circ->buf[circ->head] = c;
			circ->head = (circ->head + 1) & (SDP_UART_XMIT_SIZE - 1);
		}
		spin_unlock_irqrestore(&port->lock, flags);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26))
static int 
sdp_ser_put_char(
	struct tty_struct *			tty,
	unsigned char 				ch
	)
#else
static void 
sdp_ser_put_char(
	struct tty_struct *			tty,
	unsigned char 				ch
	)
#endif
{
	struct sdp_ser_state *		state = NULL;
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			break;
		}

		state = tty->driver_data;
		__sdp_ser_put_char(state->port, &state->info->xmit, ch);

	} while (false);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26))
	return 0;
#else

#endif
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_flush_chars(
	struct tty_struct *			tty
	)
{
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			break;
		}

		sdp_ser_start(tty);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
sdp_ser_chars_in_buffer(
	struct tty_struct *			tty
	)
{
	struct sdp_ser_state *		state = NULL;
	int							line = SDP_SER_DEVNUM(tty);
	int							retval = 0;


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			retval = 0;
			break;
		}

		state = tty->driver_data;

		retval = sdp_ser_circ_chars_pending(&state->info->xmit);

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_flush_buffer(
	struct tty_struct *			tty
	)
{
	struct sdp_ser_state *		state = NULL;
	struct sdp_ser_port	*		port = NULL;
	unsigned long				flags;
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			break;
		}

		state = tty->driver_data;
		port = state->port;

		if (!state || !state->info)
		{
			break;
		}

		spin_lock_irqsave(&port->lock, flags);
		sdp_ser_circ_clear(&state->info->xmit);
		spin_unlock_irqrestore(&port->lock, flags);

		wake_up_interruptible(&tty->write_wait);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31))
		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) && tty->ldisc->ops->write_wakeup)
		{
			(tty->ldisc->ops->write_wakeup)(tty);
		}
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27) && LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,30))
		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) && tty->ldisc.ops->write_wakeup)
		{
			(tty->ldisc.ops->write_wakeup)(tty);
		}
#else
		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) && tty->ldisc.write_wakeup)
		{
			(tty->ldisc.write_wakeup)(tty);
		}
#endif
	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_send_xchar(
	struct tty_struct *			tty,
	char 						ch
	)
{
	struct sdp_ser_state *		state = NULL;
	struct sdp_ser_port	*		port = NULL;
	unsigned long				flags;
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			break;
		}

		state = tty->driver_data;
		port = state->port;
		port->x_char = ch;

		if (ch)
		{
			spin_lock_irqsave(&port->lock, flags);
			snxdp_ser_start_tx(port, 0);
			spin_unlock_irqrestore(&port->lock, flags);
		}

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_throttle(
	struct tty_struct *			tty
	)
{
	struct sdp_ser_state *		state = NULL;
	struct sdp_ser_port	*		port = NULL;
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			break;
		}

		state = tty->driver_data;
		port = state->port;

		port->ldisc_stop_rx = 1;

		if (I_IXOFF(tty))
		{
			sdp_ser_send_xchar(tty, STOP_CHAR(tty));
		}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		if (tty->termios.c_cflag & CRTSCTS)
		{
			sdp_clear_mctrl(state->port, TIOCM_RTS);
		}
#else
		if (tty->termios->c_cflag & CRTSCTS)
		{
			sdp_clear_mctrl(state->port, TIOCM_RTS);
		}
#endif
	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_unthrottle(
	struct tty_struct *			tty
	)
{
	struct sdp_ser_state *		state = NULL;
	struct sdp_ser_port *		port = NULL;
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			break;
		}

		state = tty->driver_data;
		port = state->port;

		port->ldisc_stop_rx = 0;

		if (I_IXOFF(tty))
		{
			if (port->x_char)
			{
				port->x_char = 0;
			}
			else
			{
				sdp_ser_send_xchar(tty, START_CHAR(tty));
			}
		}


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		if (tty->termios.c_cflag & CRTSCTS)
		{
			sdp_set_mctrl(port, TIOCM_RTS);
		}
#else
		if (tty->termios->c_cflag & CRTSCTS)
		{
			sdp_set_mctrl(port, TIOCM_RTS);
		}
#endif
	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
sdp_ser_get_info(
	struct sdp_ser_state *		state,
	struct serial_struct *		retinfo
	)
{
	struct sdp_ser_port	*		port = state->port;
	struct serial_struct		tmp;
	int							retval = 0;


	memset(&tmp, 0, sizeof(tmp));
	tmp.type			= port->type;
	tmp.line 			= port->line;
	//--- no iobase and irq for DP TTY
	/*
	tmp.port			= port->iobase;
	if (SDP_HIGH_BITS_OFFSET)
	{
		tmp.port_high = (long) port->iobase >> SDP_HIGH_BITS_OFFSET;
	}
	tmp.irq				= port->irq;
	*/
	tmp.port			= 0;
	if (SDP_HIGH_BITS_OFFSET)
	{
		tmp.port_high = 0;
	}
	tmp.irq				= 0;
	//---
	tmp.flags			= port->flags;
	tmp.xmit_fifo_size	= port->fifosize;
	tmp.baud_base		= port->uartclk / 16;
	tmp.close_delay		= state->close_delay;
	tmp.closing_wait	= state->closing_wait;

	tmp.custom_divisor	= port->custom_divisor;
	tmp.io_type			= port->iotype;

	if (copy_to_user(retinfo, &tmp, sizeof(*retinfo)))
	{
		retval = -EFAULT;
	}

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
sdp_ser_set_info(
	struct sdp_ser_state *		state,
	struct serial_struct *		newinfo
	)
{
	struct serial_struct		new_serial;
	struct sdp_ser_port	*		port = state->port;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0))
	struct tty_port	*			tport = &state->tport;
#endif
	unsigned long				new_port;
	//--- no irq for DP TTY
	//unsigned int				change_irq;
	//---
	unsigned int				change_port;
	unsigned int				old_custom_divisor;
	unsigned int				closing_wait;
	unsigned int				close_delay;
	unsigned int				old_flags;
	unsigned int				new_flags;
	int							retval = 0;


	if (copy_from_user(&new_serial, newinfo, sizeof(new_serial)))
	{
		return -EFAULT;
	}

	new_port = new_serial.port;

	if (SDP_HIGH_BITS_OFFSET)
	{
		new_port += (unsigned long) new_serial.port_high << SDP_HIGH_BITS_OFFSET;
	}

	//--- no irq for DP TTY
	/*
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
	new_serial.irq = irq_canonicalize(new_serial.irq);
#endif
	*/
	//---

	close_delay = new_serial.close_delay;
	closing_wait = new_serial.closing_wait == ASYNC_CLOSING_WAIT_NONE ?	SDP_USF_CLOSING_WAIT_NONE : new_serial.closing_wait;

	down(&state->sem);

	//--- no iobase and irq for DP TTY
	/*
	change_irq  = new_serial.irq != port->irq;

	change_port =   new_port != port->iobase ||
					new_serial.io_type != port->iotype ||
					new_serial.type != port->type;
	*/
	change_port =   new_serial.io_type != port->iotype ||
					new_serial.type != port->type;
	//---
	old_flags = port->flags;
	new_flags = new_serial.flags;
	old_custom_divisor = port->custom_divisor;

	if (!capable(CAP_SYS_ADMIN))
	{
		retval = -EPERM;

		//--- no irq for DP TTY
		/*
		if (change_irq ||
			change_port ||
			(new_serial.baud_base != port->uartclk / 16) ||
			(close_delay != state->close_delay) ||
			(closing_wait != state->closing_wait) ||
			(new_serial.xmit_fifo_size != port->fifosize) ||
			(((new_flags ^ old_flags) & ~SDP_UPF_USR_MASK) != 0))
		*/
		if (change_port ||
			(new_serial.baud_base != port->uartclk / 16) ||
			(close_delay != state->close_delay) ||
			(closing_wait != state->closing_wait) ||
			(new_serial.xmit_fifo_size != port->fifosize) ||
			(((new_flags ^ old_flags) & ~SDP_UPF_USR_MASK) != 0))
		//---
		{
			goto exit;
		}

		port->flags = ((port->flags & ~SDP_UPF_USR_MASK) | (new_flags & SDP_UPF_USR_MASK));
		port->custom_divisor = new_serial.custom_divisor;
		goto check_and_exit;
	}

	//--- no irq for DP TTY
	//if (change_port || change_irq)
	if (change_port)
	//---
	{
		retval = -EBUSY;

		if (snxdp_ser_users(state) > 1)
		{
			goto exit;
		}
		sdp_ser_shutdown(state);
    }

	if (change_port)
	{
		unsigned int old_type;
		old_type = port->type;

		if (old_type != PORT_UNKNOWN)
        {
			snxdp_ser_release_io(port);
        }

		//--- no iobase for DP TTY
		//port->iobase = new_port;
		//---
		port->type = new_serial.type;
		port->iotype = new_serial.io_type;

		retval = 0;
	}

	//--- no irq for DP TTY
	//port->irq				= new_serial.irq;
	//---
	port->uartclk			= new_serial.baud_base * 16;
	port->flags				= ((port->flags & ~SDP_UPF_CHANGE_MASK) | (new_flags & SDP_UPF_CHANGE_MASK));
	port->custom_divisor	= new_serial.custom_divisor;
	state->close_delay		= close_delay;
	state->closing_wait		= closing_wait;
	port->fifosize			= new_serial.xmit_fifo_size;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0))
	tport->low_latency = (port->flags * SDP_UPF_LOW_LATENCY) ? 1 : 0;
#else
	if (state->info->tty)
	{
		state->info->tty->low_latency = (port->flags & SDP_UPF_LOW_LATENCY) ? 1 : 0;
	}
#endif


check_and_exit:
	retval = 0;
	if (port->type == PORT_UNKNOWN)
	{
		goto exit;
	}

	if (state->info->flags & SDP_UIF_INITIALIZED) 
	{
		if (((old_flags ^ port->flags) & SDP_UPF_SPD_MASK) || old_custom_divisor != port->custom_divisor) 
		{
			if (port->flags & SDP_UPF_SPD_MASK) 
            {
				printk("SDDVIRTUAL : (M:%d) %s, %s sets custom speed on ttyDP%d. This is deprecated.\n", port->line, __FUNCTION__, current->comm, port->line);
            }
			sdp_ser_change_speed(state, NULL);
		}
	}
	else
	{
		retval = sdp_ser_startup(state, 1);
	}
exit:

	up(&state->sem);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
sdp_ser_write_room(
	struct tty_struct *			tty
	)
{
	struct sdp_ser_state *		state = NULL;
	int							line = SDP_SER_DEVNUM(tty);
	int							status = 0;


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			status = 0;
			break;
		}

		state = tty->driver_data;

		status = sdp_ser_circ_chars_free(&state->info->xmit);

	} while (false);

	return status;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,10))
static int 
sdp_ser_write(
	struct tty_struct *			tty,
	const unsigned char *		buf,
	int 						count
	)
#else
static int 
sdp_ser_write(
	struct tty_struct *			tty,
	int 						from_user,
	const unsigned char *		buf,
	int 						count
	)
#endif
{
	int							line = SDP_SER_DEVNUM(tty);
	struct sdp_ser_port *		port = &snxdp_ser_table[line].port;
	struct sdp_ser_state *		state = tty->driver_data;
	struct circ_buf	*			circ = NULL;
	int							c;
	int							ret = 0;


	do
	{
		if (!state || !state->info)
		{
			ret = -EL3HLT;
			break;
		}

		circ = &state->info->xmit;

		if (!circ->buf)
		{
			ret = 0;
			break;
		}


		mutex_lock(&port->xmit_buff_mutex);
		while (1)
		{
			c = CIRC_SPACE_TO_END(circ->head, circ->tail, SDP_UART_XMIT_SIZE);
			if (count < c)
			{
				c = count;
			}

			if (c <= 0)
			{
				break;
			}
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9))
			memcpy(circ->buf + circ->head, buf, c);
#else
			if (from_user)
			{
				if (copy_from_user((circ->buf + circ->head), buf, c) == c)
				{
					ret = -EFAULT;
					break;
				}
			}
			else
			{
				memcpy(circ->buf + circ->head, buf, c);
			}
#endif

			circ->head = (circ->head + c) & (SDP_UART_XMIT_SIZE - 1);
			buf += c;
			count -= c;
			ret += c;
		}

		mutex_unlock(&port->xmit_buff_mutex);

		sdp_ser_start(tty);

	} while (false);

	return ret;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
sdp_ser_get_lsr_info(
	struct sdp_ser_state *		state,
	unsigned int *				value
	)
{
	struct sdp_ser_port	*		port = state->port;
	unsigned int				result = 0;
	int							retval = 0;


	result = snxdp_ser_tx_empty(port);

	if ((port->x_char) || 
		((sdp_ser_circ_chars_pending(&state->info->xmit) > 0) &&
		!state->info->tty->stopped && !state->info->tty->hw_stopped))
	{
		result &= ~TIOCSER_TEMT;
	}


#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,18))
	if (copy_to_user(value, &result, sizeof(int)))
	{
		retval = -EFAULT;
	}
#else
	retval = put_user(result, value);
#endif
	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,39))
static int 
sdp_ser_tiocmget(
	struct tty_struct *			tty
	)
{
	struct sdp_ser_state *		state = NULL;
	struct sdp_ser_port	*		port = NULL;
	int							result = -EIO;
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			result = 0;
			break;
		}

		state = tty->driver_data;
		port = state->port;

		down(&state->sem);

		if (!(tty->flags & (1 << TTY_IO_ERROR)))
		{
			result = port->mctrl;
			result |= snxdp_ser_get_mctrl(port);
		}

		up(&state->sem);

	} while (false);

	return result;
}


static int 
sdp_ser_tiocmset(
	struct tty_struct *			tty,
	unsigned int 				set,
	unsigned int 				clear
	)
{
	struct sdp_ser_state *		state = NULL;
	struct sdp_ser_port	*		port = NULL;
	int							ret = -EIO;
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			ret = 0;
			break;
		}

		state = tty->driver_data;
		port = state->port;

		down(&state->sem);

		if (!(tty->flags & (1 << TTY_IO_ERROR)))
		{
			sdp_ser_update_mctrl(port, set, clear);
			ret = 0;
		}

		up(&state->sem);

	} while (false);

	return ret;
}


#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))


static int 
sdp_ser_tiocmget(
	struct tty_struct *			tty,
	struct file *				file
	)
{
	struct sdp_ser_state *		state = NULL;
	struct sdp_ser_port	*		port = NULL;
	int							result = -EIO;
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			result = 0;
			break;
		}

		state = tty->driver_data;
		port = state->port;

		down(&state->sem);

		if ((!file || !tty_hung_up_p(file)) && !(tty->flags & (1 << TTY_IO_ERROR))) 
		{
			result = port->mctrl;
			result |= snxdp_ser_get_mctrl(port);
		}

		up(&state->sem);

	} while (false);

	return result;
}


static int 
sdp_ser_tiocmset(
	struct tty_struct *			tty,
	struct file *				file,
	unsigned int 				set,
	unsigned int 				clear
	)
{
	struct sdp_ser_state *		state = NULL;
	struct sdp_ser_port	*		port = NULL;
	int							ret = -EIO;
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			ret = 0;
			break;
		}

		state = tty->driver_data;
		port = state->port; 

		down(&state->sem);

		if ((!file || !tty_hung_up_p(file)) && !(tty->flags & (1 << TTY_IO_ERROR)))
		{
			sdp_ser_update_mctrl(port, set, clear);
			ret = 0;
		}

		up(&state->sem);

	} while (false);

	return ret;
}


#else


static int 
sdp_ser_get_modem_info(
	struct sdp_ser_state *		state,
	unsigned int *				value
	)
{
	struct sdp_ser_port	*		port = NULL;
	int							line;
	unsigned int				result;
	int							retval = 0;


	do
	{
		if (!state)
		{
			retval = -EIO;
			break;
		}

		port = state->port;

		if (!port)
		{
			retval = -EIO;
			break;
		}

		line = port->line;

		if (line >= SDP_SER_TOTAL_MAX)
		{
			retval = -EIO;
			break;
		}

		result = port->mctrl;
		result |= snxdp_ser_get_mctrl(port);

		put_user(result, (unsigned long *)value);

	} while (false);

	return retval;
}


static int 
sdp_ser_set_modem_info(
	struct sdp_ser_state *		state,
	unsigned int 				cmd,
	unsigned int *				value
	)
{
	struct sdp_ser_port	*		port = NULL;
	int							line;
	unsigned int				set = 0;
	unsigned int				clr = 0;
	unsigned int				arg;
	int							retval = 0;


	do
	{
		if (!state)
		{
			retval = -EIO;
			break;
		}

		port = state->port;

		if (!port)
		{
			retval = -EIO;
			break;
		}

		line = port->line;

		if (line >= SDP_SER_TOTAL_MAX)
		{
			retval = -EIO;
			break;
		}

		get_user(arg,(unsigned long *)value);

		switch (cmd)
		{
			case TIOCMBIS:
			{
				if (arg & TIOCM_RTS)
				{
					set |= TIOCM_RTS;
				}

				if (arg & TIOCM_DTR)
				{
					set |= TIOCM_DTR;
				}

				if (arg & TIOCM_LOOP)
				{
					set |= TIOCM_LOOP;
				}
				break;
			}

			case TIOCMBIC:
			{
				if (arg & TIOCM_RTS)
				{
					clr |= TIOCM_RTS;
				}

				if (arg & TIOCM_DTR)
				{
					clr |= TIOCM_DTR;
				}

				if (arg & TIOCM_LOOP)
				{
					clr |= TIOCM_LOOP;
				}
				break;
			}

			case TIOCMSET:
			{
				if (arg & TIOCM_RTS)
				{
					set |= TIOCM_RTS;
				}
				else
				{
					clr |= TIOCM_RTS;
				}

				if (arg & TIOCM_DTR)
				{
					set |= TIOCM_DTR;
				}
				else
				{
					clr |= TIOCM_DTR;
				}

				if (arg & TIOCM_LOOP)
				{
					set |= TIOCM_LOOP;
				}
				else
				{
					clr |= TIOCM_LOOP;
				}
				break;
			}

			default:
			{
				retval = -EINVAL;
				break;
			}
		}

		if (retval != 0)
		{
			break;
		}

		sdp_ser_update_mctrl(port, set, clr);

	} while (false);

	return retval;
}


#endif


/*-----------------------------------------------------------------------------------------------------------------------------*/


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27))
static int 
sdp_ser_break_ctl(
	struct tty_struct *			tty,
	int 						break_state
	)
#else
static void 
	sdp_ser_break_ctl(
	struct tty_struct *			tty,
	int 						break_state
	)
#endif
{
	struct sdp_ser_state *		state = NULL;
	struct sdp_ser_port	*		port = NULL;
	int							line = SDP_SER_DEVNUM(tty);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27))
	int							retval = 0;
#endif


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27))
			retval = 0;
#endif
			break;
		}

		state = tty->driver_data;
		port = state->port;

		down(&state->sem);

		if (port->type != PORT_UNKNOWN)
		{
			snxdp_ser_break_ctl(port, break_state);
		}

		up(&state->sem);

	} while (false);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27))
	return retval;
#else

#endif
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
sdp_ser_wait_modem_status(
	struct sdp_ser_state *		state,
	unsigned long 				arg
	)
{
	struct sdp_ser_port	*		port = state->port;
	DECLARE_WAITQUEUE(wait, current);
	struct sdp_ser_icount		cprev;
	struct sdp_ser_icount		cnow;
	int							ret = 0;


	spin_lock_irq(&port->lock);
	memcpy(&cprev, &port->icount, sizeof(struct sdp_ser_icount));

	snxdp_ser_enable_ms(port);

	spin_unlock_irq(&port->lock);

	add_wait_queue(&state->info->delta_msr_wait, &wait);

	for (;;)
	{
		spin_lock_irq(&port->lock);
		memcpy(&cnow, &port->icount, sizeof(struct sdp_ser_icount));
		spin_unlock_irq(&port->lock);

		set_current_state(TASK_INTERRUPTIBLE);

		if (((arg & TIOCM_RNG) && (cnow.rng != cprev.rng)) ||
			((arg & TIOCM_DSR) && (cnow.dsr != cprev.dsr)) ||
			((arg & TIOCM_CD)  && (cnow.dcd != cprev.dcd)) ||
			((arg & TIOCM_CTS) && (cnow.cts != cprev.cts)))
		{
			ret = 0;
			break;
		}

		schedule();


		if (signal_pending(current))
		{
			ret = -ERESTARTSYS;
			break;
		}

		cprev = cnow;
	}

	current->state = TASK_RUNNING;
	remove_wait_queue(&state->info->delta_msr_wait, &wait);

	return ret;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
sdp_ser_get_count(
	struct sdp_ser_state *		state,
	struct serial_icounter_struct * icnt
	)
{
	struct serial_icounter_struct icount;
	struct sdp_ser_icount 		cnow;
	struct sdp_ser_port *		port = state->port;
	int							retval = 0;


	spin_lock_irq(&port->lock);
	memcpy(&cnow, &port->icount, sizeof(struct sdp_ser_icount));
	spin_unlock_irq(&port->lock);

	icount.cts			= cnow.cts;
	icount.dsr			= cnow.dsr;
	icount.rng			= cnow.rng;
	icount.dcd			= cnow.dcd;
	icount.rx			= cnow.rx;
	icount.tx			= cnow.tx;
	icount.frame		= cnow.frame;
	icount.overrun		= cnow.overrun;
	icount.parity		= cnow.parity;
	icount.brk			= cnow.brk;
	icount.buf_overrun	= cnow.buf_overrun;

	if (copy_to_user(icnt, &icount, sizeof(icount)))
	{
		retval = -EFAULT;
	}

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,39))
static int 
sdp_ser_ioctl(
	struct tty_struct *			tty,
	unsigned int 				cmd,
	unsigned long 				arg
	)
#else
static int 
sdp_ser_ioctl(
	struct tty_struct *			tty,
	struct file *				filp,
	unsigned int 				cmd,
	unsigned long 				arg
	)
#endif
{
	struct sdp_ser_state *		state = NULL;
	int							ret = -ENOIOCTLCMD;
	int							line = SDP_SER_DEVNUM(tty);
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))    
	int							status = 0;
#endif


	if (line < SDP_SER_TOTAL_MAX)
	{
		state = tty->driver_data;
	}


	switch (cmd)
	{
		case TIOCGSERIAL:
		{
			if (line < SDP_SER_TOTAL_MAX)
			{
				ret = sdp_ser_get_info(state, (struct serial_struct *)arg);
			}
			break;
		}

		case TIOCSSERIAL:
		{
			if (line < SDP_SER_TOTAL_MAX)
			{
				state->port->setserial_flag = SDP_SER_BAUD_SETSERIAL;
				ret = sdp_ser_set_info(state, (struct serial_struct *)arg);
			}
			break;
		}

		case TCSETS:
		{
			if (line < SDP_SER_TOTAL_MAX)
			{
				state->port->flags &= ~(SDP_UPF_SPD_HI | SDP_UPF_SPD_VHI | SDP_UPF_SPD_SHI | SDP_UPF_SPD_WARP);
				state->port->setserial_flag = SDP_SER_BAUD_NOTSETSER;
				sdp_ser_update_termios(state);
			}
			break;
		}

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
		case TIOCMGET:
		{
			if (line < SDP_SER_TOTAL_MAX)
			{
				ret = verify_area(VERIFY_WRITE, (void *)arg, sizeof(unsigned int));
				if (ret)
				{
					return ret;
				}

				status = sdp_ser_get_modem_info(state, (unsigned int *)arg);
				return status;
			}
			break;
		}

		case TIOCMBIS:
		case TIOCMBIC:
		case TIOCMSET:
		{
			if (line < SDP_SER_TOTAL_MAX)
			{
				status = sdp_ser_set_modem_info(state, cmd, (unsigned int *)arg);
				return status;
			}
			break;
		}
#endif

		case TIOCSERGWILD:
		case TIOCSERSWILD:
		{
			if (line < SDP_SER_TOTAL_MAX)
			{
				ret = 0;
			}
			break;
		}
	}


	if (ret != -ENOIOCTLCMD)
	{
		goto out;
	}


	if (tty->flags & (1 << TTY_IO_ERROR))
	{
		ret = -EIO;
		goto out;
	}


	switch (cmd)
	{
		case TIOCMIWAIT:
			if (line < SDP_SER_TOTAL_MAX)
			{
				ret = sdp_ser_wait_modem_status(state, arg);
			}
			break;

		case TIOCGICOUNT:
			if (line < SDP_SER_TOTAL_MAX)
			{
				ret = sdp_ser_get_count(state, (struct serial_icounter_struct *)arg);
			}
			break;
	}


	if (ret != -ENOIOCTLCMD)
	{
		goto out;
	}


	if (line < SDP_SER_TOTAL_MAX)
	{
		down(&state->sem);

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,39))
		if (tty_hung_up_p(filp))
		{
			ret = -EIO;
			goto out_up;
		}
#endif 

		switch (cmd)
		{
			case TIOCSERGETLSR:
				ret = sdp_ser_get_lsr_info(state, (unsigned int *)arg);
				break;

			default: 
                break;
		}
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,39))
out_up:
#endif
		up(&state->sem);

	}

out:
	return ret;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_hangup(
	struct tty_struct *			tty
	)
{
	struct sdp_ser_state *		state = NULL;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
	struct tty_port	*			tport = &state->tport;
#endif
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			break;
		}

		state = tty->driver_data;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		tport = &state->tport;
#endif

		down(&state->sem);

		if (state->info && state->info->flags & SDP_UIF_NORMAL_ACTIVE) 
		{
			sdp_ser_flush_buffer(tty);
			sdp_ser_shutdown(state);
			state->count = 0;
			state->info->flags &= ~SDP_UIF_NORMAL_ACTIVE;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
			tty_port_tty_set(tport, NULL);
#endif

			state->info->tty = NULL;
			wake_up_interruptible(&state->info->open_wait);
			wake_up_interruptible(&state->info->delta_msr_wait);
		}

		up(&state->sem);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static unsigned int 
sdp_ser_get_divisor(
	struct sdp_ser_port *		port,
	unsigned int 				baud
	)
{
	unsigned int				quot;


	if (baud == 38400 && (port->flags & SDP_UPF_SPD_MASK) == SDP_UPF_SPD_CUST)
	{
		quot = port->custom_divisor;
	}
	else
	{
		quot = port->uartclk / (16 * baud);
	}

	return quot;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static unsigned int 
sdp_ser_get_baud_rate(
	struct sdp_ser_port *		port,
	struct SDPTERMIOS *			termios,
	struct SDPTERMIOS *			old,
	unsigned int 				min,
	unsigned int 				max
	)
{
	unsigned int				try;
	unsigned int				baud;
	unsigned int				altbaud = 0;
	unsigned int				flags = port->flags & SDP_UPF_SPD_MASK;


	for (try = 0; try < 2; try++)
    {
		if ((port->setserial_flag == SDP_SER_BAUD_SETSERIAL) || (port->flags & SDP_UPF_SPD_MASK))
		{
			altbaud = 38400;

			if (flags == SDP_UPF_SPD_HI)
			{
				altbaud = 57600;
			}

			if (flags == SDP_UPF_SPD_VHI)
			{
				altbaud = 115200;
			}

			if (flags == SDP_UPF_SPD_SHI)
			{
				altbaud = 230400;
			}

			if (flags == SDP_UPF_SPD_WARP)
			{
				altbaud = 460800;
			}

			baud = altbaud;
		}
		else
		{
			switch (termios->c_cflag & (CBAUD | CBAUDEX))
			{
				case B921600	: baud = 921600;	break;
				case B460800	: baud = 460800;	break;
				case B230400	: baud = 230400;	break;
				case B115200	: baud = 115200;	break;
				case B57600		: baud = 57600;		break;
				case B38400		: baud = 38400;		break;
				case B19200		: baud = 19200;		break;
				case B9600		: baud = 9600;		break;
				case B4800		: baud = 4800;		break;
				case B2400		: baud = 2400;		break;
				case B1800		: baud = 1800;		break;
				case B1200		: baud = 1200;		break;
				case B600		: baud = 600;		break;
				case B300		: baud = 300;		break;
				case B200		: baud = 200;		break;
				case B150		: baud = 150;		break;
				case B134		: baud = 134;		break;
				case B110		: baud = 110;		break;
				case B75		: baud = 75;		break;
				case B50		: baud = 50;		break;
				default			: baud = 9600;		break;
			}
		}

		if (baud == 0)
		{
			baud = 9600;
		}

		if (baud >= min && baud <= max)
		{
			return baud;
		}

		termios->c_cflag &= ~CBAUD;

		if (old)
		{
			termios->c_cflag |= old->c_cflag & CBAUD;
			old = NULL;
			continue;
		}

		termios->c_cflag |= B9600;
	}

	return 0;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_change_speed(
	struct sdp_ser_state *		state,
	struct SDPTERMIOS *			old_termios
	)
{
	struct tty_struct *			tty = state->info->tty;
	struct sdp_ser_port	*		port = state->port;
	struct SDPTERMIOS *			termios;


	do
	{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		if (!tty || port->type == PORT_UNKNOWN)
		{
			break;
		}
#else
		if (!tty || !tty->termios || port->type == PORT_UNKNOWN)
		{
			break;
		}
#endif


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		termios = &tty->termios;
#else
		termios = tty->termios;
#endif

		if (termios->c_cflag & CRTSCTS)
		{
			state->info->flags |= SDP_UIF_CTS_FLOW;
		}
		else
		{
			state->info->flags &= ~SDP_UIF_CTS_FLOW;
		}

		if (termios->c_cflag & CLOCAL)
		{
			state->info->flags &= ~SDP_UIF_CHECK_CD;
		}
		else
		{
			state->info->flags |= SDP_UIF_CHECK_CD;
		}

		snxdp_ser_set_termios(port, termios, old_termios);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


#define RELEVANT_IFLAG(iflag)			((iflag) & (IGNBRK|BRKINT|IGNPAR|PARMRK|INPCK))

static void 
sdp_ser_set_termios(
	struct tty_struct *			tty,
	struct SDPTERMIOS *			old_termios
	)
{
	struct sdp_ser_state *		state = NULL;
	unsigned long				flags;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
	unsigned int				cflag = tty->termios.c_cflag;
#else
	unsigned int				cflag = tty->termios->c_cflag;
#endif
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			break;
		}

		state = tty->driver_data;


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		if ((cflag ^ old_termios->c_cflag) == 0 && RELEVANT_IFLAG(tty->termios.c_iflag ^ old_termios->c_iflag) == 0)
		{
			break;
		}
#else
		if ((cflag ^ old_termios->c_cflag) == 0 && RELEVANT_IFLAG(tty->termios->c_iflag ^ old_termios->c_iflag) == 0)
		{
			break;
		}
#endif

		sdp_ser_change_speed(state, old_termios);

		if ((old_termios->c_cflag & CBAUD) && !(cflag & CBAUD))
		{
			sdp_clear_mctrl(state->port, TIOCM_RTS | TIOCM_DTR);
		}

		if (!(old_termios->c_cflag & CBAUD) && (cflag & CBAUD))
		{
			unsigned int mask = TIOCM_DTR;
			if (!(cflag & CRTSCTS) || !test_bit(TTY_THROTTLED, &tty->flags))
			{
				mask |= TIOCM_RTS;
			}

			sdp_set_mctrl(state->port, mask);
		}

		if ((old_termios->c_cflag & CRTSCTS) && !(cflag & CRTSCTS))
		{
			spin_lock_irqsave(&state->port->lock, flags);
			tty->hw_stopped = 0;
			__sdp_ser_start(tty);
			spin_unlock_irqrestore(&state->port->lock, flags);
		}

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_update_termios(
	struct sdp_ser_state *		state
	)
{
	struct tty_struct *			tty = state->info->tty;
	struct sdp_ser_port	*		port = state->port;


	if (!(tty->flags & (1 << TTY_IO_ERROR)))
	{
		sdp_ser_change_speed(state, NULL);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		if (tty->termios.c_cflag & CBAUD)
		{
			sdp_set_mctrl(port, TIOCM_DTR | TIOCM_RTS);
		}
#else
		if (tty->termios->c_cflag & CBAUD)
		{
			sdp_set_mctrl(port, TIOCM_DTR | TIOCM_RTS);
		}
#endif
    }
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_update_timeout(
	struct sdp_ser_port *		port,
	unsigned int 				cflag,
	unsigned int				baud
	)
{
	unsigned int				bits;


	switch (cflag & CSIZE)
	{
		case CS5:
			bits = 7;
			break;

		case CS6:
			bits = 8;
			break;

		case CS7:
			bits = 9;
			break;

		default:
			bits = 10;
			break;
	}

	if (cflag & CSTOPB)
	{
		bits++;
	}

	if (cflag & PARENB)
	{
		bits++;
	}

	bits = bits * port->fifosize;

	port->timeout = (HZ * bits) / baud + HZ / 50;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static struct sdp_ser_state * 
sdp_ser_get(
	struct sdp_ser_driver *		drv,
	int 						line
	)
{
	struct sdp_ser_state *		state = NULL;


	down(&ser_port_sem);

	state = drv->state + line;

	if (down_interruptible(&state->sem))
	{
		state = ERR_PTR(-ERESTARTSYS);
		goto out;
	}

	state->count++;

	if (!state->port)
	{
		state->count--;
		up(&state->sem);
		state = ERR_PTR(-ENXIO);
		goto out;
	}

	//--- no iobase for DP TTY
	/*
	if (!state->port->iobase)
	{
		state->count--;
		up(&state->sem);
		state = ERR_PTR(-ENXIO);
		goto out;
	}
	*/
	//---

	if (!state->info)
	{
		state->info = kmalloc(sizeof(struct sdp_ser_info), GFP_KERNEL);

		if (state->info)
		{
			memset(state->info, 0, sizeof(struct sdp_ser_info));
			init_waitqueue_head(&state->info->open_wait);
			init_waitqueue_head(&state->info->delta_msr_wait);

			state->port->info = state->info;

			tasklet_init(&state->info->tlet, sdp_ser_tasklet_action, (unsigned long)state);
		}
		else
		{
			state->count--;
			up(&state->sem);
			state = ERR_PTR(-ENOMEM);
		}
	}

out:
	up(&ser_port_sem);

	return state;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
sdp_ser_block_til_ready(
	struct file *				filp,
	struct sdp_ser_state *		state
	)
{
	DECLARE_WAITQUEUE(wait, current);
	struct sdp_ser_info	*		info = state->info;
	struct sdp_ser_port	*		port = state->port;


	info->blocked_open++;
	state->count--;

	add_wait_queue(&info->open_wait, &wait);

	while (1)
	{
		set_current_state(TASK_INTERRUPTIBLE);

		if (tty_hung_up_p(filp) || info->tty == NULL)
		{
			break;
		}

		if (!(info->flags & SDP_UIF_INITIALIZED))
		{
			break;
		}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		if ((filp->f_flags & O_NONBLOCK) ||
			(info->tty->termios.c_cflag & CLOCAL) ||
			(info->tty->flags & (1 << TTY_IO_ERROR)))
		{
			break;
		}

		if (info->tty->termios.c_cflag & CBAUD)
		{
			sdp_set_mctrl(port, TIOCM_DTR);
		}
#else
		if ((filp->f_flags & O_NONBLOCK) ||
			(info->tty->termios->c_cflag & CLOCAL) ||
			(info->tty->flags & (1 << TTY_IO_ERROR)))
		{
			break;
		}

		if (info->tty->termios->c_cflag & CBAUD)
		{
			sdp_set_mctrl(port, TIOCM_DTR);
		}
#endif


		if (snxdp_ser_get_mctrl(port) & TIOCM_CAR)
		{
			break;
		}

		up(&state->sem);
		schedule();
		down(&state->sem);

		if (signal_pending(current))
		{
			break;
		}
	}

	set_current_state(TASK_RUNNING);
	remove_wait_queue(&info->open_wait, &wait);

	state->count++;
	info->blocked_open--;

	if (signal_pending(current))
	{
		return -ERESTARTSYS;
	}

	if (!info->tty || tty_hung_up_p(filp))
	{
		return -EAGAIN;
	}

	return 0;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_wait_until_sent(
	struct tty_struct *			tty,
	int 						timeout
	)
{
	struct sdp_ser_state *		state = NULL;
	struct sdp_ser_port	*		port = NULL;
	unsigned long				char_time;
	unsigned long				expire;
	int							line = SDP_SER_DEVNUM(tty);


	do
	{
		if (line >= SDP_SER_TOTAL_MAX)
		{
			break;
		}

		state = tty->driver_data;
		port = state->port;

		if (port->type == PORT_UNKNOWN || port->fifosize == 0)
		{
			break;
		}

		char_time = (port->timeout - HZ/50) / port->fifosize;

		char_time = char_time / 5;

		if (char_time == 0)
		{
			char_time = 1;
		}

		if (timeout && timeout < char_time)
		{
			char_time = timeout;
		}

		if (timeout == 0 || timeout > 2 * port->timeout)
		{
			timeout = 2 * port->timeout;
		}

		expire = jiffies + timeout;

		while (!snxdp_ser_tx_empty(port))
		{
			set_current_state(TASK_INTERRUPTIBLE);
			schedule_timeout(char_time);

			if (signal_pending(current))
			{
				break;
			}

			if (time_after(jiffies, expire))
			{
				break;
			}
		}
		set_current_state(TASK_RUNNING);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
sdp_ser_open(
	struct tty_struct *			tty,
	struct file *				filp
	)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
	struct sdp_ser_driver *		drv = (struct sdp_ser_driver *)tty->driver->driver_state;
#else
	struct sdp_ser_driver *		drv = (struct sdp_ser_driver *)tty->driver.driver_state;
#endif
	struct sdp_ser_state *		state = NULL;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
	struct tty_port	*			tport = NULL;
#endif
	int							retval = 0;
	int							line = SDP_SER_DEVNUM(tty);


	//--- DP NLK
	retval = ser_open(tty, filp);
	if (retval < 0)
	{
		return retval;
	}
	//---

	if (line < SDP_SER_TOTAL_MAX)
	{
		retval = -ENODEV;

		if (line >= SDP_SER_TOTAL_MAX)
		{
			goto fail;
		}

		state = sdp_ser_get(drv, line);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		tport = &state->tport;
#endif

		if (IS_ERR(state))
		{
			retval = PTR_ERR(state);
			goto fail;
		}

		if (!state)
		{
			goto fail;
		}

		state->port->suspended = 1;
		tty->driver_data = state;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0))
		tport->low_latency = (state->port->flags & SDP_UPF_LOW_LATENCY) ? 1 : 0;
#else
		tty->low_latency = (state->port->flags & SDP_UPF_LOW_LATENCY) ? 1 : 0;
#endif


#if (LINUX_VERSION_CODE <= KERNEL_VERSION(4,12,0))
		tty->alt_speed = 0;
#endif
		state->info->tty = tty;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		tty_port_tty_set(tport, tty);
#endif

		if (tty_hung_up_p(filp))
		{
			retval = -EAGAIN;
			state->count--;
			up(&state->sem);
			goto fail;
		}

		retval = sdp_ser_startup(state, 0);
		if (retval == 0)
		{
			retval = sdp_ser_block_til_ready(filp, state);
		}

		up(&state->sem);

		if (retval == 0 && !(state->info->flags & SDP_UIF_NORMAL_ACTIVE))
		{
			state->info->flags |= SDP_UIF_NORMAL_ACTIVE;
			sdp_ser_update_termios(state);
		}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
		try_module_get(THIS_MODULE);
#else
		MOD_INC_USE_COUNT;
#endif
	}
	else
	{

	}

fail:

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
sdp_ser_close(
	struct tty_struct *			tty,
	struct file *				filp
	)
{
	struct sdp_ser_state *		state = tty->driver_data;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
	struct tty_port	*			tport;
#endif
	struct sdp_ser_port	*		port = NULL;
	int							retval = 0;
	int							line = SDP_SER_DEVNUM(tty);


	if (line < SDP_SER_TOTAL_MAX)
	{
		if (!state || !state->port)
		{
			return;
		}

		port = state->port;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		tport = &state->tport;
#endif

		down(&state->sem);

		if (tty_hung_up_p(filp))
		{
			goto done;
		}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
		if ((tty->count == 1) && (state->count != 1))
		{
			state->count = 1;
		}
#endif

		if (--state->count < 0)
		{
			state->count = 0;
		}

		if (state->count)
		{
			goto done;
		}

		tty->closing = 1;

		port->suspended = 0;
		if (state->closing_wait != SDP_USF_CLOSING_WAIT_NONE)
		{
			tty_wait_until_sent(tty, state->closing_wait);
		}

		if (state->info->flags & SDP_UIF_INITIALIZED)
		{
			unsigned long flags;

			spin_lock_irqsave(&port->lock, flags);
			snxdp_ser_stop_rx(port);
			spin_unlock_irqrestore(&port->lock, flags);

			sdp_ser_wait_until_sent(tty, port->timeout);
		}

		sdp_ser_shutdown(state);
		sdp_ser_flush_buffer(tty);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31))
		if (tty->ldisc->ops->flush_buffer)
		{
			tty->ldisc->ops->flush_buffer(tty);
		}
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)&& LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,30))
		if (tty->ldisc.ops->flush_buffer)
		{
			tty->ldisc.ops->flush_buffer(tty);
		}
#else
		if (tty->ldisc.flush_buffer)
		{
			tty->ldisc.flush_buffer(tty);
		}
#endif


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		tty_port_tty_set(tport, NULL);
#endif

		tty->closing = 0;
		state->info->tty = NULL;

		if (state->info->blocked_open)
		{
			if (state->close_delay)
			{
				set_current_state(TASK_INTERRUPTIBLE);
				schedule_timeout(state->close_delay);
			}
		}

		state->info->flags &= ~SDP_UIF_NORMAL_ACTIVE;
		wake_up_interruptible(&state->info->open_wait);

done:
		up(&state->sem);

		//--- DP NLK
		retval = ser_close(tty, filp);
		if (retval < 0)
		{

		}
		//---
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
		module_put(THIS_MODULE);
#else
		MOD_DEC_USE_COUNT;
#endif
	}
	else
	{

	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_ser_set_mctrl(
	struct sdp_ser_port *		port,
	unsigned int 				mctrl
	)
{
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;
	unsigned char				mcr = 0;
	int							retval = 0;


	//--- DP NLK
	/*
   	if (mctrl & TIOCM_RTS)
	{
		mcr |= UART_MCR_RTS;
	}

	if (mctrl & TIOCM_DTR)
	{
		mcr |= UART_MCR_DTR;
	}

	if (mctrl & TIOCM_OUT1)
	{
		mcr |= UART_MCR_OUT1;
	}

	if (mctrl & TIOCM_OUT2)
	{
		mcr |= UART_MCR_OUT2;
	}

	if (mctrl & TIOCM_LOOP)
	{
		mcr |= UART_MCR_LOOP;
	}

	mcr = (mcr & sp->mcr_mask) | sp->mcr_force | sp->mcr;

	WRITE_UART_MCR(sp, mcr);
	*/
   	if (mctrl & TIOCM_RTS)
	{
		mcr |= 0x01;
	}

	if (mctrl & TIOCM_DTR)
	{
		mcr |= 0x02;
	}

	if (mctrl & TIOCM_OUT1)
	{
		mcr |= 0x04;
	}

	if (mctrl & TIOCM_OUT2)
	{
		mcr |= 0x08;
	}

	if (mctrl & TIOCM_LOOP)
	{
		mcr |= 0x10;
	}

	mcr = mcr | (sp->mcr & 0x20);

	if (sp->uart_version <= 0x03)
	{
		retval = ser_cmd_set_mcr(sp, mcr);
		if (retval < 0)
		{

		}
	}
	else
	{
		retval = ser_cmd_set_mcr(sp, mcr);
		if (retval < 0)
		{

		}
		retval = ser_cmd_set_on_chip_hw_flow_ctrl(sp, sp->on_chip_hw_flow_ctrl);
		if (retval < 0)
		{

		}
	}
	//---
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static unsigned int 
snxdp_ser_tx_empty(
	struct sdp_ser_port	*		port
	)
{
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;
	unsigned long				flags;
	unsigned int				ret;
	//int						retval = 0;


	spin_lock_irqsave(&sp->port.lock, flags);

	//--- DP NLK
	//ret = READ_UART_LSR(sp) & UART_LSR_TEMT ? TIOCSER_TEMT : 0;
	/*
	retval = ser_cmd_get_msr(sp);
	if (retval < 0)
	{

	}
	*/
	ret = (sp->msr & 0x10) ? TIOCSER_TEMT : 0;
	//---
	spin_unlock_irqrestore(&sp->port.lock, flags);

	return ret;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static unsigned int 
snxdp_ser_get_mctrl(
	struct sdp_ser_port *		port
	)
{
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;
#if (DRIVERTARGET == 1)
	unsigned long				flags;
#endif
	unsigned char				status;
	unsigned int				ret;
	int							retval = 0;


	ret = 0;
#if (DRIVERTARGET == 1)
	spin_lock_irqsave(&sp->port.lock, flags);
#endif
	//--- DP NLK
	/*
	status = READ_UART_MSR(sp);

	spin_unlock_irqrestore(&sp->port.lock, flags);

	if (status & UART_MSR_DCD)
	{
		ret |= TIOCM_CAR;
	}

	if (status & UART_MSR_RI)
	{
		ret |= TIOCM_RNG;
	}

	if (status & UART_MSR_DSR)
	{
		ret |= TIOCM_DSR;
	}

	if (status & UART_MSR_CTS)
	{
		ret |= TIOCM_CTS;
	}
	*/


	//--- DP NLK
	retval = ser_cmd_get_msr(sp);
	if (retval < 0)
	{

	}
	status = sp->msr;
#if (DRIVERTARGET == 1)
	spin_unlock_irqrestore(&sp->port.lock, flags);
#endif

	if (status & 0x08)
	{
		ret |= TIOCM_CAR;
	}

	if (status & 0x04)
	{
		ret |= TIOCM_RNG;
	}

	if (status & 0x02)
	{
		ret |= TIOCM_DSR;
	}

	if (status & 0x01)
	{
		ret |= TIOCM_CTS;
	}
	//---
	return ret;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_ser_stop_tx(
	struct sdp_ser_port *		port,
	unsigned int 				tty_stop
	)
{
	//--- DP NLK
	/*
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;


	if (sp->ier & UART_IER_THRI)
	{
		sp->ier &= ~UART_IER_THRI;
		WRITE_UART_IER(sp, sp->ier);
	}
	*/
	//---
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_ser_start_tx(
	struct sdp_ser_port *		port,
	unsigned int 				tty_start
	)
{
    struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;


	//--- DP NLK
	/*
	if (!(sp->ier & UART_IER_THRI))
	{
		sp->ier |= UART_IER_THRI;
		WRITE_UART_IER(sp, sp->ier);
	}
	*/
	mutex_lock(&port->xmit_buff_mutex);

	if (sp->msr & 0x10)
	{
		snxdp_ser_transmit_chars(sp);
	}
	else
	{

	}

	mutex_unlock(&port->xmit_buff_mutex);
	//---
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_ser_stop_rx(
	struct sdp_ser_port *		port
	)
{
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;
	int							retval = 0;


	//--- DP NLK
	/*
	sp->ier &= ~UART_IER_RLSI;
	sp->port.read_status_mask &= ~UART_LSR_DR;
	WRITE_UART_IER(sp, sp->ier);
	*/
	if (sp->uart_version <= 0x03)
	{
		sp->event_ctrl &= ~0x01;
	}
	else
	{
		sp->event_ctrl &= ~0x05;
	}
	sp->port.read_status_mask &= ~UART_LSR_DR;
	retval = ser_cmd_set_event_ctrl(sp, sp->event_ctrl);
	if (retval < 0)
	{

	}
	//---
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_ser_enable_ms(
	struct sdp_ser_port *		port
	)
{
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;
	int							retval = 0;


	//--- DP NLK
	/*
	sp->ier &= ~UART_IER_RLSI;
	sp->port.read_status_mask &= ~UART_LSR_DR;
	WRITE_UART_IER(sp, sp->ier);
	*/
	if (sp->uart_version <= 0x03)
	{
		sp->event_ctrl &= ~0x01;
	}
	else
	{
		sp->event_ctrl &= ~0x05;
	}
	sp->port.read_status_mask &= ~UART_LSR_DR;
	retval = ser_cmd_set_event_ctrl(sp, sp->event_ctrl);
	if (retval < 0)
	{

	}
	//---
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_ser_break_ctl(
	struct sdp_ser_port *		port,
	int 						break_state
	)
{
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;
	unsigned long				flags;
	int							retval = 0;


	spin_lock_irqsave(&sp->port.lock, flags);

	if (break_state == -1)
	{
		sp->lcr |= UART_LCR_SBC;
	}
	else
	{
		sp->lcr &= ~UART_LCR_SBC;
	}

	//--- DP NLK
	//WRITE_UART_LCR(sp, sp->lcr);
	retval = ser_cmd_set_lcr(sp, sp->lcr);
	if (retval < 0)
	{

	}
	//---

	spin_unlock_irqrestore(&sp->port.lock, flags);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
snxdp_ser_startup(
	struct sdp_ser_port *		port
	)
{
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;
	int							retval = 0;


	sp->capabilities = sdp_uart_config[sp->port.type].flags;
	sp->mcr = 0;

	//--- DP NLK
	if (sp->capabilities & UART_CLEAR_FIFO)
	{
		/*
		WRITE_UART_FCR(sp, UART_FCR_ENABLE_FIFO);
		WRITE_UART_FCR(sp, UART_FCR_ENABLE_FIFO | UART_FCR_CLEAR_RCVR | UART_FCR_CLEAR_XMIT);
		WRITE_UART_FCR(sp, 0);
		*/
		retval = ser_cmd_set_fifo_ctrl(sp, 0x03);
		if (retval < 0)
		{

		}
		retval = ser_cmd_set_fifo_ctrl(sp, 0);
		if (retval < 0)
		{

		}
	}


	/*
	(void) READ_UART_LSR(sp);
	(void) READ_UART_RX(sp);
	(void) READ_UART_IIR(sp);
	(void) READ_UART_MSR(sp);
	*/


	//WRITE_UART_LCR(sp, UART_LCR_WLEN8);
	retval = ser_cmd_set_lcr(sp, UART_LCR_WLEN8);
	if (retval < 0)
	{

	}


	/*
	sp->ier = UART_IER_RLSI | UART_IER_RDI;
	WRITE_UART_IER(sp, sp->ier);
	*/
	if (sp->uart_version <= 0x03)
	{
		sp->event_ctrl |= 0x01;
	}
	else
	{
		sp->event_ctrl |= 0x05;
	}
	retval = ser_cmd_set_event_ctrl(sp, sp->event_ctrl);
	if (retval < 0)
	{

	}


	/*
	(void) READ_UART_LSR(sp);
	(void) READ_UART_RX(sp);
	(void) READ_UART_IIR(sp);
	(void) READ_UART_MSR(sp);
	*/
	//---
	return 0;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_ser_shutdown(
	struct sdp_ser_port	*		port
	)
{
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;
	int							retval = 0;


	//--- DP NLK
	/*
	sp->ier = 0;
	WRITE_UART_IER(sp, 0);
	*/
	sp->event_ctrl =0;
	retval = ser_cmd_set_event_ctrl(sp, sp->event_ctrl);
	if (retval < 0)
	{

	}


	//WRITE_UART_LCR(sp, READ_UART_LCR(sp) & ~UART_LCR_SBC);
	/*
	sp->lcr &= ~UART_LCR_SBC;
	retval = ser_cmd_set_lcr(sp, sp->lcr);
	if (retval < 0)
	{

	}
	*/


	/*
	WRITE_UART_FCR(sp, UART_FCR_ENABLE_FIFO | UART_FCR_CLEAR_RCVR | UART_FCR_CLEAR_XMIT);
	WRITE_UART_FCR(sp, 0);
	*/
	retval = ser_cmd_set_fifo_ctrl(sp, 0x03);
	if (retval < 0)
	{

	}
	/*
	retval = ser_cmd_set_fifo_ctrl(sp, 0);
	if (retval < 0)
	{

	}
	*/

	if (sp->uart_version <= 0x03)
	{
		retval = ser_cmd_set_mcr(sp, 0);
		if (retval < 0)
		{

		}
	}
	else
	{
		retval = ser_cmd_set_mcr(sp, 0);
		if (retval < 0)
		{

		}
		retval = ser_cmd_set_on_chip_hw_flow_ctrl(sp, 0);
		if (retval < 0)
		{

		}
	}
	//(void) READ_UART_RX(sp);
	//---
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static unsigned int 
snxdp_ser_get_divisor(
	struct sdp_ser_port *		port,
	unsigned int 				baud
	)
{
	unsigned int				quot;


	if ((port->flags & SDP_UPF_MAGIC_MULTIPLIER) && baud == (port->uartclk/4))
	{
		quot = 0x8001;
	}
	else if ((port->flags & SDP_UPF_MAGIC_MULTIPLIER) && baud == (port->uartclk/8))
	{
		quot = 0x8002;
	}
	else
	{
		quot = sdp_ser_get_divisor(port, baud);
	}

	return quot;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_ser_set_termios(
	struct sdp_ser_port *		port,
	struct SDPTERMIOS *			termios,
	struct SDPTERMIOS *			old
	)
{
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;
	unsigned char				cval;
	unsigned char				fcr = 0;
	unsigned long				flags;
	unsigned int				baud;
	unsigned int				quot;
	int							retval = 0;
	unsigned char				tx_trigger_level = 0x3f;
	unsigned char				rx_trigger_level = 0x3f;


	switch (termios->c_cflag & CSIZE)
	{
		case CS5:
			cval = 0x00;
			break;

		case CS6:
			cval = 0x01;
			break;

		case CS7:
			cval = 0x02;
			break;

		default:
		case CS8:
			cval = 0x03;
			break;
	}

	if (termios->c_cflag & CSTOPB)
	{
		cval |= 0x04;
	}

	if (termios->c_cflag & PARENB)
	{
		cval |= UART_LCR_PARITY;
	}

	if (!(termios->c_cflag & PARODD))
	{
		cval |= UART_LCR_EPAR;
	}

#ifdef CMSPAR
	if (termios->c_cflag & CMSPAR)
	{
		cval |= UART_LCR_SPAR;
	}
#endif

	baud = sdp_ser_get_baud_rate(port, termios, old, 0, port->uartclk / 16);
	quot = snxdp_ser_get_divisor(port, baud);

	//--- DP NLK
	/*
	if (sp->capabilities & UART_USE_FIFO)
	{
		if (baud < 2400)
		{
			fcr = UART_FCR_ENABLE_FIFO | UART_FCR_TRIGGER_1;
		}
		else
		{
			fcr = UART_FCR_ENABLE_FIFO | UART_FCR_TRIGGER_8;
		}
	}

	sp->mcr &= ~UART_MCR_AFE;
	if (termios->c_cflag & CRTSCTS)
	{
		sp->mcr |= UART_MCR_AFE;
	}
	*/
	if (sp->uart_version <= 0x03)
	{
		if (sp->capabilities & UART_USE_FIFO)
		{
			if (baud < 2400)
			{
				fcr = 0x00;
			}
			else
			{
				fcr = 0x2c;
			}
		}

		sp->mcr &= ~0x20;
		if (termios->c_cflag & CRTSCTS)
		{
			sp->mcr |= 0x20;
		}
	}
	else
	{
		if (sp->capabilities & UART_USE_FIFO)
		{
			if (baud < 2400)
			{
				tx_trigger_level = 0x01;
				rx_trigger_level = 0x01;
			}
			else
			{
				tx_trigger_level = 0xbf;
				rx_trigger_level = 0xbf;
			}
		}

		sp->on_chip_hw_flow_ctrl = 0x00;
		if (termios->c_cflag & CRTSCTS)
		{
			sp->on_chip_hw_flow_ctrl = 0x0d;
		}
	}
	//---


	spin_lock_irqsave(&sp->port.lock, flags);


	sdp_ser_update_timeout(port, termios->c_cflag, baud);


	sp->port.read_status_mask = UART_LSR_OE | UART_LSR_THRE | UART_LSR_DR;

	if (termios->c_iflag & INPCK)
	{
		sp->port.read_status_mask |= UART_LSR_FE | UART_LSR_PE;
	}

	if (termios->c_iflag & (BRKINT | PARMRK))
	{
		sp->port.read_status_mask |= UART_LSR_BI;
	}

	sp->port.ignore_status_mask = 0;

	if (termios->c_iflag & IGNPAR)
	{
		sp->port.ignore_status_mask |= UART_LSR_PE | UART_LSR_FE;
	}


	if (termios->c_iflag & IGNBRK)
	{
		sp->port.ignore_status_mask |= UART_LSR_BI;

		if (termios->c_iflag & IGNPAR)
		{
			sp->port.ignore_status_mask |= UART_LSR_OE;
		}
	}

	if ((termios->c_cflag & CREAD) == 0)
	{
		sp->port.ignore_status_mask |= UART_LSR_DR;
	}


	//--- DP NLK
	//sp->ier &= ~UART_IER_MSI;
	sp->event_ctrl &= ~0x02;
	if (SDP_ENABLE_MS(&sp->port, termios->c_cflag))
	{
		//sp->ier |= UART_IER_MSI;
		sp->event_ctrl |= 0x02;
	}


	/*
	WRITE_UART_LCR(sp, cval | UART_LCR_DLAB);
	WRITE_UART_DLL(sp, quot & 0xff);
	WRITE_UART_DLM(sp, quot >> 8);
	*/
	retval = ser_cmd_set_dll_dlm(sp, quot & 0xff, quot >> 8);
	if (retval < 0)
	{

	}


	if (sp->uart_version <= 0x03)
	{
		//WRITE_UART_FCR(sp, fcr);
		retval = ser_cmd_set_fifo_ctrl(sp, fcr);
		if (retval < 0)
		{

		}
	}
	else
	{
		//WRITE_UART_FCR(sp, fcr);
		retval = ser_cmd_set_fifo_ctrl(sp, fcr);
		if (retval < 0)
		{

		}
		retval = ser_cmd_set_tx_trigger_level(sp, tx_trigger_level);
		if (retval < 0)
		{

		}
		retval = ser_cmd_set_rx_trigger_level(sp, rx_trigger_level);
		if (retval < 0)
		{

		}
	}


	sp->lcr = cval;
	//WRITE_UART_LCR(sp, cval);
	retval = ser_cmd_set_lcr(sp, sp->lcr);
	if (retval < 0)
	{

	}


	snxdp_ser_set_mctrl(&sp->port, sp->port.mctrl);


	//WRITE_UART_IER(sp, sp->ier);
	retval = ser_cmd_set_event_ctrl(sp, sp->event_ctrl);
	if (retval < 0)
	{

	}
	//---

	spin_unlock_irqrestore(&sp->port.lock, flags);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/

/*
static void 
snxdp_ser_timeout(
	unsigned long 				data
	)
{
	//--- DP NLK
	//
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)data;
	unsigned int				timeout;
	unsigned int				iir = UART_IIR_NO_INT;


	iir = READ_UART_IIR(sp);

	if (!(iir & UART_IIR_NO_INT))
	{
		spin_lock(&sp->port.lock);
		snxdp_ser_handle_port(sp, iir);
		spin_unlock(&sp->port.lock);
	}

	timeout = sp->port.timeout;
	timeout = timeout > 6 ? (timeout / 2 - 2) : 1;

	mod_timer(&sp->timer, jiffies + timeout);
	//
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)data;
	struct sdp_ser_port *		port = &sp->port;
	int							retval = 0;

	sp->tx_idle_cnt++;
	if (sp->tx_idle_cnt > SDP_SER_TX_IDLE_CNT_MAX)
	{
		retval = ser_cmd_get_msr(sp);
		if (retval == 0)
		{
			if (sp->msr & NLK_TTY_MSR_TX_EMPTY)
			{
				//mutex_lock(&port->xmit_buff_mutex);
				snxdp_ser_transmit_chars(sp);
				//mutex_unlock(&port->xmit_buff_mutex);
			}
		}
		sp->tx_idle_cnt = 0;
	}
	mod_timer(&sp->timer, jiffies + SDP_SER_TIMEOUT);
	//---
}
*/

/*-----------------------------------------------------------------------------------------------------------------------------*/


static _INLINE_ void
snxdp_ser_receive_chars_for_data_pipe_normal(
	struct snxdp_ser_port *		sp
	)
{
	struct tty_struct *			tty = sp->port.info->tty;
	unsigned char				ch;
	unsigned int				dp_data_length = (sp->port.rx_data_length - 1);
	unsigned char				lsr = 0x00;
	char						flag;
	int							index = 1;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0))
	struct sdp_ser_state *		state = NULL;
    struct tty_port	*			tport = NULL;


    state = tty->driver_data;
	tport = &state->tport;
#endif


	do
	{
		//--- DP NLK
		//ch = READ_UART_RX(sp);
		ch = *(sp->port.rx_data_buff + index);
		lsr = UART_LSR_DR;
		flag = TTY_NORMAL;
		sp->port.icount.rx++;


		if ((I_IXOFF(tty)) || I_IXON(tty))
		{
			if (ch == START_CHAR(tty))
			{
				tty->stopped = 0;
				snxdp_ser_start_tx(&sp->port, 1);
				goto ignore_char1;
			}
			else if (ch == STOP_CHAR(tty))
			{
				tty->stopped = 1;
				snxdp_ser_stop_tx(&sp->port, 1);
				goto ignore_char1;
			}
		}

		sdp_ser_insert_char(&sp->port, lsr, UART_LSR_OE, ch, flag);

ignore_char1:
		index += 1;
		//--- DP NLK
		//lsr = READ_UART_LSR(sp);
		//---

	} while (--dp_data_length > 0);


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0))
	tty_flip_buffer_push(tport);
#else
	tty_flip_buffer_push(tty);
#endif
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static _INLINE_ void 
snxdp_ser_receive_chars_for_data_pipe_error(
	struct snxdp_ser_port *		sp
	)
{
	struct tty_struct *			tty = sp->port.info->tty;
	unsigned char				ch;
	int							max_count = 1024;
	unsigned char				dp_data_lsr = 0x00;
	unsigned char				lsr = 0x00;
	char						flag;
	int							index = 1;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0))
	struct sdp_ser_state *		state = NULL;
    struct tty_port	*			tport = NULL;


    state = tty->driver_data;
	tport = &state->tport;
#endif


	do
	{
		//--- DP NLK
		//ch = READ_UART_RX(sp);
		ch = *(sp->port.rx_data_buff + index);
		dp_data_lsr = *(sp->port.rx_data_buff + index + 1);
		if (!(dp_data_lsr & 0x08))
		{
			break;
		}

		lsr = 0x00;
		if (dp_data_lsr & 0x01)
		{
			lsr |= UART_LSR_FE;
		}

		if (dp_data_lsr & 0x02)
		{
			lsr |= UART_LSR_PE;
		}

		if (dp_data_lsr & 0x04)
		{
			lsr |= UART_LSR_BI;
		}

		if (dp_data_lsr & 0x08)
		{
			lsr |= UART_LSR_DR;
		}
		//---


		flag = TTY_NORMAL;
		sp->port.icount.rx++;

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,18))
		if (unlikely(lsr & (UART_LSR_BI | UART_LSR_PE | UART_LSR_FE | UART_LSR_OE)))
#else
		if (lsr & (UART_LSR_BI | UART_LSR_PE | UART_LSR_FE | UART_LSR_OE))
#endif
		{
			if (lsr & UART_LSR_BI)
			{
				lsr &= ~(UART_LSR_FE | UART_LSR_PE);
				sp->port.icount.brk++;

				if (sdp_ser_handle_break(&sp->port))
				{
					goto ignore_char2;
				}
			}
			else if (lsr & UART_LSR_PE)
			{
				sp->port.icount.parity++;
			}
			else if (lsr & UART_LSR_FE)
			{
				sp->port.icount.frame++;
			}

			if (lsr & UART_LSR_OE)
			{
				sp->port.icount.overrun++;
			}

			lsr &= sp->port.read_status_mask;


			if (lsr & UART_LSR_BI)
			{
				flag = TTY_BREAK;
			}
			else if (lsr & UART_LSR_PE)
			{
				flag = TTY_PARITY;
			}
			else if (lsr & UART_LSR_FE)
			{
				flag = TTY_FRAME;
			}
		}


		if ((I_IXOFF(tty)) || I_IXON(tty))
		{
			if (ch == START_CHAR(tty))
			{
				tty->stopped = 0;
				snxdp_ser_start_tx(&sp->port, 1);
				goto ignore_char2;
            }
			else if (ch == STOP_CHAR(tty))
			{
				tty->stopped = 1;
				snxdp_ser_stop_tx(&sp->port, 1);
				goto ignore_char2;
			}
		}

		sdp_ser_insert_char(&sp->port, lsr, UART_LSR_OE, ch, flag);

ignore_char2:
		index += 2;
		//--- DP NLK
		//lsr = READ_UART_LSR(sp);
		//---

	} while ((lsr & UART_LSR_DR) && (max_count-- > 0));


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0))
	tty_flip_buffer_push(tport);
#else
	tty_flip_buffer_push(tty);
#endif
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static _INLINE_ void 
snxdp_ser_receive_chars_for_event(
	struct snxdp_ser_port *		sp
	)
{
	struct tty_struct *			tty = sp->port.info->tty;
	unsigned char				ch;
	int							max_count = 1024;
	unsigned char				dp_data_lsr = 0x00;
	unsigned char				lsr = 0x00;
	char						flag;
	int							index = 0;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0))
	struct sdp_ser_state *		state = NULL;
    struct tty_port	*			tport = NULL;


    state = tty->driver_data;
	tport = &state->tport;
#endif


	do
	{
		//--- DP NLK
		//ch = READ_UART_RX(sp);
		ch = *(sp->port.rx_event_buff + index);
		dp_data_lsr = *(sp->port.rx_event_buff + index + 1);
		if (!(dp_data_lsr & 0x08))
		{
			break;
		}

		lsr = 0x00;
		if (dp_data_lsr & 0x01)
		{
			lsr |= UART_LSR_FE;
		}

		if (dp_data_lsr & 0x02)
		{
			lsr |= UART_LSR_PE;
		}

		if (dp_data_lsr & 0x04)
		{
			lsr |= UART_LSR_BI;
		}

		if (dp_data_lsr & 0x08)
		{
			lsr |= UART_LSR_DR;
		}
		//---


		flag = TTY_NORMAL;
		sp->port.icount.rx++;

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,18))
		if (unlikely(lsr & (UART_LSR_BI | UART_LSR_PE | UART_LSR_FE | UART_LSR_OE)))
#else
		if (lsr & (UART_LSR_BI | UART_LSR_PE | UART_LSR_FE | UART_LSR_OE))
#endif
		{
			if (lsr & UART_LSR_BI)
			{
				lsr &= ~(UART_LSR_FE | UART_LSR_PE);
				sp->port.icount.brk++;

				if (sdp_ser_handle_break(&sp->port))
				{
					goto ignore_char3;
				}
			}
			else if (lsr & UART_LSR_PE)
			{
				sp->port.icount.parity++;
			}
			else if (lsr & UART_LSR_FE)
			{
				sp->port.icount.frame++;
			}

			if (lsr & UART_LSR_OE)
			{
				sp->port.icount.overrun++;
			}

			lsr &= sp->port.read_status_mask;


			if (lsr & UART_LSR_BI)
			{
				flag = TTY_BREAK;
			}
			else if (lsr & UART_LSR_PE)
			{
				flag = TTY_PARITY;
			}
			else if (lsr & UART_LSR_FE)
			{
				flag = TTY_FRAME;
			}
		}


		if ((I_IXOFF(tty)) || I_IXON(tty))
		{
			if (ch == START_CHAR(tty))
			{
				tty->stopped = 0;
				snxdp_ser_start_tx(&sp->port, 1);
				goto ignore_char3;
            }
			else if (ch == STOP_CHAR(tty))
			{
				tty->stopped = 1;
				snxdp_ser_stop_tx(&sp->port, 1);
				goto ignore_char3;
			}
		}

		sdp_ser_insert_char(&sp->port, lsr, UART_LSR_OE, ch, flag);

ignore_char3:
		index += 2;
		//--- DP NLK
		//lsr = READ_UART_LSR(sp);
		//---

	} while ((lsr & UART_LSR_DR) && (max_count-- > 0));


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0))
	tty_flip_buffer_push(tport);
#else
	tty_flip_buffer_push(tty);
#endif
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static _INLINE_ void 
snxdp_ser_transmit_chars(
	struct snxdp_ser_port *		sp
	)
{
	struct sdp_ser_port *		port = &sp->port;
	struct circ_buf	*			xmit = &sp->port.info->xmit;
	int							count;
	int							retval = 0;


	do
	{
		if (!sp)
		{
			break;
		}

		if (!sp->port.info)
		{
			break;
		}

		if (!xmit)
		{
			break;
		}


		if (sp->port.x_char)
		{
			//--- DP NLK
			//WRITE_UART_TX(sp, sp->port.x_char);
			memset(port->tx_data_buff, 0, NLK_DATA_MAX_LENGTH);
			memcpy(port->tx_data_buff, &sp->port.x_char, 1);
			port->tx_data_length = 1;
			if (port->tx_data_length > 0)
			{
				port->wpt += 1;
				sp->msr &= ~0x10;
				retval = ser_data_tx(sp);
				if (retval < 0)
				{

				}
			}
			//---
			sp->port.icount.tx++;
			sp->port.x_char = 0;
			break;
		}

		if (sdp_ser_circ_empty(xmit) || sdp_ser_tx_stopped(&sp->port)) 
		{
			snxdp_ser_stop_tx(&sp->port, 0);
			break;
		}

		//--- DP NLK
		//count = sp->port.fifosize;
		count = 1024 - ((((port->wpt & 0x1ff) - port->rpt) & 0x1ff) << 2);
		if (count > 1020)
		{
			count = 1020;
		}

		memset(port->tx_data_buff, 0, NLK_DATA_MAX_LENGTH);
		port->tx_data_length = 0;
		if (count > 0)
		{
			do
			{
				//WRITE_UART_TX(sp, xmit->buf[xmit->tail]);
				port->tx_data_buff[port->tx_data_length] = xmit->buf[xmit->tail];
				port->tx_data_length++;

				xmit->tail = (xmit->tail + 1) & (SDP_UART_XMIT_SIZE - 1);
				sp->port.icount.tx++;

				if (sdp_ser_circ_empty(xmit))
				{
					break;
				}

			} while (--count > 0);
		}

		if (port->tx_data_length > 0)
		{
			if ((port->tx_data_length % 4) == 0)
			{
				port->wpt += (port->tx_data_length / 4);
			}
			else
			{
				port->wpt += (port->tx_data_length / 4) + 1;
			}

			sp->msr &= ~0x10;
			retval = ser_data_tx(sp);
			if (retval < 0)
			{

			}
		}
		else
		{

		}
		//---

		if (sdp_ser_circ_chars_pending(xmit) < WAKEUP_CHARS)
		{
			sdp_ser_write_wakeup(&sp->port);
		}

		if (sdp_ser_circ_empty(xmit))
		{
			snxdp_ser_stop_tx(&sp->port, 0);
		}

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static _INLINE_ void 
snxdp_ser_check_modem_status(
	struct snxdp_ser_port *		sp,
	unsigned char 				status
	)
{
	do
	{
		if ((status & UART_MSR_ANY_DELTA) == 0)
		{
			break;
		}

		if (!sp->port.info)
		{
			break;
		}

		if (status & UART_MSR_TERI)
		{
			sp->port.icount.rng++;
		}

		if (status & UART_MSR_DDSR)
		{
			sp->port.icount.dsr++;
		}

		if (status & UART_MSR_DDCD)
		{
			sdp_ser_handle_dcd_change(&sp->port, status & UART_MSR_DCD);
		}

		if (status & UART_MSR_DCTS)
		{
			sdp_ser_handle_cts_change(&sp->port, status & UART_MSR_CTS);
		}

		wake_up_interruptible(&sp->port.info->delta_msr_wait);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


//--- DP NLK
/*
void
snxdp_ser_handle_port(
	struct snxdp_ser_port *		sp,
	unsigned char				iir
	)
{
	unsigned char				lsr = READ_UART_LSR(sp);
	unsigned char				msr = 0;


	spin_lock(&sp->port.lock);

	if ((iir == UART_IIR_RLSI) || (iir == UART_IIR_CTO) || (iir == UART_IIR_RDI))
	{
		snxdp_ser_receive_chars(sp, &lsr);
	}

	if ((iir == UART_IIR_THRI) && (lsr & UART_LSR_THRE))
	{
		snxdp_ser_transmit_chars(sp);
	}


	msr = READ_UART_MSR(sp);

	if (msr & UART_MSR_ANY_DELTA)
	{
		snxdp_ser_check_modem_status(sp, msr);
	}

	spin_unlock(&sp->port.lock);
}
*/
void
snxdp_ser_handle_port(
	struct snxdp_ser_port *		sp
	)
{
	struct sdp_ser_port *		port = &sp->port;
	unsigned char				dp_event_lsr = 0x00;
	unsigned char				dp_event_msr = 0x00;
	unsigned char				dp_msr = 0x00;
	unsigned char				msr = 0;
	unsigned short				rpt = 0;
	unsigned char				data_flag = 0x00;


	if (sp->port.rx_data_length > 0)
	{
		data_flag = *(sp->port.rx_data_buff + 0);
		//printk("SDDVIRTUAL : (M:%d) DATA_SERVICE_TO_KERNEL, data_flag:x%02x, rx_data_length:%d\n", sp->port.line, data_flag, sp->port.rx_data_length);

		if (data_flag == 0x00)
		{
			snxdp_ser_receive_chars_for_data_pipe_normal(sp);
		}
		else if (data_flag == 0x01)
		{
			snxdp_ser_receive_chars_for_data_pipe_error(sp);
		}
		else
		{
			printk("SDDVIRTUAL : (M:%d) DATA_SERVICE_TO_KERNEL, unknow data_flag:x%02x\n", sp->port.line, data_flag);
		}

		memset(sp->port.rx_data_buff, 0, NLK_MSG_MAX_PAYLOAD);
		sp->port.rx_data_length = 0;
	}


	if (sp->port.rx_event_type == NLK_TTY_EVENT_DATA_SERVICE_TO_KERNEL)
	{
		//printk("SDDVIRTUAL : (M:%d) EVENT_DATA_SERVICE_TO_KERNEL, rx_event_length:%d\n", sp->port.line, sp->port.rx_event_length);
		snxdp_ser_receive_chars_for_event(sp);
	}


	if (sp->port.rx_event_type == NLK_TTY_EVENT_LSR)
	{
		dp_event_lsr = *(sp->port.rx_event_buff + 0);
		//printk("SDDVIRTUAL : (M:%d) EVENT_LSR, LSR:x%02x\n", sp->port.line, dp_event_lsr);

		if (dp_event_lsr & (NLK_TTY_EVENT_LSR_TX_TRIGGER | NLK_TTY_EVENT_LSR_TX_EMPTY))
		{
			mutex_lock(&port->xmit_buff_mutex);
			if (sp->uart_version <= 0x00)
			{
				if (dp_event_lsr & NLK_TTY_EVENT_LSR_TX_EMPTY)
				{
					rpt = port->wpt;
				}
				else if (dp_event_lsr & NLK_TTY_EVENT_LSR_TX_TRIGGER)
				{
					rpt = port->wpt - 32;
				}
				else
				{
					rpt = port->wpt - 32;
				}
			}
			else
			{
				rpt = (unsigned short)((*(sp->port.rx_event_buff + 1) << 8) | (*(sp->port.rx_event_buff + 2)));
			}
			port->rpt = rpt;
			sp->msr |= 0x10;

			snxdp_ser_transmit_chars(sp);
			mutex_unlock(&port->xmit_buff_mutex);
		}

		if (dp_event_lsr & NLK_TTY_EVENT_LSR_RX_ERROR)
		{
			printk("SDDVIRTUAL : (M:%d) EVENT_LSR, rx error event\n", sp->port.line);
		}

		if (dp_event_lsr & NLK_TTY_EVENT_LSR_RX_OVER_FLOW)
		{
			printk("SDDVIRTUAL : (M:%d) EVENT_LSR, rx over flow event\n", sp->port.line);
		}
	}


	if (sp->port.rx_event_type == NLK_TTY_EVENT_MSR)
	{
		dp_event_msr = *(sp->port.rx_event_buff + 0);
		//printk("SDDVIRTUAL : (M:%d) EVENT_MSR, MSR:x%02x\n", sp->port.line, dp_event_msr);

		if (dp_event_msr & NLK_TTY_EVENT_MSR_CTS)
		{
			msr |= UART_MSR_CTS;
			dp_msr |= 0x01;
			if (!(sp->msr & 0x01))
			{
				msr |= UART_MSR_DCTS;
			}
		}
		else
		{
			msr &= ~UART_MSR_CTS;
			dp_msr &= ~0x01;
			if (sp->msr & 0x01)
			{
				msr |= UART_MSR_DCTS;
			}
		}


		if (dp_event_msr & NLK_TTY_EVENT_MSR_DSR)
		{
			msr |= UART_MSR_DSR;
			dp_msr |= 0x02;
			if (!(sp->msr & 0x02))
			{
				msr |= UART_MSR_DDSR;
			}
		}
		else
		{
			msr &= ~UART_MSR_DSR;
			dp_msr &= ~0x02;
			if (sp->msr & 0x02)
			{
				msr |= UART_MSR_DDSR;
			}
		}


		if (dp_event_msr & NLK_TTY_EVENT_MSR_RI)
		{
			msr |= UART_MSR_RI;
			dp_msr |= 0x04;
			if (!(sp->msr & 0x04))
			{
				msr |= UART_MSR_TERI;
			}
		}
		else
		{
			msr &= ~UART_MSR_RI;
			dp_msr &= ~0x04;
			if (sp->msr & 0x04)
			{
				msr |= UART_MSR_TERI;
			}
		}


		if (dp_event_msr & NLK_TTY_EVENT_MSR_DCD)
		{
			msr |= UART_MSR_DCD;
			dp_msr |= 0x08;
			if (!(sp->msr & 0x08))
			{
				msr |= UART_MSR_DDCD;
			}
		}
		else
		{
			msr &= ~UART_MSR_DCD;
			dp_msr &= ~0x08;
			if (sp->msr & 0x08)
			{
				msr |= UART_MSR_DDCD;
			}
		}

		sp->msr = (sp->msr & 0xf0) | dp_msr;
		//printk("SDDVIRTUAL : (M:%d) EVENT_MSR, sp->msr:x%02x, msr:x%02x\n", sp->port.line, sp->msr, msr);
		snxdp_ser_check_modem_status(sp, msr);
	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_ser_request_io(
	struct sdp_ser_port	*		port
	)
{
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;


	switch (sp->port.iotype)
	{
		case SDP_UPIO_PORT :
			break;

		default:
			break;
	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_ser_release_io(
	struct sdp_ser_port *		port
	)
{
	struct snxdp_ser_port *		sp = (struct snxdp_ser_port *)port;


	switch (sp->port.iotype)
	{
		case SDP_UPIO_PORT :
			break;

		default:
			break;
	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_ser_configure_port(
	struct sdp_ser_driver *		drv,
	struct sdp_ser_state *		state,
	struct sdp_ser_port *		port
	)
{
	unsigned long				flags;


	//--- no iobase for DP TTY
	/*
	if (!port->iobase)
	{
		return;
	}
	*/
	//---

	flags = SDP_UART_CONFIG_TYPE;

	if (port->type != PORT_UNKNOWN)
	{
		snxdp_ser_request_io(port);

		spin_lock_irqsave(&port->lock, flags);

		//--- DP NLK
		//snxdp_ser_set_mctrl(port, 0);
		//---
		spin_unlock_irqrestore(&port->lock, flags);

		//--- DP NLK
		port->opened = false;
		port->rx_response_buff = NULL;
		port->rx_response_length = 0;
		port->rx_event_buff = NULL;
		port->rx_event_length = 0;
		port->rx_event_type = 0;

		init_completion(&port->open_comp);
		init_completion(&port->close_comp);
#if (DRIVERTARGET == 1)
		init_completion(&port->set_dll_dlm_comp);
		init_completion(&port->set_lcr_comp);
		init_completion(&port->set_fifo_ctrl_comp);
		init_completion(&port->set_mcr_comp);
		init_completion(&port->set_event_ctrl_comp);
		init_completion(&port->set_on_chip_hw_flow_ctrl_comp);
		init_completion(&port->set_tx_trigger_level_comp);
		init_completion(&port->set_rx_trigger_level_comp);
#endif
		init_completion(&port->get_msr_comp);
		//---
    }
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
snxdp_ser_unconfigure_port(
	struct sdp_ser_driver *		drv,
	struct sdp_ser_state *		state
	)
{
	struct sdp_ser_port	*		port = state->port;
	struct sdp_ser_info	*		info = state->info;   


	if (info && info->tty)
	{
		tty_hangup(info->tty);
	}

	down(&state->sem);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
	tty_unregister_device(drv->tty_driver, port->line);
#endif

	state->info = NULL;

	if (port->type != PORT_UNKNOWN)
	{
		//--- DP NLK
		if (port->rx_response_buff != NULL)
		{
			kfree(port->rx_response_buff);
			port->rx_response_buff = NULL;
		}

		if (port->rx_event_buff != NULL)
		{
			kfree(port->rx_event_buff);
			port->rx_event_buff = NULL;
		}
		//---

		snxdp_ser_release_io(port);
	}

	port->type = PORT_UNKNOWN;

	if (info)
	{
		tasklet_kill(&info->tlet);
		kfree(info);
	}

	up(&state->sem);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
snxdp_ser_add_one_port(
	struct sdp_ser_driver *		drv,
	struct sdp_ser_port *		port
	)
{
	struct sdp_ser_state *		state = NULL;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
	struct tty_port	*			tport;
	struct device *				tty_dev;
#endif
	int							ret = 0;


	if (port->line >= drv->nr)
	{
		return -EINVAL;
	}

	state = drv->state + port->line;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
	tport = &state->tport;
#endif

	down(&ser_port_sem);

	if (state->port)
	{
		ret = -EINVAL;
		goto out;
	}

	state->port = port;

	port->info = state->info;

	snxdp_ser_configure_port(drv, state, port);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
	tty_dev = tty_port_register_device(tport, drv->tty_driver, port->line, port->dev);

	if (likely(!IS_ERR(tty_dev)))
	{
		device_set_wakeup_capable(tty_dev, 1);
	}
	else
	{
		printk("SDDVIRTUAL : (M:%d) %s, Cannot register tty device.\n", port->line, __FUNCTION__);
	}
#endif

out:
	up(&ser_port_sem);

	return ret;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int 
snxdp_ser_remove_one_port(
	struct sdp_ser_driver *		drv,
	struct sdp_ser_port	*		port
	)
{
	struct sdp_ser_state *		state = drv->state + port->line;


	if (state->port != port)
	{
		printk("SDDVIRTUAL : (M:%d) %s, Removing wrong port: %p != %p\n\n", port->line, __FUNCTION__, state->port, port);
	}

	down(&ser_port_sem);

	snxdp_ser_unconfigure_port(drv, state);

	state->port = NULL;

	up(&ser_port_sem);

	return 0;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
snxdp_ser_register_ports(
	struct sdp_ser_driver *		drv
	)
{
	int							i;
	int							ret;


	for (i = 0; i < SDP_SER_TOTAL_MAX; i++) 
	{
		struct snxdp_ser_port *sp = &snxdp_ser_table[i];

		if (!sp)
		{
			return -1;
		}


		//--- no iobase for DP TTY
		//if (sp->port.iobase)
		//---
		{
			//--- no timer for DP TTY
			/*
			init_timer(&sp->timer);
			sp->timer.function = snxdp_ser_timeout;
			sp->timer.data = ((unsigned long) sp);
			sp->timer.expires = jiffies + SDP_SER_TIMEOUT;
			*/
			//---
			sp->mcr_mask = ~0;
			sp->mcr_force = 0;

			ret = snxdp_ser_add_one_port(drv, &sp->port);

			if (ret != 0)
			{
				return ret;
			}
		}
	}

	return 0;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void 
snxdp_ser_unregister_ports(
	struct sdp_ser_driver *		drv
	)
{
	int							i;


	for (i = 0; i < SDP_SER_TOTAL_MAX; i++) 
	{
		struct snxdp_ser_port *sp = &snxdp_ser_table[i];

		//--- no iobase for DP TTY
		//if (sp->port.iobase)
		//---
		{
			snxdp_ser_remove_one_port(drv, &sp->port);
		}
	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void 
snxdp_ser_initiate_ports(
	struct sdp_ser_driver *		drv
	)
{
	int							i;


	for (i = 0; i < SDP_SER_TOTAL_MAX; i++) 
	{
		struct snxdp_ser_port *sp = &snxdp_ser_table[i];

		if (sp)
		{
			sp->port.baud_base = 921600;
			sp->port.line = i;
			sp->port.uartclk = sp->port.baud_base * 16;
			sp->port.iotype = SDP_UPIO_PORT;
			sp->port.flags = ASYNC_SHARE_IRQ;
			sp->port.ldisc_stop_rx = 0;

			spin_lock_init(&sp->port.lock);

			sp->port.type = PORT_SER_16750;
			sp->port.fifosize = SUNIXDP_FIFOSIZE;
			sp->port.rx_trigger = SUNIXDP_TRIGGER_LEVEL;

			sp->port.setserial_flag = SDP_SER_BAUD_NOTSETSER;
		}
	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------------------------------------*/


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))  
static struct tty_operations snxdp_tty_ops = 
{
	.open					= sdp_ser_open,
	.close					= sdp_ser_close,
	.write					= sdp_ser_write,
	.put_char				= sdp_ser_put_char,
	.flush_chars			= sdp_ser_flush_chars,
	.write_room				= sdp_ser_write_room,
	.chars_in_buffer		= sdp_ser_chars_in_buffer,
	.flush_buffer			= sdp_ser_flush_buffer,
	.ioctl					= sdp_ser_ioctl,
	.throttle				= sdp_ser_throttle,
	.unthrottle				= sdp_ser_unthrottle,
	.send_xchar				= sdp_ser_send_xchar,
	.set_termios			= sdp_ser_set_termios,
	.stop					= sdp_ser_stop,
	.start					= sdp_ser_start,
	.hangup					= sdp_ser_hangup,
	.break_ctl				= sdp_ser_break_ctl,
	.wait_until_sent		= sdp_ser_wait_until_sent,  
	.tiocmget				= sdp_ser_tiocmget,
	.tiocmset				= sdp_ser_tiocmset,
};
#endif 


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
snxdp_ser_register_driver(
	struct sdp_ser_driver *		drv
	)
{
	struct tty_driver *			normal = NULL;
	int							i;
	int							ret = 0;


	drv->state = kmalloc(sizeof(struct sdp_ser_state) * drv->nr, GFP_KERNEL);
	ret = -ENOMEM;

	if (!drv->state)
	{
		printk("SDDVIRTUAL : Allocate memory fail !\n\n");
		goto out;
	}

	memset(drv->state, 0, sizeof(struct sdp_ser_state) * drv->nr);

	for (i = 0; i < drv->nr; i++) 
	{
		struct sdp_ser_state *state = drv->state + i;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
		struct tty_port *tport = &state->tport;
		tty_port_init(tport);
#endif
		if (!state)
		{
			ret = -1;
			printk("SDDVIRTUAL : Memory error !\n\n");
			goto out;
		}

		state->close_delay = 5 * HZ / 100;
		state->closing_wait = 3 * HZ;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,37))
		sema_init(&state->sem,1);
#else
		init_MUTEX(&state->sem);
#endif
	}


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
	normal = tty_alloc_driver(SDP_SER_TOTAL_MAX + 1 , TTY_DRIVER_REAL_RAW | TTY_DRIVER_DYNAMIC_DEV);
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
	normal = alloc_tty_driver(drv->nr);
#else
	normal = &drv->tty_driver;
#endif


	if (!normal)
	{
		printk("SDDVIRTUAL : Allocate tty driver fail !\n\n");
		goto out;
	}


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))

#else
	memset(normal, 0, sizeof(struct tty_driver));
#endif

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
	drv->tty_driver = normal;
#endif

	normal->magic					= TTY_DRIVER_MAGIC;
	normal->name					= drv->dev_name;
	normal->major					= drv->major;
	normal->minor_start				= drv->minor;
	normal->num						= (SDP_SER_TOTAL_MAX + 1);
	normal->type					= TTY_DRIVER_TYPE_SERIAL;
	normal->subtype					= SERIAL_TYPE_NORMAL;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))

#else
	normal->flags					= TTY_DRIVER_REAL_RAW;
#endif
	normal->init_termios			= tty_std_termios;
	normal->init_termios.c_cflag	= B9600 | CS8 | CREAD | HUPCL | CLOCAL;
	normal->init_termios.c_iflag	= 0;
	normal->driver_state			= drv;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
	tty_set_operations(normal, &snxdp_tty_ops);
#else
	normal->refcount				= &snxdp_ser_refcount;
	normal->table					= snxdp_ser_tty;

	normal->termios					= snxdp_ser_termios;
	normal->termios_locked			= snxdp_ser_termios_locked;

	normal->open					= sdp_ser_open;
	normal->close					= sdp_ser_close;
	normal->write					= sdp_ser_write;
	normal->put_char				= sdp_ser_put_char;
	normal->flush_chars				= sdp_ser_flush_chars;
	normal->write_room				= sdp_ser_write_room;
	normal->chars_in_buffer			= sdp_ser_chars_in_buffer;
	normal->flush_buffer			= sdp_ser_flush_buffer;
	normal->ioctl					= sdp_ser_ioctl;
	normal->throttle				= sdp_ser_throttle;
	normal->unthrottle				= sdp_ser_unthrottle;
	normal->send_xchar				= sdp_ser_send_xchar;
	normal->set_termios				= sdp_ser_set_termios;
	normal->stop					= sdp_ser_stop;
	normal->start					= sdp_ser_start;
	normal->hangup					= sdp_ser_hangup;
	normal->break_ctl				= sdp_ser_break_ctl;
	normal->wait_until_sent			= sdp_ser_wait_until_sent;
#endif


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,7,0))
	tty_port_link_device(&sdp_service_port, normal, SDP_SER_TOTAL_MAX);
#endif

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28))
	kref_init(&normal->kref);
#endif

	ret = tty_register_driver(normal);

	if (ret < 0)
	{
		printk("SDDVIRTUAL : Register tty driver fail !\n\n");
		goto out;
	}
    
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,8,0))
	for (i = 0; i < drv->nr; i++)
	{
		struct sdp_ser_state *state = drv->state + i;
		struct tty_port *tport = &state->tport;

		tty_port_destroy(tport);
	}
#endif


out:
	if (ret < 0)
	{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
		put_tty_driver(normal);
#endif
		kfree(drv->state);
	}

	return ret;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void 
snxdp_ser_unregister_driver(
	struct sdp_ser_driver *		drv
	)
{
	struct tty_driver *			normal = NULL;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,8,0))
	unsigned int				i;
#endif    


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
	normal = drv->tty_driver;

	if (!normal)
	{
		return;
	}

	tty_unregister_driver(normal);
	put_tty_driver(normal);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,8,0))
    for (i = 0; i < drv->nr; i++)
    {
		struct sdp_ser_state *state = drv->state + i;
		struct tty_port *tport = &state->tport;

		tty_port_destroy(tport);
	}
#endif

	drv->tty_driver = NULL;

#else
	normal = &drv->tty_driver;
	if (!normal)
	{
		return;
	}

	tty_unregister_driver(normal);
#endif

	if (drv->state)
	{
		kfree(drv->state);
	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


