/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
ser_open(
	struct tty_struct *			tty,
	struct file *				filp
	)
{
	int							line = SDP_SER_DEVNUM(tty);
	struct sdp_ser_port *		port = &snxdp_ser_table[line].port;
	struct snxdp_ser_port *		sp = &snxdp_ser_table[line];
	int							retval = 0;
	unsigned char				data[256];
	int							length = 0;
	PNLK_HEADER					h_ptr = NULL;


	do
	{
		if (port->opened == true)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Port opened.\n", line, __FUNCTION__);
			retval = -EBUSY;
			break;
		}


		//--- alloc resource
		port->rx_response_buff = (unsigned char *)kmalloc(NLK_MSG_MAX_PAYLOAD, GFP_KERNEL);
		if (port->rx_response_buff == NULL)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Alloc rx response buff fail.\n", line, __FUNCTION__);
			retval = -ENOMEM;
			break;
		}
		memset(port->rx_response_buff, 0, NLK_MSG_MAX_PAYLOAD);
		port->rx_response_length = 0;

		port->rx_event_buff = (unsigned char *)kmalloc(NLK_MSG_MAX_PAYLOAD, GFP_KERNEL);
		if (port->rx_event_buff == NULL)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Alloc rx event buff fail.\n", line, __FUNCTION__);
			retval = -ENOMEM;
			break;
		}
		memset(port->rx_event_buff, 0, NLK_MSG_MAX_PAYLOAD);
		port->rx_event_length = 0;
		port->rx_event_type = 0;

		port->rx_data_buff = (unsigned char *)kmalloc(NLK_MSG_MAX_PAYLOAD, GFP_KERNEL);
		if (port->rx_data_buff == NULL)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Alloc rx data buff fail.\n", line, __FUNCTION__);
			retval = -ENOMEM;
			break;
		}
		memset(port->rx_data_buff, 0, NLK_MSG_MAX_PAYLOAD);
		port->rx_data_length = 0;

		port->tx_data_buff = (unsigned char *)kmalloc(NLK_DATA_MAX_LENGTH, GFP_KERNEL);
		if (port->tx_data_buff == NULL)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Alloc tx data buff fail.\n", line, __FUNCTION__);
			retval = -ENOMEM;
			break;
		}
		memset(port->tx_data_buff, 0, NLK_DATA_MAX_LENGTH);
		port->tx_data_length = 0;

		port->tx_data_msg = (unsigned char *)kmalloc(NLK_MSG_MAX_PAYLOAD, GFP_KERNEL);
		if (port->tx_data_msg == NULL)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Alloc tx data message fail.\n", line, __FUNCTION__);
			retval = -ENOMEM;
			break;
		}
		memset(port->tx_data_msg, 0, NLK_MSG_MAX_PAYLOAD);

		mutex_init(&port->xmit_buff_mutex);
		port->wpt = 0;
		port->rpt = 0;
		//---


		memset(data, 0, sizeof(data));
		h_ptr = (PNLK_HEADER)data;
		h_ptr->Version = NLK_PROTOCOL_VERSION;
		h_ptr->DeviceType = NLK_DEVICE_TYPE_TTY;
		h_ptr->MinorNumber = (unsigned char)line;
		h_ptr->CmdResponseEventData = NLK_TTY_CMD_OPEN;
		h_ptr->ResponseStatus = 0;
		h_ptr->Length = 0;
		length = sizeof(NLK_HEADER);

		retval = snxdp_nlk_tx(data, length);
		if (retval < 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, NLK tx fail(retval:%d).\n", line, __FUNCTION__, retval);
			break;
		}


		retval = wait_for_completion_timeout(&port->open_comp, msecs_to_jiffies(1000));
		if (retval == 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Wait command complete timeout.\n", line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (port->rx_response_length < sizeof(NLK_HEADER))
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response length(%d) too small.\n", line, __FUNCTION__, port->rx_response_length);
			retval = -ENODEV;
			break;
		}

		h_ptr = (PNLK_HEADER)port->rx_response_buff;

		if (h_ptr->ResponseStatus != NLK_STATUS_SUCCESS)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response status failure.\n", line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}

		if (h_ptr->Length != 17)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response data length(%d) invalid.\n", line, __FUNCTION__, h_ptr->Length);
			retval = -ENODEV;
			break;
		}

		sp->uart_version = *(port->rx_response_buff + sizeof(NLK_HEADER) + 0);
		sp->dll = *(port->rx_response_buff + sizeof(NLK_HEADER) + 1);
		sp->dlm = *(port->rx_response_buff + sizeof(NLK_HEADER) + 2);
		sp->lcr = *(port->rx_response_buff + sizeof(NLK_HEADER) + 3);
		sp->fifo_ctrl = *(port->rx_response_buff + sizeof(NLK_HEADER) + 4);
		sp->mcr = *(port->rx_response_buff + sizeof(NLK_HEADER) + 5);
		sp->event_ctrl = *(port->rx_response_buff + sizeof(NLK_HEADER) + 6);
		sp->on_chip_hw_flow_ctrl = *(port->rx_response_buff + sizeof(NLK_HEADER) + 7);
		sp->msr = *(port->rx_response_buff + sizeof(NLK_HEADER) + 8);
		sp->tx_current_credit  = (unsigned short)(*(port->rx_response_buff + sizeof(NLK_HEADER) + 9) << 8);
		sp->tx_current_credit |= (unsigned short)(*(port->rx_response_buff + sizeof(NLK_HEADER) + 10));
		sp->tx_fifo_data_cnt  = (unsigned short)(*(port->rx_response_buff + sizeof(NLK_HEADER) + 11) << 8);
		sp->tx_fifo_data_cnt |= (unsigned short)(*(port->rx_response_buff + sizeof(NLK_HEADER) + 12));
		sp->tx_trigger_level = *(port->rx_response_buff + sizeof(NLK_HEADER) + 13);
		sp->rx_trigger_level = *(port->rx_response_buff + sizeof(NLK_HEADER) + 14);
		sp->mode_1 = *(port->rx_response_buff + sizeof(NLK_HEADER) + 15);
		sp->mode_2 = *(port->rx_response_buff + sizeof(NLK_HEADER) + 16);


		// debug
		/*
		printk("SDDVIRTUAL : ----------------------------------\n");
		printk("SDDVIRTUAL : (M:%d) %s, VERSION          :x%02x\n", line, __FUNCTION__, sp->uart_version);
		printk("SDDVIRTUAL : (M:%d) %s, DLL              :x%02x\n", line, __FUNCTION__, sp->dll);
		printk("SDDVIRTUAL : (M:%d) %s, DLM              :x%02x\n", line, __FUNCTION__, sp->dlm);
		printk("SDDVIRTUAL : (M:%d) %s, LCR              :x%02x\n", line, __FUNCTION__, sp->lcr);
		printk("SDDVIRTUAL : (M:%d) %s, FIFO_CTRL        :x%02x\n", line, __FUNCTION__, sp->fifo_ctrl);
		printk("SDDVIRTUAL : (M:%d) %s, MCR              :x%02x\n", line, __FUNCTION__, sp->mcr);
		printk("SDDVIRTUAL : (M:%d) %s, EVENT_CTRL       :x%02x\n", line, __FUNCTION__, sp->event_ctrl);
		printk("SDDVIRTUAL : (M:%d) %s, OnChipHWFlowCtrl :x%02x\n", line, __FUNCTION__, sp->on_chip_hw_flow_ctrl);
		printk("SDDVIRTUAL : (M:%d) %s, MSR              :x%02x\n", line, __FUNCTION__, sp->msr);
		printk("SDDVIRTUAL : (M:%d) %s, TxCurrentCredit  :x%04x\n", line, __FUNCTION__, sp->tx_current_credit);
		printk("SDDVIRTUAL : (M:%d) %s, TxFifoDataCnt    :x%04x\n", line, __FUNCTION__, sp->tx_fifo_data_cnt);
		printk("SDDVIRTUAL : (M:%d) %s, TxTriggerLevel   :x%02x\n", line, __FUNCTION__, sp->tx_trigger_level);
		printk("SDDVIRTUAL : (M:%d) %s, RxTriggerLevel   :x%02x\n", line, __FUNCTION__, sp->rx_trigger_level);
		printk("SDDVIRTUAL : (M:%d) %s, MODE_1           :x%02x\n", line, __FUNCTION__, sp->mode_1);
		printk("SDDVIRTUAL : (M:%d) %s, MODE_2           :x%02x\n", line, __FUNCTION__, sp->mode_2);
		printk("SDDVIRTUAL : ----------------------------------\n");
		*/


		port->opened = true;

	} while (false);

	if (port->opened != true)
	{
		if (port->rx_response_buff != NULL)
		{
			kfree(port->rx_response_buff);
			port->rx_response_buff = NULL;
		}

		if (port->rx_event_buff != NULL)
		{
			kfree(port->rx_event_buff);
			port->rx_event_buff = NULL;
		}

		if (port->rx_data_buff != NULL)
		{
			kfree(port->rx_data_buff);
			port->rx_data_buff = NULL;
		}

		if (port->tx_data_buff != NULL)
		{
			kfree(port->tx_data_buff);
			port->tx_data_buff = NULL;
		}

		if (port->tx_data_msg != NULL)
		{
			kfree(port->tx_data_msg);
			port->tx_data_msg = NULL;
		}
	}

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


int 
ser_close(
	struct tty_struct *			tty,
	struct file *				filp
	)
{
	int							line = SDP_SER_DEVNUM(tty);
	struct sdp_ser_port *		port = &snxdp_ser_table[line].port;
	int							retval = 0;
	unsigned char				data[256];
	int							length = 0;
	PNLK_HEADER					h_ptr = NULL;
	

	do
	{
		if (port->opened == false)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Port not opened.\n", line, __FUNCTION__);
			retval = -ENODEV;
			break;
		}


		memset(data, 0, sizeof(data));
		h_ptr = (PNLK_HEADER)data;
		h_ptr->Version = NLK_PROTOCOL_VERSION;
		h_ptr->DeviceType = NLK_DEVICE_TYPE_TTY;
		h_ptr->MinorNumber = (unsigned char)line;
		h_ptr->CmdResponseEventData = NLK_TTY_CMD_CLOSE;
		h_ptr->ResponseStatus = 0;
		h_ptr->Length = 0;
		length = sizeof(NLK_HEADER);

		retval = snxdp_nlk_tx(data, length);
		if (retval < 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, NLK tx fail(retval:%d).\n", line, __FUNCTION__, retval);
			break;
		}


		retval = wait_for_completion_timeout(&port->close_comp, msecs_to_jiffies(1000));
		if (retval == 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Wait command complete timeout.\n", line, __FUNCTION__);
			retval = -ENODEV;
		}

		if (port->rx_response_length < sizeof(NLK_HEADER))
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response length(%d) too small.\n", line, __FUNCTION__, port->rx_response_length);
			retval = -ENODEV;
		}

		h_ptr = (PNLK_HEADER)port->rx_response_buff;

		if (h_ptr->ResponseStatus != NLK_STATUS_SUCCESS)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response status failure.\n", line, __FUNCTION__);
			retval = -ENODEV;
		}

		if (h_ptr->Length != 0)
		{
			printk("SDDVIRTUAL : (M:%d) %s, Response data length(%d) invalid.\n", line, __FUNCTION__, h_ptr->Length);
			retval = -ENODEV;
		}


		//--- release resource
		if (port->rx_response_buff != NULL)
		{
			kfree(port->rx_response_buff);
			port->rx_response_buff = NULL;
		}

		if (port->rx_event_buff != NULL)
		{
			kfree(port->rx_event_buff);
			port->rx_event_buff = NULL;
		}

		if (port->rx_data_buff != NULL)
		{
			kfree(port->rx_data_buff);
			port->rx_data_buff = NULL;
		}

		if (port->tx_data_buff != NULL)
		{
			kfree(port->tx_data_buff);
			port->tx_data_buff = NULL;
		}

		if (port->tx_data_msg != NULL)
		{
			kfree(port->tx_data_msg);
			port->tx_data_msg = NULL;
		}
		//---


		port->opened = false;

	} while (false);

	return retval;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


