/*
 *
 *                SUNIX DevicePort Virtual Driver
 *
 *
 *
 *	Driver for SUNIX DevicePort virtual driver
 *	Based on drivers/char/serial.c by Linus Torvalds, Theodore Ts'o.
 *
 * 	This program is free software; you can redistribute it and/or modify
 * 	it under the terms of the GNU General Public License as published by
 * 	the Free Software Foundation; either version 2 of the License, or
 * 	(at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 	GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program; if not, write to the Free Software
 * 	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


struct snxdp_ser_port			snxdp_ser_table[SDP_SER_TOTAL_MAX];


static struct sdp_ser_driver snxdp_ser_reg =
{
	.dev_name				= "ttyDP",
	.major					= NLK_TTY_MAJOR,
	.minor					= 0,
	.nr						= (SDP_SER_TOTAL_MAX + 1),
};


/*-----------------------------------------------------------------------------------------------------------------------------*/


static int __init
sddvirtual_init(
	void
	)
{
	int							nStatus = 0;
	bool						bStep_1 = false;
	bool						bStep_2 = false;
	bool						bStep_3 = false;


	printk("SDDVIRTUAL : TTY Version - %s\n", NLK_TTY_DRIVER_VERSION);
	printk("SDDVIRTUAL : BUILDNUMBER - %d\n", DRIVERBUILDNUM);
	do
	{
		nStatus = snxdp_ser_register_driver(&snxdp_ser_reg);
		if (nStatus != 0)
		{
			break;
		}
		bStep_1 = true;


		snxdp_ser_initiate_ports(&snxdp_ser_reg);


		nStatus = snxdp_ser_register_ports(&snxdp_ser_reg);
		if (nStatus != 0)
		{
			break;
		}
		bStep_2 = true;


		nStatus = snxdp_nlk_init();
		if (nStatus != 0)
		{
			break;
		}
		bStep_3 = true;

	} while (false);


	if (nStatus != 0)
	{
		if (bStep_3 == true)
		{
			snxdp_nlk_exit();
		}

		if (bStep_2 == true)
		{
			snxdp_ser_unregister_ports(&snxdp_ser_reg);
		}

		if (bStep_1 == true)
		{
			snxdp_ser_unregister_driver(&snxdp_ser_reg);
		}
	}
	else
	{

	}
	
	return nStatus;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void __exit
sddvirtual_exit(
	void
	)
{
	do
	{
		snxdp_nlk_exit();

		snxdp_ser_unregister_ports(&snxdp_ser_reg);

		snxdp_ser_unregister_driver(&snxdp_ser_reg);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


module_init(sddvirtual_init);
module_exit(sddvirtual_exit);

MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_DESCRIPTION(DRIVER_DESC);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,18))
MODULE_LICENSE("GPL");
#endif
MODULE_VERSION(NLK_TTY_DRIVER_VERSION);


/*-----------------------------------------------------------------------------------------------------------------------------*/


