/***********************************************************************
#
#   Module      : 
#   Description : 
#
#
#   Author(s)   : Morris ku
#   Company     : SUNIX Co., Ltd.
#   Maintain    : Morris ku
#************************************************************************/

#define DP_DIB_MAX							64
#define DP_MAX								128
#define NIC_MAX								10

#define MNG_STATUS_SUCCESS								0x00
#define MNG_STATUS_PARAMETER_INVALID					0x01

#define MNG_STATUS_NIC_NOT_FOUND						0x10
#define MNG_STATUS_NIC_REFRESH_FAIL						0x11
#define MNG_STATUS_NIC_TX_FAIL							0x12

#define MNG_STATUS_DP_NOT_FOUND							0x20
#define MNG_STATUS_DP_KEEPALIVE_START_FAIL				0x21
#define MNG_STATUS_DP_CTU_SERVER_START_FAIL				0x22
#define MNG_STATUS_DP_SIO_SERVER_START_FAIL				0x23

#define MNG_STATUS_DP_MAPPING_ALREADY					0x30
#define MNG_STATUS_DP_NOT_MAPPING						0x31
#define MNG_STATUS_DP_NO_IO_FOR_MAPPING					0x32
#define MNG_STATUS_DP_CREATE_DEVICE_NODE_FAIL			0x33
#define MNG_STATUS_DP_DELETE_DEVICE_NODE_FAIL			0x34
#define MNG_STATUS_DP_ALLOC_TTY_MINOR_NUMBER_FAIL		0x35

#define MNG_STATUS_DP_CONNECT_ALREADY					0x40
#define MNG_STATUS_DP_CONNECT_TIMEOUT					0x41
#define MNG_STATUS_DP_CONNECT_BUSY						0x42
#define MNG_STATUS_DP_CONNECT_UNSUPPORT_VERSION			0x43
#define MNG_STATUS_DP_CONNECT_FAIL						0x44
#define MNG_STATUS_DP_CONNECT_PASSWD_INCORRECT			0x45
#define MNG_STATUS_DP_DISCONNECT_ALREADY				0x46
#define MNG_STATUS_DP_DISCONNECT_TIMEOUT				0x47
#define MNG_STATUS_DP_DISCONNECT_BUSY					0x48
#define MNG_STATUS_DP_DISCONNECT_UNSUPPORT_VERSION		0x49
#define MNG_STATUS_DP_DISCONNECT_FAIL					0x4a
#define MNG_STATUS_DP_DISCONNECT_PASSWD_INCORRECT		0x4b
#define MNG_STATUS_DP_NOT_CONNECT						0x4c

#define MNG_STATUS_DP_LIB_INIT_ALREADY					0x50
#define MNG_STATUS_GET_SERVICE_STRING_FAIL				0x51
#define MNG_STATUS_WRONG_PROTOCOL_VERSION				0x52
#define MNG_STATUS_WRONG_PACKET_RECEIVED				0x53	
#define MNG_STATUS_EXTRA_DATA_LENGTH_INVALID			0x54	

#define MNG_STATUS_SERVICE_CONNECT_ALREADY				0x61
#define	MNG_STATUS_CONNECT_SERVICE_FAIL  				0x62
#define MNG_STATUS_SOCKET_TX_FAIL						0x63
#define MNG_STATUS_SOCKET_RX_FAIL						0x64
#define MNG_STATUS_SOCKET_RX_TIMEOUT					0x65
#define MNG_STATUS_MALLOC_MEMORY_FAIL					0x66
#define MNG_STATUS_CREATE_THREAD_FAIL					0x67
#define MNG_STATUS_SERVICE_NOT_CONNECT					0x68	
	

#define MNG_SERVICE_CONNECTION_ESTABLISHED				0x01
#define MNG_SERVICE_CONNECTION_DISESTABLISHED			0x00

#define CTU_STATUS_CTU_NOT_CONNECT						0x70
#define CTU_STATUS_DP_CTL_INIT_FAIL						0x71
#define CTU_STATUS_MUTEX_INIT_FAIL						0x72
#define CTU_STATUS_CTU_CONNECT_ALREADY					0x73
#define CTU_STATUS_CTU_RX_FAIL							0x74
#define CTU_STATUS_CTU_TX_FAIL							0x75
#define CTU_STATUS_CTU_PASSWORD_SETTING_INVALID			0x76
#define CTU_STATUS_CTU_WRONG_DEVICE_TYPE				0x77


typedef struct _VERSION_INFO
{	
	unsigned char				ServiceVersion[15];	
	unsigned char				LibVersion[15];	
	unsigned char				TtyDriverVersion[64];
	unsigned char				TtyDriverStartup;
	
}VERSION_INFO,*PVERSION_INFO;		


typedef struct _NIC_INFO
{	
	unsigned char				Index;
	unsigned char				Mac[6];
	unsigned char				InterfaceName[64];
	
}NIC_INFO,*PNIC_INFO;

typedef struct _BANK_INFO
{
	unsigned char 	BankIndex;
	unsigned char	IoNumber;
	unsigned char	WritingSettingsCapability;
	unsigned char 	DirectionCapability;
	unsigned char	EdgeTriggerCapability;
	unsigned char	OutputPinConfigCapability;
	unsigned char	OutputPinResistorApplied;
	unsigned int	DirectionMode;
	unsigned int	InputInvertEnable;
	unsigned char	EventMode;
	unsigned char	SamplingClockControl; 
	unsigned char	OutputPinConfig;						
	
}BANK_INFO, *PBANK_INFO;


typedef struct _DIO_INFO
{
	unsigned char 	Version;
	unsigned char 	NumberOfBanks;
	unsigned char	ShareDirectionControl;
	unsigned char	SamplingCapability;
	unsigned int	ClockFrequency;	

	BANK_INFO		BankInfo[32];	
	
}DIO_INFO, *PDIO_INFO;


typedef struct _AI_PORT_INFO
{
	unsigned char	PortIndex;
	unsigned int	SampleRateDivider;
	unsigned int	GainControl;
	unsigned char   LoggingMode;
	unsigned char	TriggerMode;
	unsigned short	UpperTriggerValue;
	unsigned short	LowerTriggerValue;
	unsigned char	FifoTriggerLevel;	
		
}AI_PORT_INFO, *PAI_PORT_INFO;

typedef struct _AI_INFO
{	
	unsigned char	Version;
	unsigned char 	NumberOfInputPort;
	unsigned char	Resolution;
	unsigned char	SupportInputType;
	unsigned char	SupportProgrammableGain;
	unsigned int	MaxSamplingRate;
	unsigned int	VrefData;	
	
	AI_PORT_INFO	AiPortInfo[32];
			
}AI_INFO, *PAI_INFO;

typedef struct _AO_PORT_INFO
{
	unsigned char	PortIndex;
	unsigned int	UpdateRateDivider;
	unsigned char	FifoTriggerLevel;
	unsigned char	GainControl;
	
}AO_PORT_INFO, *PAO_PORT_INFO;

typedef struct _AO_INFO
{	
	unsigned char	Version;
	unsigned char 	NumberOfOutputPort;
	unsigned char	Resolution;
	unsigned char	SupportProgrammableGain;
	unsigned int	MaxUpdatingRateCapability;	
	unsigned int	VrefData;	
	
	AO_PORT_INFO	AoPortInfo[32];	
			
}AO_INFO, *PAO_INFO;

typedef struct _SER_INFO
{
	unsigned char 				RS232Capability; 
	unsigned char 				RS422Capability; 
	unsigned char 				RS485Capability; 
	unsigned char 				AHDCCapability; // 0x00 none , 0x01 Enable
	unsigned char 				CSCapability; // 0x00 none , 0x01 Enable
	unsigned char 				RS422TerminationCtlCapability;// 0x00 none , 0x01 Enable
	unsigned char				RS485TerminationCtlCapability;// 0x00 none , 0x01 Enable
	unsigned char				RIPowerIOCapability; // 0x00 None ,0x01 5v , 0x02 12v , 0x03 no distinguish
	unsigned char 				DCDPowerIOCapability;// 0x00 None ,0x01 5v , 0x02 12v , 0x03 no distinguish
	
}SER_INFO,*PSER_INFO;

typedef struct _STR_INFO
{
	unsigned int	DeviceNameLen;
	unsigned char	DeviceName[64];
	
}STR_INFO,*PSTR_INFO;


typedef struct _SER_STATE_INFO
{
	unsigned char	version;
	unsigned char	baudrate;
	unsigned char	lcr;
	unsigned char	mcr;
	unsigned char	event_ctrl;
	unsigned char	on_chip_hw_flow_ctrl;
	unsigned char	msr;
	unsigned char	tx_trigger_level;
	unsigned char	rx_trigger_level;	
	unsigned char	power_io;
	unsigned char	mode_control;	
	
}SER_STATE_INFO,*PSER_STATE_INFO;


typedef struct _MAI_PORT_INFO
{
	unsigned char	PortIndex;
	unsigned char 	AiMode;
	unsigned char	LogginMode;
	unsigned char	TriggerMode;
	double   	UpperTriggerValue;
	double		LowerTriggerValue;
	unsigned char	Type1InputType; //(0x00 -> bipolar, 0x01 -> unipolar)
	unsigned char	Type1InputChannel; // (0x00 -> AIN1, 0x01 -> AIN2)
	unsigned char	Type1PgaEnabel;
	unsigned char	Type1GainControl; 
	
}MAI_PORT_INFO, *PMAI_PORT_INFO;


typedef struct _MAI_INFO
{
	unsigned char 	Version;
	unsigned int	FifoSize;
	unsigned char	FifoType;
	unsigned char	MAI_Type;
	unsigned char	SupportInputSource;
	unsigned char	NumInputPort;
	unsigned char	Resolution;
	unsigned char	SupportInputType;				
	unsigned char	MaxPromGains; 
	double		Verf;
	
	MAI_PORT_INFO	MaiPortInfo[16];	
	
}MAI_INFO,*PMAI_INFO;


typedef struct _DIB_INFO
{
	unsigned char 				ChlNumber;
	unsigned char 				Type;
	unsigned char 				MappingMinorNum;
	unsigned char				DataPipeFlowCtrlCap;
		
	DIO_INFO					DioInfo;	
	SER_INFO					SerInfo;
	STR_INFO					StrInfo;
	MAI_INFO					MaiInfo;
	
}DIB_INFO, *PDIB_INFO;


typedef struct _DP_INFO
{		
	unsigned char				Mac[6];
	unsigned char				ModelName[20];
	unsigned char				DpMode;
	unsigned char				PasswordEnable; //0x00 disable , 0a01 enable
	unsigned char				ConnectionStatus;
	unsigned char				ConnectedHostMac[6];
	unsigned char				MappingStatus;// (0x01 YES, 0x00 NO)
	unsigned char				Channels;
	unsigned char				Major;
	unsigned char				Minor;
	unsigned int				CtuTcpPort;	
	unsigned int				SioTcpPort;	
	
	DIB_INFO					DibInfo[DP_DIB_MAX];		
	
}DP_INFO,*PDP_INFO;	


typedef struct _DP_LIST
{		
	int							DpAmount;
	DP_INFO						DpInfo[DP_MAX];		
		
}DP_LIST, *PDP_LIST;


typedef struct _NIC_LIST
{		
	int							NicAmount;
	NIC_INFO					NicInfo[NIC_MAX];
		
} NIC_LIST, *PNIC_LIST;


typedef struct _CHL_MAPPING_INFO
{	
	unsigned char				DibChlNum;
	unsigned char				DibMappingNumber;
	
}CHL_MAPPING_INFO,*PCHL_MAPPING_INFO;


typedef struct _DP_MAPPING_INFO
{		
	unsigned char			Mac[6];
	int						ChlMappingAmount;
	CHL_MAPPING_INFO		ChlMappingInfo[DP_DIB_MAX];		
		
}DP_MAPPING_INFO, *PDP_MAPPING_INFO;

typedef struct _DP_MAPPING_LIST
{		
	int							DpMappingAmount;
	DP_MAPPING_INFO				DpMappingInfo[DP_MAX];
		
} DP_MAPPING_LIST, *PDP_MAPPING_LIST;


int dp_lib_init(void); 
void dp_lib_free(void);
int dp_get_service_connection_status(void);
int	dp_get_all_nic_info(PNIC_LIST pNicList);
int	dp_refresh_nic(PNIC_LIST pNicList);
int	dp_get_version(PVERSION_INFO pInfo);
int dp_search_dp(PDP_LIST pDpList);
int dp_get_specific_dp_detail_info(unsigned char * id, PDP_INFO pInfo);
int	dp_connect(unsigned char * id, int * SioTcpPort , int * CtuTcpPort ,unsigned int pPwSetting , unsigned char * pPassWord , unsigned int PassWordLength);
int dp_disconnect(unsigned char * id);
int	dp_mapping(unsigned char * id);
int	dp_unmapping(unsigned char * id);
int	dp_get_all_dp_mapping_info(PDP_MAPPING_LIST pMappingList);
int dp_get_all_dp_info_connected_by_host(PDP_LIST DpList);

int dp_get_password(unsigned char * Mac, unsigned char *pPassWord, unsigned int * pPwLength);
int dp_update_specific_dp_detail_info(unsigned char * id);

void (*managementcallback)(unsigned char * mac);
void management_register_function(void (*dp_main_func)(unsigned char * mac));
int dp_set_disconnet_event_callback(unsigned char * Mac , void (*managementcallback)(unsigned char * mac));

int ctu_connect(unsigned char * DpMac);
int ctu_disconnect(unsigned char * DpMac);
int ctu_set_mac(unsigned char * DpMac,unsigned char * SetMac);
int ctu_set_device_name(unsigned char * DpMac,unsigned char * DeviceName);
int ctu_set_passsword(unsigned char * DpMac,unsigned char pwsetting ,unsigned char * pw);
int ctu_set_uart_setting(unsigned char * DpMac);
int ctu_set_dio_setting(unsigned char * DpMac);
int ctu_update_firmware(unsigned char * DpMac , unsigned char * FilePath);
