/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioSetOutputTimerEnable(
	PSDP_DIB					pDib,
	bool						Enable
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	unsigned char				Setting = Enable?0x01:0x00;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		bReturn = LibDioSetOutputSafeValueTimer(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, Setting, pDib->DioSaveTimerTime);
		if (bReturn != true)
		{
			break;
		}


		pDib->DioSaveTimerEnable = Setting;
		OpcDioUpdateSetOutputTimerEnable(pDib);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioSetOutputTimerTime(
	PSDP_DIB					pDib,
	double						Time
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	unsigned char				TimerTime = ((Time * 1000) / 100);


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		bReturn = LibDioSetOutputSafeValueTimer(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, pDib->DioSaveTimerEnable, TimerTime);
		if (bReturn != true)
		{
			break;
		}


		pDib->DioSaveTimerTime = TimerTime;
		OpcDioUpdateSetOutputTimerTime(pDib);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioSetBankDirection(
	PSDP_DIO_BANK				pDioBank,
	unsigned int				Direction
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioBank->SDpPtr;
	bool						bReturn = false;
	unsigned int				BankDirection = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		if (Direction == 1)
		{
			BankDirection = pDioBank->IoMask;
		}
		else
		{
			BankDirection = 0x00000000;
		}


		bReturn = LibDioSetBankDirection(pSDp->DpInfo.Mac, pDioBank->DibPtr->Info.ChlNumber, pDioBank->BankIndex, BankDirection);
		if (bReturn != true)
		{
			break;
		}


		pDioBank->Direction = BankDirection;
		OpcDioUpdateSetBankDirection(pDioBank);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioSetBankInputEventTriggerMode(
	PSDP_DIO_BANK				pDioBank,
	unsigned char				InputEventMode
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioBank->SDpPtr;
	bool						bReturn = false;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}


		bReturn = LibDioSetBankInputEventMode(pSDp->DpInfo.Mac, pDioBank->DibPtr->Info.ChlNumber, pDioBank->BankIndex, InputEventMode);
		if (bReturn != true)
		{
			break;
		}


		pDioBank->InputEventMode = InputEventMode;
		OpcDioUpdateSetBankInputEventTriggerMode(pDioBank);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioSetBankOutputPinMode(
	PSDP_DIO_BANK				pDioBank,
	unsigned char				OutputPinMode
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioBank->SDpPtr;
	bool						bReturn = false;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}


		bReturn = LibDioSetBankOutputPinConfig(pSDp->DpInfo.Mac, pDioBank->DibPtr->Info.ChlNumber, pDioBank->BankIndex, OutputPinMode);
		if (bReturn != true)
		{
			break;
		}


		pDioBank->OutputPinMode = OutputPinMode;
		OpcDioUpdateSetBankOutputPinMode(pDioBank);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioSetPortOutputInitValue(
	PSDP_DIO_PORT				pDioPort,
	bool						State
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	unsigned int				BankOutputInitValue = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		BankOutputInitValue = pDioPort->BankPtr->OutputInitValue;
		BankOutputInitValue &= ~(0x00000001 << pDioPort->PortIndex);
		if (State == true)
		{
			BankOutputInitValue |= (0x00000001 << pDioPort->PortIndex);
		}

		bReturn = LibDioSetBankOutputInitialValue(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, BankOutputInitValue);
		if (bReturn != true)
		{
			break;
		}

		bReturn = LibCtuStoreDioSetting(pSDp->DpInfo.Mac);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->BankPtr->OutputInitValue = BankOutputInitValue;
		OpcDioUpdateSetPortOutputInitValue(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioSetPortOutputSaveValue(
	PSDP_DIO_PORT				pDioPort,
	bool						State
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	unsigned int				BankOutputSaveValue = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		BankOutputSaveValue = pDioPort->BankPtr->OutputSafeValue;
		BankOutputSaveValue &= ~(0x00000001 << pDioPort->PortIndex);
		if (State == true)
		{
			BankOutputSaveValue |= (0x00000001 << pDioPort->PortIndex);
		}

		bReturn = LibDioSetBankOutputSafeValue(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, BankOutputSaveValue);
		if (bReturn != true)
		{
			break;
		}

		bReturn = LibCtuStoreDioSetting(pSDp->DpInfo.Mac);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->BankPtr->OutputSafeValue = BankOutputSaveValue;
		OpcDioUpdateSetPortOutputSaveValue(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioSetPortState(
	PSDP_DIO_PORT				pDioPort,
	bool						State
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	unsigned int				BankState = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		BankState = pDioPort->BankPtr->State;
		BankState &= ~(0x00000001 << pDioPort->PortIndex);
		if (State == true)
		{
			BankState |= (0x00000001 << pDioPort->PortIndex);
		}

		bReturn = LibDioSetBankState(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, BankState);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->BankPtr->State = BankState;
		OpcDioUpdateSetPortState(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioGetPortState(
	PSDP_DIO_PORT				pDioPort
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	unsigned int				BankState = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		bReturn = LibDioGetBankState(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, &BankState);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->BankPtr->State = BankState;
		OpcDioUpdateGetPortState(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioGetPortInputCounter(
	PSDP_DIO_PORT				pDioPort
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	unsigned int				PortInputCounter = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		bReturn = LibDioGetBankIoInputCounterCounterValue(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, pDioPort->PortIndex, &PortInputCounter);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->InputCounterValue = PortInputCounter;
		OpcDioUpdateGetPortInputCounter(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioGetPortInputLatchRisingEdge(
	PSDP_DIO_PORT				pDioPort
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	int							BankInputLatchRisingEdge = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		bReturn = LibDioGetBankInputLatchRisingEdge(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, &BankInputLatchRisingEdge);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->BankPtr->InputLatchRisingEdge = BankInputLatchRisingEdge;
		OpcDioUpdateGetPortInputLatchRisingEdge(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioGetPortInputLatchFallingEdge(
	PSDP_DIO_PORT				pDioPort
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	int							BankInputLatchFallingEdge = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		bReturn = LibDioGetBankInputLatchFallingEdge(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, &BankInputLatchFallingEdge);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->BankPtr->InputLatchFallingEdge = BankInputLatchFallingEdge;
		OpcDioUpdateGetPortInputLatchFallingEdge(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioResetPortInputCounter(
	PSDP_DIO_PORT				pDioPort
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	int							BankInputCounterReset = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		BankInputCounterReset = (0x00000001 << pDioPort->PortIndex);

		bReturn = LibDioSetBankInputCounterReset(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, BankInputCounterReset);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->InputCounterValue = 0;
		OpcDioUpdateResetPortInputCounter(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioResetPortInputLatchRisingEdge(
	PSDP_DIO_PORT				pDioPort
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	int							BankInputLatchRisingEdge = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		BankInputLatchRisingEdge = (0x00000001 << pDioPort->PortIndex);

		bReturn = LibDioSetBankInputLatchRisingEdge(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, BankInputLatchRisingEdge);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->BankPtr->InputLatchRisingEdge &= ~(0x00000001 << pDioPort->PortIndex);
		OpcDioUpdateResetPortInputLatchRisingEdge(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioResetPortInputLatchFallingEdge(
	PSDP_DIO_PORT				pDioPort
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	int							BankInputLatchFallingEdge = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		BankInputLatchFallingEdge = (0x00000001 << pDioPort->PortIndex);

		bReturn = LibDioSetBankInputLatchFallingEdge(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, BankInputLatchFallingEdge);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->BankPtr->InputLatchFallingEdge &= ~(0x00000001 << pDioPort->PortIndex);
		OpcDioUpdateResetPortInputLatchFallingEdge(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioSetPortInputCounterIncrementRisingEdgeEnable(
	PSDP_DIO_PORT				pDioPort,
	bool						State
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	int							BankInputCounterIncrementRisingEdge = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		BankInputCounterIncrementRisingEdge = pDioPort->BankPtr->InputCounterIncrementRisingEdge;
		BankInputCounterIncrementRisingEdge &= ~(0x00000001 << pDioPort->PortIndex);
		if (State == true)
		{
			BankInputCounterIncrementRisingEdge |= (0x00000001 << pDioPort->PortIndex);
		}

		bReturn = LibDioSetBankInputCounterIncrementRisingEdge(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, BankInputCounterIncrementRisingEdge);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->BankPtr->InputCounterIncrementRisingEdge = BankInputCounterIncrementRisingEdge;
		OpcDioUpdateSetPortInputCounterIncrementRisingEdgeEnable(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioSetPortInputCounterIncrementFallingEdgeEnable(
	PSDP_DIO_PORT				pDioPort,
	bool						State
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	int							BankInputCounterIncrementFallingEdge = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		BankInputCounterIncrementFallingEdge = pDioPort->BankPtr->InputCounterIncrementFallingEdge;
		BankInputCounterIncrementFallingEdge &= ~(0x00000001 << pDioPort->PortIndex);
		if (State == true)
		{
			BankInputCounterIncrementFallingEdge |= (0x00000001 << pDioPort->PortIndex);
		}

		bReturn = LibDioSetBankInputCounterIncrementFallingEdge(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, BankInputCounterIncrementFallingEdge);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->BankPtr->InputCounterIncrementFallingEdge = BankInputCounterIncrementFallingEdge;
		OpcDioUpdateSetPortInputCounterIncrementFallingEdgeEnable(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntDioSetPortInputInvertEnable(
	PSDP_DIO_PORT				pDioPort,
	bool						State
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDioPort->BankPtr->SDpPtr;
	bool						bReturn = false;
	int							BankInputInvert = 0;
	unsigned int				BankState = 0;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		BankInputInvert = pDioPort->BankPtr->InputInvert;
		BankInputInvert &= ~(0x00000001 << pDioPort->PortIndex);
		if (State == true)
		{
			BankInputInvert |= (0x00000001 << pDioPort->PortIndex);
		}

		bReturn = LibDioSetBankInputInvert(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, BankInputInvert);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->BankPtr->InputInvert = BankInputInvert;
		OpcDioUpdateSetPortInputInvertEnable(pDioPort);


		bReturn = LibDioGetBankState(pSDp->DpInfo.Mac, pDioPort->BankPtr->DibPtr->Info.ChlNumber, pDioPort->BankPtr->BankIndex, &BankState);
		if (bReturn != true)
		{
			break;
		}


		pDioPort->BankPtr->State = BankState;
		OpcDioUpdateGetPortState(pDioPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
IntDioBankInputEventCallback(
	unsigned char *				pMac,
	int							DibChl,
	int							BankIndex,
	unsigned int 				DeltaValue,
	unsigned int				InputValue
	)
{
	PSDP_DEVICE					pSDp = NULL;
	PSDP_DIO_BANK				pDioBank = NULL;


	//printf("INT (:%.2x:%.2x:%.2x), ##### DIO Bank input event #####, Dib:%d, Bank:%d, DeltaValue:x%08x, InputValue:x%08x\n", pMac[3], pMac[4], pMac[5], DibChl, BankIndex, DeltaValue, InputValue);

	do
	{
		pSDp = SDpMainGetByMac(pMac);
		if (pSDp == NULL)
		{
			break;
		}

		pDioBank = &pSDp->DibArray[DibChl].DioBank[BankIndex];
		if (pDioBank == NULL)
		{
			break;
		}

		pDioBank->DeltaState = DeltaValue;
		pDioBank->State = InputValue;
		OpcDioUpdateBankInputEvent(pDioBank);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


