/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntMAiReadData(
	PSDP_MAI_PORT				pMAiPort
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pMAiPort->SDpPtr;
	bool						bReturn = false;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		OpcMAiUpdateReadData(pMAiPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntMAiSetPortEnable(
	PSDP_MAI_PORT				pMAiPort,
	bool						PortEnable
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pMAiPort->SDpPtr;
	bool						bReturn = false;
	unsigned short				PortsEnable = pMAiPort->DibPtr->MAiPortsEnable;
	unsigned char				Value = PortEnable?0x01:0x00;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		if (PortEnable == true)
		{
			bReturn = LibMAiSetPortLoggingMode(pSDp->DpInfo.Mac, pMAiPort->DibPtr->Info.ChlNumber, pMAiPort->PortIndex, 0x01);
			if (bReturn != true)
			{
				break;
			}
		}
		else
		{
			bReturn = LibMAiSetPortLoggingMode(pSDp->DpInfo.Mac, pMAiPort->DibPtr->Info.ChlNumber, pMAiPort->PortIndex, 0x00);
			if (bReturn != true)
			{
				break;
			}
		}

		PortsEnable &= ~(0x0001 << pMAiPort->PortIndex);
		PortsEnable |= (Value << pMAiPort->PortIndex);

		//printf("INT (:%.2x:%.2x:%.2x), IntMAiSetPortEnable, PortsEnable:x%04x\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], PortsEnable);
		bReturn = LibMAiSetPortsEnable(pSDp->DpInfo.Mac, pMAiPort->DibPtr->Info.ChlNumber, PortsEnable);
		if (bReturn != true)
		{
			break;
		}


		pMAiPort->PortEnable = Value;
		pMAiPort->DibPtr->MAiPortsEnable = PortsEnable;
		OpcMAiUpdateSetPortEnable(pMAiPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntMAiSetMeasureMode(
	PSDP_MAI_PORT				pMAiPort,
	unsigned int				MeasureMode,
	unsigned int				InputRange
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pMAiPort->SDpPtr;
	bool						bReturn = false;
	unsigned int				ADCCtrlReg = 0;
	unsigned char				InputType = 0x00;
	unsigned char				InputChannelSelection = 0x00;
	unsigned char				PullupResistorCtrl = 0x00;
	unsigned char				PgaEnable = 0x00;
	unsigned char				GainCtrl = 0x00;
	unsigned short				FilterCtrl = 0x0000;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		switch (pMAiPort->DibPtr->Info.MaiInfo.MAI_Type)
		{
			case 0x00 :
				break;

			case 0x01 :
				{
					switch (MeasureMode)
					{
						case 0 :
							{
								switch (InputRange)
								{
									case 0 : ADCCtrlReg = 0x00000480; break;	// +-100mv
									case 1 : ADCCtrlReg = 0x00000280; break;	// +-500mV
									case 2 : ADCCtrlReg = 0x00000180; break;	// +-1V
									case 3 : ADCCtrlReg = 0x00000102; break;	// +-5V
									case 4 : ADCCtrlReg = 0x00000002; break;	// +-10V
									case 5 : ADCCtrlReg = 0x00000481; break;	// 0~100mV
									case 6 : ADCCtrlReg = 0x00000281; break;	// 0~500mV
									case 7 : ADCCtrlReg = 0x00000181; break;	// 0~1V
									case 8 : ADCCtrlReg = 0x00000183; break;	// 0~5V
									case 9 : ADCCtrlReg = 0x00000003; break;	// 0~10V
								}
							}
							break;

						case 1 :
							{
								switch (InputRange)
								{
									case 0 : ADCCtrlReg = 0x00000000; break;	// +-20mA
									case 1 : ADCCtrlReg = 0x00000001; break;	// 0~20mA
								}

							}
							break;
					}
				}
				break;
		}


		//printf("INT (:%.2x:%.2x:%.2x), IntMAiSetMeasureMode, ADCCtrlReg:x%08x\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], ADCCtrlReg);


		switch (pMAiPort->DibPtr->Info.MaiInfo.MAI_Type)
		{
			case 0x00 :
				break;

			case 0x01 :
				{
					InputType = ADCCtrlReg & 0x00000001;
					InputChannelSelection = ((ADCCtrlReg & 0x00000002) >> 1);
					PgaEnable = ((ADCCtrlReg & 0x00000080) >> 7);
					GainCtrl = ((ADCCtrlReg & 0x00003f00) >> 8);
					//printf("INT (:%.2x:%.2x:%.2x), IntMAiSetMeasureMode, InputType:x%02x, InputChl:x%02x, PgaEnable:x%02x, GainCtrl:x%02x\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], InputType, InputChannelSelection, PgaEnable, GainCtrl);


					bReturn = LibMAiSetPortAiMode(pSDp->DpInfo.Mac, pMAiPort->DibPtr->Info.ChlNumber, pMAiPort->PortIndex, MeasureMode);
					if (bReturn != true)
					{
						//printf("INT (:%.2x:%.2x:%.2x), IntMAiSetMeasureMode, LibMAiSetPortAiMode fail\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
						break;
					}

					bReturn = LibMAiSetType1InputType(pSDp->DpInfo.Mac, pMAiPort->DibPtr->Info.ChlNumber, pMAiPort->PortIndex, InputType);
					if (bReturn != true)
					{
						//printf("INT (:%.2x:%.2x:%.2x), IntMAiSetMeasureMode, LibMAiSetType1InputType fail\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
						break;
					}

					bReturn = LibMAiSetType1InputSelect(pSDp->DpInfo.Mac, pMAiPort->DibPtr->Info.ChlNumber, pMAiPort->PortIndex, InputChannelSelection);
					if (bReturn != true)
					{
						//printf("INT (:%.2x:%.2x:%.2x), IntMAiSetMeasureMode, LibMAiSetType1InputSelect fail\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
						break;
					}

					bReturn = LibMAiSetType1PgaEnable(pSDp->DpInfo.Mac, pMAiPort->DibPtr->Info.ChlNumber, pMAiPort->PortIndex, PgaEnable);
					if (bReturn != true)
					{
						//printf("INT (:%.2x:%.2x:%.2x), IntMAiSetMeasureMode, LibMAiSetType1PgaEnable fail\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
						break;
					}

					bReturn = LibMAiSetType1Gain(pSDp->DpInfo.Mac, pMAiPort->DibPtr->Info.ChlNumber, pMAiPort->PortIndex, GainCtrl);
					if (bReturn != true)
					{
						//printf("INT (:%.2x:%.2x:%.2x), IntMAiSetMeasureMode, LibMAiSetType1Gain fail\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
						break;
					}

					pMAiPort->MeasureMode = MeasureMode;
					pMAiPort->ADCCtrlReg = ADCCtrlReg;
				}
				break;
		}


		OpcMAiUpdateSetMeasureMode(pMAiPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntMAiSetTriggerRange(
	PSDP_MAI_PORT				pMAiPort,
	bool						TriggerEnable,
	double						StartRange,
	double						EndRange
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pMAiPort->SDpPtr;
	bool						bReturn = false;
	unsigned char				TriggerEnableChar = 0x00;
	unsigned char				TriggerMode = 0x00;
	double						UpperTriggerValue = EndRange;
	double						LowerTriggerValue = StartRange;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		if (TriggerEnable == true)
		{
			TriggerEnableChar = 0x01;
			TriggerMode = 0x04;
		}
		else
		{
			TriggerEnableChar = 0x00;
			TriggerMode = 0x00;
		}
		
		//printf("INT (:%.2x:%.2x:%.2x), IntMAiSetTriggerRange, LowerTriggerValue:%.2f, UpperTriggerValue:%.2f, TriggerEnableChar:x%02x\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], LowerTriggerValue, UpperTriggerValue, TriggerEnableChar);


		bReturn = LibMAiSetPortTriggerMode(pSDp->DpInfo.Mac, pMAiPort->DibPtr->Info.ChlNumber, pMAiPort->PortIndex, TriggerMode);
		if (bReturn != true)
		{
			//printf("INT (:%.2x:%.2x:%.2x), IntMAiSetTriggerRange, LibMAiSetPortTriggerMode fail\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
			break;
		}

		if (TriggerEnableChar == 0x01)
		{
			bReturn = LibMAiSetPortLowerTriggerValue(pSDp->DpInfo.Mac, pMAiPort->DibPtr->Info.ChlNumber, pMAiPort->PortIndex, LowerTriggerValue);
			if (bReturn != true)
			{
				//printf("INT (:%.2x:%.2x:%.2x), IntMAiSetTriggerRange, LibMAiSetPortLowerTriggerValue fail\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
				break;
			}

			bReturn = LibMAiSetPortUpperTriggerValue(pSDp->DpInfo.Mac, pMAiPort->DibPtr->Info.ChlNumber, pMAiPort->PortIndex, UpperTriggerValue);
			if (bReturn != true)
			{
				//printf("INT (:%.2x:%.2x:%.2x), IntMAiSetTriggerRange, LibMAiSetPortUpperTriggerValue fail\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
				break;
			}

			pMAiPort->TriggerEnable = TriggerEnableChar;
			pMAiPort->LowerTriggerValue = LowerTriggerValue;
			pMAiPort->UpperTriggerValue = UpperTriggerValue;
			OpcMAiUpdateSetTriggerRange(pMAiPort);
		}
		else
		{
			pMAiPort->TriggerEnable = TriggerEnableChar;
			OpcMAiUpdateSetTriggerRange(pMAiPort);
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntMAiSetDataReturnInterval(
	PSDP_MAI_PORT				pMAiPort,
	unsigned int				Interval
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pMAiPort->SDpPtr;
	bool						bReturn = false;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}


		pMAiPort->DataReturnInterval = Interval;
		OpcMAiUpdateSetDataReturnInterval(pMAiPort);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
IntMAiEventCallback(
	unsigned char *				pMac,
	int							DibChl,
	int							PortChl,
	unsigned char				Event,
	double						EventValue
	)
{
	PSDP_DEVICE					pSDp = NULL;
	PSDP_MAI_PORT				pMAiPort = NULL;

	
	do
	{
		pSDp = SDpMainGetByMac(pMac);
		if (pSDp == NULL)
		{
			break;
		}

		pMAiPort = &pSDp->DibArray[DibChl].MAiPort[PortChl];
		if (pMAiPort == NULL)
		{
			break;
		}

		if (pMAiPort->TriggerEnable != 0x01)
		{
			break;
		}

		//printf("INT (:%.2x:%.2x:%.2x), ##### MAI event #####, Dib:%d, Port:%d, Event:x%02x, EventValue:%f\n", pMac[3], pMac[4], pMac[5], DibChl, PortChl, Event, EventValue);

		pMAiPort->CurrentData = (double)EventValue;

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
IntMAiDataCallback(
	unsigned char *				pMac,
	int							DibChl,
	int							PortChl,
	double						Data
	)
{
	PSDP_DEVICE					pSDp = NULL;
	PSDP_MAI_PORT				pMAiPort = NULL;

	
	do
	{
		pSDp = SDpMainGetByMac(pMac);
		if (pSDp == NULL)
		{
			break;
		}

		pMAiPort = &pSDp->DibArray[DibChl].MAiPort[PortChl];
		if (pMAiPort == NULL)
		{
			break;
		}

		if (pMAiPort->TriggerEnable != 0x00)
		{
			break;
		}

		//printf("INT (:%.2x:%.2x:%.2x), ##### MAI data #####, Dib:%d, Port:%d, Data:%f\n", pMac[3], pMac[4], pMac[5], DibChl, PortChl, Data);

		pMAiPort->CurrentData = (double)Data;

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


