/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntSerSetInterface(
	PSDP_DIB					pDib,
	unsigned int				Interface
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	unsigned char				OldModeCtrl = pDib->SerModeCtrl;
	unsigned char				NewModeCtrl = 0x00;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		if (Interface == 1)
		{
			// change to RS232
			NewModeCtrl = 0x01;
		}
		else if (Interface == 2)
		{
			// change to RS422
			NewModeCtrl = 0x02;
		}
		else if (Interface == 3)
		{
			// change to RS485
			NewModeCtrl = 0x04;
		}
		else
		{
			break;
		}

		bReturn = LibSerSetModeCtrl(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, NewModeCtrl);
		if (bReturn != true)
		{
			break;
		}


		pDib->SerModeCtrl = NewModeCtrl;
		OpcSerUpdateSetInterface(pDib, OldModeCtrl);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntSerSetParameters(
	PSDP_DIB					pDib,
	unsigned int				Baudrate,
	unsigned int				Databit,
	unsigned int				Parity,
	unsigned int				Stopbit
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	bool						bCheckParameter = true;
	unsigned char				NewBaudrate = 0x00;
	unsigned char				NewDatabit = 0x00;
	unsigned char				NewParity = 0x00;
	unsigned char				NewStopbit = 0x00;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		switch(Baudrate)
		{
			case 50 : NewBaudrate = 0x00; break;
			case 75 : NewBaudrate = 0x01; break;
			case 110 : NewBaudrate = 0x02; break;
			case 134 : NewBaudrate = 0x03; break;
			case 150 : NewBaudrate = 0x04; break;
			case 300 : NewBaudrate = 0x05; break;
			case 600 : NewBaudrate = 0x06; break;
			case 1200 : NewBaudrate = 0x07; break;
			case 2400 : NewBaudrate = 0x08; break;
			case 4800 : NewBaudrate = 0x09; break;
			case 9600 : NewBaudrate = 0x0a; break;
			case 19200 : NewBaudrate = 0x0b; break;
			case 38400 : NewBaudrate = 0x0c; break;
			case 57600 : NewBaudrate = 0x0d; break;
			case 115200 : NewBaudrate = 0x0e; break;
			case 230400 : NewBaudrate = 0x0f; break;
			case 460800 : NewBaudrate = 0x10; break;
			case 921600 : NewBaudrate = 0x11; break;
			default : bCheckParameter = false; break;
		}
		if (bCheckParameter != true)
		{
			break;
		}

		switch(Databit)
		{
			case 5 : NewDatabit = 0x00; break;
			case 6 : NewDatabit = 0x01; break;
			case 7 : NewDatabit = 0x02; break;
			case 8 : NewDatabit = 0x03; break;
			default : bCheckParameter = false; break;
		}
		if (bCheckParameter != true)
		{
			break;
		}

		switch(Parity)
		{
			case 0 : NewParity = 0x00; break;
			case 1 : NewParity = 0x01; break;
			case 2 : NewParity = 0x02; break;
			case 3 : NewParity = 0x03; break;
			case 4 : NewParity = 0x04; break;
			default : bCheckParameter = false; break;
		}
		if (bCheckParameter != true)
		{
			break;
		}

		switch(Stopbit)
		{
			case 1 : NewStopbit = 0x00; break;
			case 2 : NewStopbit = 0x02; break;
			case 3 : NewStopbit = 0x01; break;
			default : bCheckParameter = false; break;
		}
		if (bCheckParameter != true)
		{
			break;
		}

		bReturn = LibSerSetting(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, NewBaudrate, NewDatabit, NewParity, NewStopbit, pDib->SerFlowctrl);
		if (bReturn != true)
		{
			break;
		}


		pDib->SerBaudrate = NewBaudrate;
		pDib->SerDatabit = NewDatabit;
		pDib->SerParity = NewParity;
		pDib->SerStopbit = NewStopbit;
		OpcSerUpdateSetParameter(pDib);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntSerSetFlowctrl(
	PSDP_DIB					pDib,
	unsigned int				FlowCtrl
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	bool						bCheckParameter = true;
	unsigned char				NewFlowCtrl = 0x00;
	unsigned char				NewOnChipHwFlowCtrl = 0x00;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		if (pDib->SerVersion <= 3)
		{
			switch (FlowCtrl)
			{
				case 0 : NewFlowCtrl = 0x00; break;	// none
				case 1 : NewFlowCtrl = 0x01; break;	// RTS/CTS
				case 2 : NewFlowCtrl = 0x02; break;	// DTR/DSR
				default : bCheckParameter = false; break;
			}
			if (bCheckParameter != true)
			{
				break;
			}

			bReturn = LibSerSetting(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, pDib->SerBaudrate, pDib->SerDatabit, pDib->SerParity, pDib->SerStopbit, NewFlowCtrl);
			if (bReturn != true)
			{
				break;
			}


			pDib->SerFlowctrl = NewFlowCtrl;
			OpcSerUpdateSetFlowCtrl(pDib);

			bSuccess = true;
		}
		else
		{
			switch (FlowCtrl)
			{
				case 0 : NewOnChipHwFlowCtrl = 0x00; break;	// none
				case 1 : NewOnChipHwFlowCtrl = 0x01; break;	// RTS/CTS
				case 2 : NewOnChipHwFlowCtrl = 0x02; break;	// DTR/DSR
				default : bCheckParameter = false; break;
			}
			if (bCheckParameter != true)
			{
				break;
			}

			bReturn = LibSerSetOnChipHwFlowCtrl(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, NewOnChipHwFlowCtrl);
			if (bReturn != true)
			{
				break;
			}


			pDib->SerOnChipHwFlowCtrl = NewOnChipHwFlowCtrl;
			OpcSerUpdateSetFlowCtrl(pDib);

			bSuccess = true;
		}

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntSerSetDtr(
	PSDP_DIB					pDib,
	bool						State
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	unsigned char				NewMcr = pDib->SerMcr & 0xfd;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		if (State == true)
		{
			NewMcr |= 0x02;
		}

		bReturn = LibSerSetMcr(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, NewMcr);
		if (bReturn != true)
		{
			break;
		}


		pDib->SerMcr = NewMcr;
		OpcSerUpdateSetDtr(pDib);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntSerSetRts(
	PSDP_DIB					pDib,
	bool						State
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	unsigned char				NewMcr = pDib->SerMcr & 0xfe;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		if (State == true)
		{
			NewMcr |= 0x01;
		}

		bReturn = LibSerSetMcr(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, NewMcr);
		if (bReturn != true)
		{
			break;
		}


		pDib->SerMcr = NewMcr;
		OpcSerUpdateSetRts(pDib);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntSerSetBreak(
	PSDP_DIB					pDib,
	unsigned char				State
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	unsigned char				NewState = State;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		bReturn = LibSerSetBreak(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, NewState);
		if (bReturn != true)
		{
			break;
		}


		pDib->SerBreak = NewState;
		OpcSerUpdateSetBreak(pDib);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntSerSetTermination422Ctrl(
	PSDP_DIB					pDib,
	bool						State
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	unsigned char				NewModeCtrl = pDib->SerModeCtrl & 0xdf;

	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		if (State == true)
		{
			NewModeCtrl |= 0x20;
		}

		bReturn = LibSerSetModeCtrl(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, NewModeCtrl);
		if (bReturn != true)
		{
			break;
		}


		pDib->SerModeCtrl = NewModeCtrl;
		OpcSerUpdateSetTermination422Ctrl(pDib);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntSerSetTermination485Ctrl(
	PSDP_DIB					pDib,
	bool						State
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	unsigned char				NewModeCtrl = pDib->SerModeCtrl & 0xbf;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		if (State == true)
		{
			NewModeCtrl |= 0x40;
		}

		bReturn = LibSerSetModeCtrl(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, NewModeCtrl);
		if (bReturn != true)
		{
			break;
		}


		pDib->SerModeCtrl = NewModeCtrl;
		OpcSerUpdateSetTermination485Ctrl(pDib);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntSerSetRiPowerIO(
	PSDP_DIB					pDib,
	unsigned int				PowerIO
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	bool						bCheckParameter = true;
	unsigned char				NewPowerIO = pDib->SerPowerIO & 0xfc;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		switch (PowerIO)
		{
			case 0 : NewPowerIO |= 0x00; break;
			case 1 : NewPowerIO |= 0x01; break;
			case 2 : NewPowerIO |= 0x02; break;
			case 3 : NewPowerIO |= 0x03; break;
			default : bCheckParameter = false; break;
		}
		if (bCheckParameter != true)
		{
			break;
		}

		bReturn = LibSerSetPowerIO(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, NewPowerIO);
		if (bReturn != true)
		{
			break;
		}


		pDib->SerPowerIO = NewPowerIO;
		OpcSerUpdateSetRiPowerIO(pDib);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntSerSetDcdPowerIO(
	PSDP_DIB					pDib,
	unsigned int				PowerIO
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	bool						bCheckParameter = true;
	unsigned char				NewPowerIO = pDib->SerPowerIO & 0xf3;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		switch (PowerIO)
		{
			case 0 : NewPowerIO |= (0x00 << 2); break;
			case 1 : NewPowerIO |= (0x01 << 2); break;
			case 2 : NewPowerIO |= (0x02 << 2); break;
			case 3 : NewPowerIO |= (0x03 << 2); break;
			default : bCheckParameter = false; break;
		}
		if (bCheckParameter != true)
		{
			break;
		}

		bReturn = LibSerSetPowerIO(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, NewPowerIO);
		if (bReturn != true)
		{
			break;
		}


		pDib->SerPowerIO = NewPowerIO;
		OpcSerUpdateSetDcdPowerIO(pDib);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntSerSetCarrierSense(
	PSDP_DIB					pDib,
	bool						State
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;
	unsigned char				NewModeCtrl = pDib->SerModeCtrl & 0xef;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		if (State == true)
		{
			NewModeCtrl |= 0x10;
		}

		bReturn = LibSerSetModeCtrl(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, NewModeCtrl);
		if (bReturn != true)
		{
			break;
		}


		pDib->SerModeCtrl = NewModeCtrl;
		OpcSerUpdateSetCarrierSense(pDib);

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
IntSerWriteData(
	PSDP_DIB					pDib,
	unsigned char *				pData,
	unsigned int				DataLength
	)
{
	bool						bSuccess = false;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	bool						bReturn = false;


	do
	{
		if (SDpServiceIsStartUp != true)
		{
			break;
		}

		bReturn = LibSerWrite(pSDp->DpInfo.Mac, pDib->Info.ChlNumber, pData, DataLength);
		if (bReturn != true)
		{
			break;
		}


		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
IntSerMsrEventCallback(
	unsigned char *				pMac,
	int							DibChl,
	unsigned char				Msr
	)
{
	PSDP_DEVICE					pSDp = NULL;
	PSDP_DIB					pDib = NULL;


	//printf("INT (:%.2x:%.2x:%.2x), ##### SER MSR event #####, Dib:%d, MSR:x%02x\n", pMac[3], pMac[4], pMac[5], DibChl, Msr);

	do
	{
		pSDp = SDpMainGetByMac(pMac);
		if (pSDp == NULL)
		{
			break;
		}

		pDib = &pSDp->DibArray[DibChl];
		if (pDib == NULL)
		{
			break;
		}

		pDib->SerMsr = Msr;
		OpcSerUpdateMsrEvent(pDib);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
IntSerLsrEventCallback(
	unsigned char *				pMac,
	int							DibChl,
	unsigned char				Lsr,
	unsigned char				CreditBit8,
	unsigned char				Credit
	)
{
	PSDP_DEVICE					pSDp = NULL;
	PSDP_DIB					pDib = NULL;


	//printf("INT (:%.2x:%.2x:%.2x), ##### SER LSR event #####, Dib:%d, LSR:x%02x\n", pMac[3], pMac[4], pMac[5], DibChl, Lsr);

	do
	{
		pSDp = SDpMainGetByMac(pMac);
		if (pSDp == NULL)
		{
			break;
		}

		pDib = &pSDp->DibArray[DibChl];
		if (pDib == NULL)
		{
			break;
		}

		OpcSerUpdateLsrEvent(pDib);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
IntSerDataEventCallback(
	unsigned char *				pMac,
	int							DibChl,
	unsigned char *				pData,
	int							DataLength
	)
{
	PSDP_DEVICE					pSDp = NULL;
	PSDP_DIB					pDib = NULL;


	//printf("INT (:%.2x:%.2x:%.2x), ##### SER DATA event #####, Dib:%d, DataLength:%d\n", pMac[3], pMac[4], pMac[5], DibChl, DataLength);

	do
	{
		pSDp = SDpMainGetByMac(pMac);
		if (pSDp == NULL)
		{
			break;
		}

		pDib = &pSDp->DibArray[DibChl];
		if (pDib == NULL)
		{
			break;
		}

		OpcSerUpdateDataEvent(pDib, pData, DataLength);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


