/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngInit(
	void
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		nStatus = dp_lib_init();
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG, dp_lib_init fail, code:x%02x\n", nStatus);
			break;
		}

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
LibMngFree(
	void
	)
{
	do
	{
		dp_lib_free();

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngGetServiceStatus(
	void
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		nStatus = dp_get_service_connection_status();
		//printf("LIB MNG, dp_get_service_connection_status, code:x%02x\n", nStatus);
		if (nStatus == 0x01)
		{
			bSuccess = true;
		}

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngGetAllNicInfo(
	PNIC_LIST					pList
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if (pList == NULL)
		{
			break;
		}

		nStatus = dp_get_all_nic_info(pList);
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG, dp_get_all_nic_info fail, code:x%02x\n", nStatus);
			break;
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngRefreshNic(
	PNIC_LIST					pList
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if (pList == NULL)
		{
			break;
		}

		nStatus = dp_refresh_nic(pList);
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG, dp_refresh_nic fail, code:x%02x\n", nStatus);
			break;
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngGetVersionInfo(
	PVERSION_INFO				pInfo
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if (pInfo == NULL)
		{
			break;
		}

		nStatus = dp_get_version(pInfo);
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG, dp_get_version fail, code:x%02x\n", nStatus);
			break;
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngSearchDp(
	PDP_LIST					pList
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if (pList == NULL)
		{
			break;
		}

		nStatus = dp_search_dp(pList);
		if ((nStatus != MNG_STATUS_SUCCESS) && (nStatus != MNG_STATUS_DP_NOT_FOUND))
		{
			//printf("LIB MNG, dp_search_dp fail, code:x%02x\n", nStatus);
			break;
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngGetSpecificDpDetailInfo(
	unsigned char *				pMac,
	PDP_INFO					pInfo
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if (pInfo == NULL)
		{
			break;
		}

		nStatus = dp_get_specific_dp_detail_info(pMac, pInfo);
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG (:%.2x:%.2x:%.2x), dp_get_specific_dp_detail_info fail, code:x%02x\n", pMac[3], pMac[4], pMac[5], nStatus);
			break;
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngConnect(
	unsigned char *				pMac,
	unsigned int				PassWdSetting,
	unsigned char *				pPassWd,
	unsigned int				PassWdLength,
	int *						pSioTcpPort,
	int *						pCtuTcpPort
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if ((pMac == NULL) || (pPassWd == NULL) || (pSioTcpPort == NULL) || (pCtuTcpPort == NULL))
		{
			break;
		}

		nStatus = dp_connect(pMac, pSioTcpPort, pCtuTcpPort, PassWdSetting, pPassWd, PassWdLength);
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG, dp_connect fail, code:x%02x\n", nStatus);
			break;
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngDisconnect(
	unsigned char *				pMac
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if (pMac == NULL)
		{
			break;
		}

		nStatus = dp_disconnect(pMac);
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG, dp_disconnect fail, code:x%02x\n", nStatus);
			break;
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngMapping(
	unsigned char *				pMac
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if (pMac == NULL)
		{
			break;
		}

		nStatus = dp_mapping(pMac);
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG, dp_mapping fail, code:x%02x\n", nStatus);
			break;
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngUnMapping(
	unsigned char *				pMac
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if (pMac == NULL)
		{
			break;
		}

		nStatus = dp_unmapping(pMac);
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG, dp_unmapping fail, code:x%02x\n", nStatus);
			break;
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngGetAllDpMappingInfo(
	PDP_MAPPING_LIST			pList
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if (pList == NULL)
		{
			break;
		}

		nStatus = dp_get_all_dp_mapping_info(pList);
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG, dp_get_all_dp_mapping_info fail, code:x%02x\n", nStatus);
			break;
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngGetAllDpConnectedInfo(
	PDP_LIST					pList
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if (pList == NULL)
		{
			break;
		}

		nStatus = dp_get_all_dp_info_connected_by_host(pList);
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG, dp_get_all_dp_info_connected_by_host fail, code:x%02x\n", nStatus);
			break;
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngGetPassword(
	unsigned char *				pMac,
	unsigned char *				pPassword,
	unsigned int *				pPasswordLength
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if ((pMac == NULL) || (pPassword == NULL) || (pPasswordLength == NULL))
		{
			break;
		}

		nStatus = dp_get_password(pMac, pPassword, pPasswordLength);
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG, dp_get_password fail, code:x%02x\n", nStatus);
			break;
		}

		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibMngSetDisconnectEventCallback(
	unsigned char *				pMac
	)
{
	int							nStatus = MNG_STATUS_SUCCESS;
	bool						bSuccess = false;


	do
	{
		if (pMac == NULL)
		{
			break;
		}

		management_register_function(IntMngDisconnectEventCallback);

		nStatus = dp_set_disconnet_event_callback(pMac, IntMngDisconnectEventCallback);
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			//printf("LIB MNG, dp_set_disconnet_event_callback fail, code:x%02x\n", nStatus);
			break;
		}
		bSuccess = true;

	} while (false);

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


