/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void
_SetTimeout(
	struct timespec *			pTimeSpec,
	int							TimeOut	// in millisecond
	)
{
	struct timeval				TimeVal;
	int							SecUnit = 0;
	int							mSecUnit = 0;


	memset(pTimeSpec, 0x00, sizeof(struct timespec));
	gettimeofday(&TimeVal, NULL);


	if (TimeOut >= 1000)
	{
		SecUnit = TimeOut / 1000;
		mSecUnit = TimeOut % 1000;
	}
	else
	{
		SecUnit = 0;
		mSecUnit = TimeOut;
	}


	pTimeSpec->tv_sec = (TimeVal.tv_sec + SecUnit);
	pTimeSpec->tv_nsec = ((TimeVal.tv_usec * 1000) + (mSecUnit * 1000000));

	pTimeSpec->tv_sec += pTimeSpec->tv_nsec / 1000000000L;
	pTimeSpec->tv_nsec = pTimeSpec->tv_nsec % 1000000000L;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
_HandleSDpWhenOnLine(
	void
	)
{
	PDP_LIST					pDpList = NULL;
	bool						bReturn = false;
	int							i = 0;
	int							j = 0;
	unsigned char				Password[256];
	unsigned int				PasswordLength = 0;
	PSDP_DEVICE					pSDp = NULL;
	int							TryCnt = 5;


	do
	{
		pDpList = (PDP_LIST)malloc(sizeof(DP_LIST));
		if (pDpList == NULL)
		{
			break;
		}
		memset(pDpList, 0, sizeof(DP_LIST));


		while (TryCnt > 0)
		{
			do
			{
				bReturn = LibMngGetAllDpConnectedInfo(pDpList);
				if (bReturn != true)
				{
					break;
				}

				for (i = 0 ; i < pDpList->DpAmount; i++)
				{
					pSDp = SDpMainGetByMac(pDpList->DpInfo[i].Mac);
					if (pSDp != NULL)
					{
						//printf("LIB THD (:%.2x:%.2x:%.2x), already exist\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
						break;
					}

					bReturn = LibMngGetSpecificDpDetailInfo(pDpList->DpInfo[i].Mac, &pDpList->DpInfo[i]);
					if (bReturn != true)
					{
						//printf("LIB THD (:%.2x:%.2x:%.2x), get detail info fail\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
						break;
					}

					bReturn = LibMngSetDisconnectEventCallback(pDpList->DpInfo[i].Mac);
					if (bReturn != true)
					{
						//printf("LIB THD (:%.2x:%.2x:%.2x), Connect, LibMngSetDisconnectEventCallback fail\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
						break;
					}

					bReturn = LibCtuConnect(pDpList->DpInfo[i].Mac);
					if (bReturn != true)
					{
						//printf("LIB THD (:%.2x:%.2x:%.2x), connect, LibCtuConnect fail\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
						break;
					}

					bReturn = LibSioConnect(pDpList->DpInfo[i].Mac, pDpList->DpInfo[i].SioTcpPort);
					if (bReturn != true)
					{
						//printf("LIB THD (:%.2x:%.2x:%.2x), connect, LibSioConnect fail\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
						break;
					}

					memset(Password, 0, sizeof(Password));
					PasswordLength = 0;
					if (pDpList->DpInfo[i].PasswordEnable == 0x01)
					{
						bReturn = LibMngGetPassword(pDpList->DpInfo[i].Mac, Password, &PasswordLength);
						if (bReturn != true)
						{
							//printf("LIB THD (:%.2x:%.2x:%.2x), get password fail\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
						}
					}


					for (j = 0; j < pDpList->DpInfo[i].Channels; j++)
					{
						if (pDpList->DpInfo[i].DibInfo[j].Type == DLLP_CHL_TYPE_UART)
						{
							//printf(""LIB THD (:%.2x:%.2x:%.2x), get SER info for opc\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
						}
						if (pDpList->DpInfo[i].DibInfo[j].Type == DLLP_CHL_TYPE_DIO)
						{
							bReturn = LibDioGetInfo(pDpList->DpInfo[i].Mac, pDpList->DpInfo[i].DibInfo[j].ChlNumber, &pDpList->DpInfo[i].DibInfo[j].DioInfo);
							//printf(""LIB THD (:%.2x:%.2x:%.2x), get DIO info for opc, bReturn:%s\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5], bReturn?"TRUE":"FALSE");
						}
						if (pDpList->DpInfo[i].DibInfo[j].Type == DLLP_CHL_TYPE_MULTIFUNC_AI)
						{
							bReturn = LibMAiGetInfo(pDpList->DpInfo[i].Mac, pDpList->DpInfo[i].DibInfo[j].ChlNumber, &pDpList->DpInfo[i].DibInfo[j].MaiInfo);
							//printf("LIB THD (:%.2x:%.2x:%.2x)(%d), get MAi info for opc, bReturn:%s, MAI_Type:x%02x\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5], pDpList->DpInfo[i].DibInfo[j].ChlNumber, bReturn?"TRUE":"FALSE", pDpList->DpInfo[i].DibInfo[j].MaiInfo.MAI_Type);
						}
					}


					bReturn = SDpMainAdd(&pDpList->DpInfo[i], true, Password, PasswordLength);
					if (bReturn != true)
					{
						//printf("LIB THD (:%.2x:%.2x:%.2x), add fail\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
					}
				}

			} while (false);

			TryCnt--;
			sleep(1);
		}

	} while (false);

	if (pDpList != NULL)
	{
		free(pDpList);
		pDpList = NULL;
	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void 
_HandleSDpWhenOffLine(
	void
	)
{
	SDpMainDelAll();
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void *
_Thread(
	void *						pContext
	)
{
	PLIB_SERVICE				pService = (PLIB_SERVICE)pContext;
	struct timespec				TimeSpec;
	int							nReturn = 0;
	int							nStatus = MNG_STATUS_SUCCESS;


	//printf("LIB THD, start\n");
	while (1)
	{
		_SetTimeout(&TimeSpec, 1000);
		nReturn = pthread_cond_timedwait(&pService->ThreadCondExit, &pService->ThreadMutex, &TimeSpec);
		if (nReturn != ETIMEDOUT)
		{
			break;
		}


		nStatus = dp_get_service_connection_status();
		if ((nStatus == 0x01) && (SDpServiceIsStartUp == false))
		{
			//printf("LIB THD, service ON-LINE\n");
			SDpServiceIsStartUp = true;
			OpcServiceChangeDpServiceStartUpState(&OpcService, SDpServiceIsStartUp);

			_HandleSDpWhenOnLine();
		}


		if ((nStatus != 0x01) && (SDpServiceIsStartUp == true))
		{
			//printf("LIB THD, service OFF-LINE\n");
			SDpServiceIsStartUp = false;
			OpcServiceChangeDpServiceStartUpState(&OpcService, SDpServiceIsStartUp);

			_HandleSDpWhenOffLine();
		}
	}


	SDpServiceIsStartUp = false;
	OpcServiceChangeDpServiceStartUpState(&OpcService, SDpServiceIsStartUp);

	//printf("LIB THD, exit\n");
	pthread_exit((void *)0);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
LibServiceStart(
	PLIB_SERVICE				pService
	)
{
	int							nReturn = 0;
	bool						bSuccess = false;
	bool						bStep_1 = false;
	bool						bStep_2 = false;
	bool						bStep_3 = false;
	bool						bStep_4 = false;
	bool						bStep_5 = false;


	do
	{
		nReturn = pthread_mutex_init(&pService->ThreadMutex, NULL);
		if (nReturn != 0)
		{
			break;
		}
		bStep_1 = true;


		nReturn = pthread_cond_init(&pService->ThreadCondExit, NULL);
		if (nReturn != 0)
		{
			break;
		}
		bStep_2 = true;


		nReturn = pthread_attr_init(&pService->ThreadAttr);
		if (nReturn != 0)
		{
			break;
		}
		bStep_3 = true;


		nReturn = pthread_attr_setstacksize(&pService->ThreadAttr, 131072);
		if (nReturn != 0)
		{
			break;
		}
		bStep_4 = true;


		nReturn = pthread_create(&pService->ThreadHd, &pService->ThreadAttr, _Thread, (void *)pService);
		if (nReturn != 0)
		{
			break;
		}
		bStep_5 = true;


		bSuccess = true;

	} while (false);

	if (bSuccess != true)
	{
		if (bStep_5 == true)
		{

		}

		if (bStep_4 == true)
		{

		}

		if (bStep_3 == true)
		{
			pthread_attr_destroy(&pService->ThreadAttr);
		}

		if (bStep_2 == true)
		{
			pthread_cond_destroy(&pService->ThreadCondExit);
		}

		if (bStep_1 == true)
		{
			pthread_mutex_destroy(&pService->ThreadMutex);
		}
	}

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
LibServiceStop(
	PLIB_SERVICE				pService
	)
{
	do
	{
		pthread_mutex_lock(&pService->ThreadMutex);
		pthread_cond_signal(&pService->ThreadCondExit);
		pthread_mutex_unlock(&pService->ThreadMutex);
		pthread_join(pService->ThreadHd, NULL);

	} while (false);

	pthread_attr_destroy(&pService->ThreadAttr);
	pthread_cond_destroy(&pService->ThreadCondExit);
	pthread_mutex_destroy(&pService->ThreadMutex);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


