/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void
_SetTimeout(
	struct timespec *			pTimeSpec,
	int							TimeOut	// in millisecond
	)
{
	struct timeval				TimeVal;
	int							SecUnit = 0;
	int							mSecUnit = 0;


	memset(pTimeSpec, 0x00, sizeof(struct timespec));
	gettimeofday(&TimeVal, NULL);


	if (TimeOut >= 1000)
	{
		SecUnit = TimeOut / 1000;
		mSecUnit = TimeOut % 1000;
	}
	else
	{
		SecUnit = 0;
		mSecUnit = TimeOut;
	}


	pTimeSpec->tv_sec = (TimeVal.tv_sec + SecUnit);
	pTimeSpec->tv_nsec = ((TimeVal.tv_usec * 1000) + (mSecUnit * 1000000));

	pTimeSpec->tv_sec += pTimeSpec->tv_nsec / 1000000000L;
	pTimeSpec->tv_nsec = pTimeSpec->tv_nsec % 1000000000L;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void *
_Thread(
	void *						pContext
	)
{
	PSDP_DIB					pDib = (PSDP_DIB)pContext;
	PSDP_DEVICE					pSDp = pDib->SDpPtr;
	struct timespec				TimeSpec;
	int							nReturn = 0;
	int							nStatus = MNG_STATUS_SUCCESS;
	int							Cnt = 0;
	int							i = 0;


	//printf("OPC MAI THD, %02x:%02x:%02x-(%d), start\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pDib->ObjIndex);
	while (1)
	{
		_SetTimeout(&TimeSpec, 1000);
		nReturn = pthread_cond_timedwait(&pDib->MAiThreadCondExit, &pDib->MAiThreadMutex, &TimeSpec);
		if (nReturn != ETIMEDOUT)
		{
			break;
		}

		Cnt++;
		//printf("OPC MAI THD, %02x:%02x:%02x-(%d), CNT:%d\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pDib->ObjIndex, Cnt);


		for (i = 0; i < pDib->Info.MaiInfo.NumInputPort; i++)
		{
			if ((pDib->MAiPort[i].PortEnable == 0x01) && (pDib->MAiPort[i].DataReturnInterval > 0))
			{
				pDib->MAiPort[i].DataReturnIntervalCnt += 1;
				if (pDib->MAiPort[i].DataReturnIntervalCnt >= pDib->MAiPort[i].DataReturnInterval)
				{
					//printf("OPC MAI THD, %02x:%02x:%02x-(%d), Port:%d return data, Cnt:%d\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pDib->ObjIndex, pDib->MAiPort[i].PortIndex, Cnt);
					OpcMAiUpdateReadData(&pDib->MAiPort[i]);
					pDib->MAiPort[i].DataReturnIntervalCnt = 0;
				}
			}
			else
			{
				pDib->MAiPort[i].DataReturnIntervalCnt = 0;
			}
		}
	}

	//printf("OPC MAI THD, %02x:%02x:%02x-(%d), exit\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pDib->ObjIndex);
	pthread_exit((void *)0);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
OpcMAiThreadStart(
	PSDP_DIB					pDib
	)
{
	int							nReturn = 0;
	bool						bSuccess = false;
	bool						bStep_1 = false;
	bool						bStep_2 = false;
	bool						bStep_3 = false;
	bool						bStep_4 = false;
	bool						bStep_5 = false;


	do
	{
		nReturn = pthread_mutex_init(&pDib->MAiThreadMutex, NULL);
		if (nReturn != 0)
		{
			break;
		}
		bStep_1 = true;


		nReturn = pthread_cond_init(&pDib->MAiThreadCondExit, NULL);
		if (nReturn != 0)
		{
			break;
		}
		bStep_2 = true;


		nReturn = pthread_attr_init(&pDib->MAiThreadAttr);
		if (nReturn != 0)
		{
			break;
		}
		bStep_3 = true;


		nReturn = pthread_attr_setstacksize(&pDib->MAiThreadAttr, 131072);
		if (nReturn != 0)
		{
			break;
		}
		bStep_4 = true;


		nReturn = pthread_create(&pDib->MAiThreadHd, &pDib->MAiThreadAttr, _Thread, (void *)pDib);
		if (nReturn != 0)
		{
			break;
		}
		bStep_5 = true;


		bSuccess = true;

	} while (false);

	if (bSuccess != true)
	{
		if (bStep_5 == true)
		{

		}

		if (bStep_4 == true)
		{

		}

		if (bStep_3 == true)
		{
			pthread_attr_destroy(&pDib->MAiThreadAttr);
		}

		if (bStep_2 == true)
		{
			pthread_cond_destroy(&pDib->MAiThreadCondExit);
		}

		if (bStep_1 == true)
		{
			pthread_mutex_destroy(&pDib->MAiThreadMutex);
		}
	}

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
OpcMAiThreadStop(
	PSDP_DIB					pDib
	)
{
	do
	{
		pthread_mutex_lock(&pDib->MAiThreadMutex);
		pthread_cond_signal(&pDib->MAiThreadCondExit);
		pthread_mutex_unlock(&pDib->MAiThreadMutex);
		pthread_join(pDib->MAiThreadHd, NULL);

	} while (false);

	pthread_attr_destroy(&pDib->MAiThreadAttr);
	pthread_cond_destroy(&pDib->MAiThreadCondExit);
	pthread_mutex_destroy(&pDib->MAiThreadMutex);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


