/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void
_CreateSDpBasicNode(
	POPC_SERVICE				pService,
	PSDP_DEVICE					pSDp,
	UA_NodeId *					pUaIdObjRoot
	)
{
	char						StringId[256] = {0};
	char						Data[256] = {0};
	UA_VariableAttributes		uaAttrVariable;
	UA_String					uaString;
	UA_Int32					uaInt32;
	UA_Boolean					uaBoolean;


	// Model_name variable node
	uaAttrVariable = UA_VariableAttributes_default;
	uaString = UA_STRING((char *)pSDp->DpInfo.ModelName);
	UA_Variant_setScalar(&uaAttrVariable.value, &uaString, &UA_TYPES[UA_TYPES_STRING]);
	uaAttrVariable.displayName = UA_LOCALIZEDTEXT("en-US", "Model_name");
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Model_name", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addVariableNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		*pUaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT),
		UA_QUALIFIEDNAME(1, "Model_name"),
		UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), uaAttrVariable, NULL, NULL);


	// Device_name variable node
	if (pSDp->HasDeviceName == true)
	{
		uaAttrVariable = UA_VariableAttributes_default;
		uaString = UA_STRING((char *)pSDp->DeviceName);
		UA_Variant_setScalar(&uaAttrVariable.value, &uaString, &UA_TYPES[UA_TYPES_STRING]);
		uaAttrVariable.displayName = UA_LOCALIZEDTEXT("en-US", "Device_name");
		memset(StringId, 0, sizeof(StringId));
		sprintf(StringId, "%02x:%02x:%02x-Device_name", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

		UA_Server_addVariableNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
			*pUaIdObjRoot,
			UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT),
			UA_QUALIFIEDNAME(1, "Device_name"),
			UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), uaAttrVariable, NULL, NULL);
	}


	// Firmware_version variable node
	memset(Data, 0, sizeof(Data));
	sprintf(Data, "%d.%d", pSDp->DpInfo.Major, pSDp->DpInfo.Minor);
	uaAttrVariable = UA_VariableAttributes_default;
	uaString = UA_STRING(Data);
	UA_Variant_setScalar(&uaAttrVariable.value, &uaString, &UA_TYPES[UA_TYPES_STRING]);
	uaAttrVariable.displayName = UA_LOCALIZEDTEXT("en-US", "Firmware_version");
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Firmware_version", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addVariableNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		*pUaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT),
		UA_QUALIFIEDNAME(1, "Firmware_version"),
		UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), uaAttrVariable, NULL, NULL);


	// Object_amount variable node
	uaInt32 = pSDp->DpInfo.Channels;
	uaAttrVariable = UA_VariableAttributes_default;
	UA_Variant_setScalar(&uaAttrVariable.value, &uaInt32, &UA_TYPES[UA_TYPES_INT32]);
	uaAttrVariable.displayName = UA_LOCALIZEDTEXT("en-US", "Object_amount");
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Object_amount", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addVariableNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		*pUaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT),
		UA_QUALIFIEDNAME(1, "Object_amount"),
		UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), uaAttrVariable, NULL, NULL);


	// Is_connect_enable variable node
	if (pSDp->DpInfo.ConnectionStatus == 0x01)
	{
		uaBoolean = false; // connected by other host
	}
	else
	{
		uaBoolean = true; // not connect by other host
	}
	uaAttrVariable = UA_VariableAttributes_default;
	UA_Variant_setScalar(&uaAttrVariable.value, &uaBoolean, &UA_TYPES[UA_TYPES_BOOLEAN]);
	uaAttrVariable.displayName = UA_LOCALIZEDTEXT("en-US", "Is_connect_enable");
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Is_connect_enable", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addVariableNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		*pUaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT),
		UA_QUALIFIEDNAME(1, "Is_connect_enable"),
		UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), uaAttrVariable, NULL, NULL);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------------------------------------*/


void
OpcMngAddSDpForSearch(
	PSDP_DEVICE					pSDp
	)
{
	POPC_SERVICE				pService = &OpcService;
	char						StringId[256] = {0};
	char						Data[256] = {0};
	int							i = 0;
	UA_NodeId					uaIdSearchList = UA_NODEID_NULL;
	UA_NodeId					uaIdObjRoot = UA_NODEID_NULL;
	UA_NodeId					uaIdDeviceInfo = UA_NODEID_NULL;
	UA_NodeId					uaIdObjObj = UA_NODEID_NULL;
	UA_ObjectAttributes			uaAttrObj;
	UA_MethodAttributes			uaAttrMethod;
	UA_VariableAttributes		uaAttrVariable;
	UA_Argument					uaArgumentInput[2];
	UA_Int32					uaInt32;
	UA_Boolean					uaBoolean;


	uaIdSearchList = UA_NODEID_STRING(1, UAID_Search_device_list);


	// Root object node
	uaAttrObj = UA_ObjectAttributes_default;
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x:%02x:%02x:%02x", pSDp->DpInfo.Mac[0], pSDp->DpInfo.Mac[1], pSDp->DpInfo.Mac[2], pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
	uaAttrObj.displayName = UA_LOCALIZEDTEXT("en-US", StringId);

	UA_Server_addObjectNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		uaIdSearchList,
		UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
		UA_QUALIFIEDNAME(1, StringId),
		UA_NODEID_NUMERIC(0, UA_NS0ID_FOLDERTYPE), uaAttrObj, NULL, &uaIdObjRoot);


	// SDP basic info
	_CreateSDpBasicNode(pService, pSDp, &uaIdObjRoot);


	// Password_enable variable node
	if (pSDp->DpInfo.PasswordEnable == 0x01)
	{
		uaBoolean = true;
	}
	else
	{
		uaBoolean = false;
	}
	uaAttrVariable = UA_VariableAttributes_default;
	UA_Variant_setScalar(&uaAttrVariable.value, &uaBoolean, &UA_TYPES[UA_TYPES_BOOLEAN]);
	uaAttrVariable.displayName = UA_LOCALIZEDTEXT("en-US", "Password_enable");
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Password_enable", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addVariableNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		uaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT),
		UA_QUALIFIEDNAME(1, "Password_enable"),
		UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), uaAttrVariable, NULL, NULL);


	// Connect method node
	UA_Argument_init(&uaArgumentInput[0]);
	uaArgumentInput[0].description = UA_LOCALIZEDTEXT("en-US", "Password");
	uaArgumentInput[0].name = UA_STRING("Password");
	uaArgumentInput[0].dataType = UA_TYPES[UA_TYPES_STRING].typeId;
	uaArgumentInput[0].valueRank = -1;

	uaAttrMethod = UA_MethodAttributes_default;
	uaAttrMethod.displayName = UA_LOCALIZEDTEXT("en-US", "Connect");
	uaAttrMethod.executable = true;
	uaAttrMethod.userExecutable = true;

	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Connect", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addMethodNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		uaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_HASORDEREDCOMPONENT),
		UA_QUALIFIEDNAME(1, "Connect"),
		uaAttrMethod, &OpcMngCallbackSDpConnect,
		1, uaArgumentInput, 0, NULL, (void *)pSDp, NULL);


	// Device_info object node
	uaAttrObj = UA_ObjectAttributes_default;
	uaAttrObj.displayName = UA_LOCALIZEDTEXT("en-US", "Device_info");
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Device_info", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addObjectNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		uaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
		UA_QUALIFIEDNAME(1, "Device_info"),
		UA_NODEID_NUMERIC(0, UA_NS0ID_FOLDERTYPE), uaAttrObj, NULL, &uaIdDeviceInfo);




	for (i = 0; i < pSDp->DpInfo.Channels; i++)
	{
		uaIdObjObj = UA_NODEID_NULL;


		if ((pSDp->DibArray[i].Info.Type != DLLP_CHL_TYPE_UART) &&
			(pSDp->DibArray[i].Info.Type != DLLP_CHL_TYPE_DIO) &&
			(pSDp->DibArray[i].Info.Type != DLLP_CHL_TYPE_MULTIFUNC_AI))
		{
			continue;
		}


		// Object object node
		memset(Data, 0, sizeof(Data));
		switch (pSDp->DibArray[i].Info.Type)
		{
			case DLLP_CHL_TYPE_UART :
				sprintf(Data, "Object%d(Serial)", pSDp->DibArray[i].ObjIndex);
				break;
			case DLLP_CHL_TYPE_DIO :
				sprintf(Data, "Object%d(Dio)", pSDp->DibArray[i].ObjIndex);
				break;
			case DLLP_CHL_TYPE_MULTIFUNC_AI :
				sprintf(Data, "Object%d(Mfai)", pSDp->DibArray[i].ObjIndex);
				break;
			default :
				sprintf(Data, "Object%d(Unknow)", pSDp->DibArray[i].ObjIndex);
				break;
		}
		uaAttrObj = UA_ObjectAttributes_default;
		uaAttrObj.displayName = UA_LOCALIZEDTEXT("en-US", Data);
		memset(StringId, 0, sizeof(StringId));
		switch (pSDp->DibArray[i].Info.Type)
		{
			case DLLP_CHL_TYPE_UART :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_serial", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
			case DLLP_CHL_TYPE_DIO :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_dio", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
			case DLLP_CHL_TYPE_MULTIFUNC_AI :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_mfai", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
			default :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_unknow", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
		}
		
		UA_Server_addObjectNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
			uaIdDeviceInfo,
			UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
			UA_QUALIFIEDNAME(1, Data),
			UA_NODEID_NUMERIC(0, UA_NS0ID_FOLDERTYPE), uaAttrObj, NULL, &uaIdObjObj);


		// Object index variable node
		uaInt32 = pSDp->DibArray[i].ObjIndex;
		uaAttrVariable = UA_VariableAttributes_default;
		UA_Variant_setScalar(&uaAttrVariable.value, &uaInt32, &UA_TYPES[UA_TYPES_INT32]);
		uaAttrVariable.displayName = UA_LOCALIZEDTEXT("en-US", "Object_index");
		memset(StringId, 0, sizeof(StringId));
		sprintf(StringId, "%02x:%02x:%02x-(%d)Object_index", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);

		UA_Server_addVariableNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
			uaIdObjObj,
			UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT),
			UA_QUALIFIEDNAME(1, "Object_index"),
			UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), uaAttrVariable, NULL, NULL);


		// Object type variable node
		uaInt32 = pSDp->DibArray[i].Info.Type;
		uaAttrVariable = UA_VariableAttributes_default;
		UA_Variant_setScalar(&uaAttrVariable.value, &uaInt32, &UA_TYPES[UA_TYPES_INT32]);
		uaAttrVariable.displayName = UA_LOCALIZEDTEXT("en-US", "Object_type");
		memset(StringId, 0, sizeof(StringId));
		sprintf(StringId, "%02x:%02x:%02x-(%d)Object_type", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);

		UA_Server_addVariableNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
			uaIdObjObj,
			UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT),
			UA_QUALIFIEDNAME(1, "Object_type"),
			UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), uaAttrVariable, NULL, NULL);


		if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_UART)
		{
			//printf("create SER index:%d, SEARCH\n", pSDp->DibArray[i].ObjIndex);
			OpcSerCreateCapaNode(pService, pSDp, &pSDp->DibArray[i], &uaIdObjObj);
		}
		if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_DIO)
		{
			//printf("create DIO index:%d, SEARCH\n", pSDp->DibArray[i].ObjIndex);
			OpcDioCreateCapaNode(pService, pSDp, &pSDp->DibArray[i], &uaIdObjObj);
		}
		if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_MULTIFUNC_AI)
		{
			//printf("create MAI index:%d, SEARCH\n", pSDp->DibArray[i].ObjIndex);
			OpcMAiCreateCapaNode(pService, pSDp, &pSDp->DibArray[i], &uaIdObjObj);
		}
	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
OpcMngDelSDpForSearch(
	PSDP_DEVICE					pSDp
	)
{
	POPC_SERVICE				pService = &OpcService;
	char						StringId[256] = {0};
	int							i = 0;
	UA_NodeId					uaIdObjRoot = UA_NODEID_NULL;
	UA_NodeId					uaIdDeviceInfo = UA_NODEID_NULL;
	UA_NodeId					uaIdObjObj = UA_NODEID_NULL;


	for (i = 0; i < pSDp->DpInfo.Channels; i++)
	{
		uaIdObjObj = UA_NODEID_NULL;


		if ((pSDp->DibArray[i].Info.Type != DLLP_CHL_TYPE_UART) &&
			(pSDp->DibArray[i].Info.Type != DLLP_CHL_TYPE_DIO) &&
			(pSDp->DibArray[i].Info.Type != DLLP_CHL_TYPE_MULTIFUNC_AI))
		{
			continue;
		}


		if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_UART)
		{
			//printf("delete SER index:%d, SEARCH\n", pSDp->DibArray[i].ObjIndex);
			OpcSerDeleteCapaNode(pService, pSDp, &pSDp->DibArray[i]);
		}
		if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_DIO)
		{
			//printf("delete DIO index:%d, SEARCH\n", pSDp->DibArray[i].ObjIndex);
			OpcDioDeleteCapaNode(pService, pSDp, &pSDp->DibArray[i]);
		}
		if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_MULTIFUNC_AI)
		{
			//printf("delete MAI index:%d, SEARCH\n", pSDp->DibArray[i].ObjIndex);
			OpcMAiDeleteCapaNode(pService, pSDp, &pSDp->DibArray[i]);
		}


		// Object object node
		memset(StringId, 0, sizeof(StringId));
		switch (pSDp->DibArray[i].Info.Type)
		{
			case DLLP_CHL_TYPE_UART :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_serial", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
			case DLLP_CHL_TYPE_DIO :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_dio", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
			case DLLP_CHL_TYPE_MULTIFUNC_AI :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_mfai", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
			default :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_unknow", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
		}
		uaIdObjObj = UA_NODEID_STRING(1, StringId);
		UA_Server_deleteNode(pService->uaServerPtr, uaIdObjObj, true);
	}


	// Device_info object node
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Device_info", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
	uaIdDeviceInfo = UA_NODEID_STRING(1, StringId);
	UA_Server_deleteNode(pService->uaServerPtr, uaIdDeviceInfo, true);


	// Root object node
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x:%02x:%02x:%02x", pSDp->DpInfo.Mac[0], pSDp->DpInfo.Mac[1], pSDp->DpInfo.Mac[2], pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
	uaIdObjRoot = UA_NODEID_STRING(1, StringId);
	UA_Server_deleteNode(pService->uaServerPtr, uaIdObjRoot, true);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
OpcMngAddSDpForConnect(
	PSDP_DEVICE					pSDp
	)
{
	POPC_SERVICE				pService = &OpcService;
	char						StringId[256] = {0};
	char						Data[256] = {0};
	int							i = 0;
	UA_NodeId					uaIdConnectList = UA_NODEID_NULL;
	UA_NodeId					uaIdObjRoot = UA_NODEID_NULL;
	UA_NodeId					uaIdDeviceInfo = UA_NODEID_NULL;
	UA_NodeId					uaIdObjObj = UA_NODEID_NULL;
	UA_ObjectAttributes			uaAttrObj;
	UA_MethodAttributes			uaAttrMethod;
	UA_VariableAttributes		uaAttrVariable;
	UA_Argument					uaArgumentInput[2];
	UA_Int32					uaInt32;
	UA_String					uaString;


	uaIdConnectList = UA_NODEID_STRING(1, UAID_Connect_device_list);


	// Root object node
	uaAttrObj = UA_ObjectAttributes_default;
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x:%02x:%02x:%02x", pSDp->DpInfo.Mac[0], pSDp->DpInfo.Mac[1], pSDp->DpInfo.Mac[2], pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
	uaAttrObj.displayName = UA_LOCALIZEDTEXT("en-US", StringId);

	UA_Server_addObjectNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		uaIdConnectList,
		UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
		UA_QUALIFIEDNAME(1, StringId),
		UA_NODEID_NUMERIC(0, UA_NS0ID_FOLDERTYPE), uaAttrObj, NULL, &uaIdObjRoot);


	// SDP basic info
	_CreateSDpBasicNode(pService, pSDp, &uaIdObjRoot);


	// Password variable node
	if (pSDp->PasswordLength > 0)
	{
		uaString = UA_STRING((char *)pSDp->Password);
	}
	else
	{
		uaString = UA_STRING("");
	}
	uaAttrVariable = UA_VariableAttributes_default;
	UA_Variant_setScalar(&uaAttrVariable.value, &uaString, &UA_TYPES[UA_TYPES_STRING]);
	uaAttrVariable.displayName = UA_LOCALIZEDTEXT("en-US", "Password");
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Password", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addVariableNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		uaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT),
		UA_QUALIFIEDNAME(1, "Password"),
		UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), uaAttrVariable, NULL, NULL);


	// Disconnect method node
	uaAttrMethod = UA_MethodAttributes_default;
	uaAttrMethod.displayName = UA_LOCALIZEDTEXT("en-US", "Disconnect");
	uaAttrMethod.executable = true;
	uaAttrMethod.userExecutable = true;

	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Disconnect", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addMethodNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		uaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_HASORDEREDCOMPONENT),
		UA_QUALIFIEDNAME(1, "Disconnect"),
		uaAttrMethod, &OpcMngCallbackSDpDisconnect,
		0, NULL, 0, NULL, (void *)pSDp, NULL);


	// Set_device_name method node
	if (pSDp->HasDeviceName == true)
	{
		UA_Argument_init(&uaArgumentInput[0]);
		uaArgumentInput[0].description = UA_LOCALIZEDTEXT("en-US", "Device name");
		uaArgumentInput[0].name = UA_STRING("Device name");
		uaArgumentInput[0].dataType = UA_TYPES[UA_TYPES_STRING].typeId;
		uaArgumentInput[0].valueRank = -1;

		uaAttrMethod = UA_MethodAttributes_default;
		uaAttrMethod.displayName = UA_LOCALIZEDTEXT("en-US", "Set_device_name");
		uaAttrMethod.executable = true;
		uaAttrMethod.userExecutable = true;

		memset(StringId, 0, sizeof(StringId));
		sprintf(StringId, "%02x:%02x:%02x-Set_device_name", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

		UA_Server_addMethodNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
			uaIdObjRoot,
			UA_NODEID_NUMERIC(0, UA_NS0ID_HASORDEREDCOMPONENT),
			UA_QUALIFIEDNAME(1, "Set_device_name"),
			uaAttrMethod, &OpcMngCallbackSDpSetDeviceName,
			1, uaArgumentInput, 0, NULL, (void *)pSDp, NULL);
	}


	// Set_mac method node
	UA_Argument_init(&uaArgumentInput[0]);
	uaArgumentInput[0].description = UA_LOCALIZEDTEXT("en-US", "Device MAC, e.g. 000a0a9a7ecc");
	uaArgumentInput[0].name = UA_STRING("Device MAC");
	uaArgumentInput[0].dataType = UA_TYPES[UA_TYPES_STRING].typeId;
	uaArgumentInput[0].valueRank = -1;

	uaAttrMethod = UA_MethodAttributes_default;
	uaAttrMethod.displayName = UA_LOCALIZEDTEXT("en-US", "Set_mac");
	uaAttrMethod.executable = true;
	uaAttrMethod.userExecutable = true;

	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Set_mac", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addMethodNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		uaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_HASORDEREDCOMPONENT),
		UA_QUALIFIEDNAME(1, "Set_mac"),
		uaAttrMethod, &OpcMngCallbackSDpSetMac,
		1, uaArgumentInput, 0, NULL, (void *)pSDp, NULL);


	// Set_password method node
	UA_Argument_init(&uaArgumentInput[0]);
	uaArgumentInput[0].description = UA_LOCALIZEDTEXT("en-US", "Password");
	uaArgumentInput[0].name = UA_STRING("Password");
	uaArgumentInput[0].dataType = UA_TYPES[UA_TYPES_STRING].typeId;
	uaArgumentInput[0].valueRank = -1;

	uaAttrMethod = UA_MethodAttributes_default;
	uaAttrMethod.displayName = UA_LOCALIZEDTEXT("en-US", "Set_password");
	uaAttrMethod.executable = true;
	uaAttrMethod.userExecutable = true;

	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Set_password", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addMethodNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		uaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_HASORDEREDCOMPONENT),
		UA_QUALIFIEDNAME(1, "Set_password"),
		uaAttrMethod, &OpcMngCallbackSDpSetPassword,
		1, uaArgumentInput, 0, NULL, (void *)pSDp, NULL);


	// Clear_password method node
	uaAttrMethod = UA_MethodAttributes_default;
	uaAttrMethod.displayName = UA_LOCALIZEDTEXT("en-US", "Clear_password");
	uaAttrMethod.executable = true;
	uaAttrMethod.userExecutable = true;

	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Clear_password", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addMethodNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		uaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_HASORDEREDCOMPONENT),
		UA_QUALIFIEDNAME(1, "Clear_password"),
		uaAttrMethod, &OpcMngCallbackSDpClearPassword,
		0, NULL, 0, NULL, (void *)pSDp, NULL);


	// Device_info object node
	uaAttrObj = UA_ObjectAttributes_default;
	uaAttrObj.displayName = UA_LOCALIZEDTEXT("en-US", "Device_info");
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Device_info", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Server_addObjectNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
		uaIdObjRoot,
		UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
		UA_QUALIFIEDNAME(1, "Device_info"),
		UA_NODEID_NUMERIC(0, UA_NS0ID_FOLDERTYPE), uaAttrObj, NULL, &uaIdDeviceInfo);




	for (i = 0; i < pSDp->DpInfo.Channels; i++)
	{
		uaIdObjObj = UA_NODEID_NULL;


		if ((pSDp->DibArray[i].Info.Type != DLLP_CHL_TYPE_UART) &&
			(pSDp->DibArray[i].Info.Type != DLLP_CHL_TYPE_DIO) &&
			(pSDp->DibArray[i].Info.Type != DLLP_CHL_TYPE_MULTIFUNC_AI))
		{
			continue;
		}


		// Object object node
		memset(Data, 0, sizeof(Data));
		switch (pSDp->DibArray[i].Info.Type)
		{
			case DLLP_CHL_TYPE_UART :
				sprintf(Data, "Object%d(Serial)", pSDp->DibArray[i].ObjIndex);
				break;
			case DLLP_CHL_TYPE_DIO :
				sprintf(Data, "Object%d(Dio)", pSDp->DibArray[i].ObjIndex);
				break;
			case DLLP_CHL_TYPE_MULTIFUNC_AI :
				sprintf(Data, "Object%d(Mfai)", pSDp->DibArray[i].ObjIndex);
				break;
			default :
				sprintf(Data, "Object%d(Unknow)", pSDp->DibArray[i].ObjIndex);
				break;
		}
		uaAttrObj = UA_ObjectAttributes_default;
		uaAttrObj.displayName = UA_LOCALIZEDTEXT("en-US", Data);
		memset(StringId, 0, sizeof(StringId));
		switch (pSDp->DibArray[i].Info.Type)
		{
			case DLLP_CHL_TYPE_UART :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_serial", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
			case DLLP_CHL_TYPE_DIO :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_dio", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
			case DLLP_CHL_TYPE_MULTIFUNC_AI :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_mfai", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
			default :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_unknow", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
		}
		
		UA_Server_addObjectNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
			uaIdDeviceInfo,
			UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
			UA_QUALIFIEDNAME(1, Data),
			UA_NODEID_NUMERIC(0, UA_NS0ID_FOLDERTYPE), uaAttrObj, NULL, &uaIdObjObj);


		// Object index variable node
		uaInt32 = pSDp->DibArray[i].ObjIndex;
		uaAttrVariable = UA_VariableAttributes_default;
		UA_Variant_setScalar(&uaAttrVariable.value, &uaInt32, &UA_TYPES[UA_TYPES_INT32]);
		uaAttrVariable.displayName = UA_LOCALIZEDTEXT("en-US", "Object_index");
		memset(StringId, 0, sizeof(StringId));
		sprintf(StringId, "%02x:%02x:%02x-(%d)Object_index", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);

		UA_Server_addVariableNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
			uaIdObjObj,
			UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT),
			UA_QUALIFIEDNAME(1, "Object_index"),
			UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), uaAttrVariable, NULL, NULL);


		// Object type variable node
		uaInt32 = pSDp->DibArray[i].Info.Type;
		uaAttrVariable = UA_VariableAttributes_default;
		UA_Variant_setScalar(&uaAttrVariable.value, &uaInt32, &UA_TYPES[UA_TYPES_INT32]);
		uaAttrVariable.displayName = UA_LOCALIZEDTEXT("en-US", "Object_type");
		memset(StringId, 0, sizeof(StringId));
		sprintf(StringId, "%02x:%02x:%02x-(%d)Object_type", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);

		UA_Server_addVariableNode(pService->uaServerPtr, UA_NODEID_STRING(1, StringId),
			uaIdObjObj,
			UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT),
			UA_QUALIFIEDNAME(1, "Object_type"),
			UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), uaAttrVariable, NULL, NULL);


		if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_UART)
		{
			//printf("create SER index:%d, CONNECT\n", pSDp->DibArray[i].ObjIndex);
			OpcSerCreateInfoNode(pService, pSDp, &pSDp->DibArray[i], &uaIdObjObj);
		}
		if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_DIO)
		{
			//printf("create DIO index:%d, CONNECT\n", pSDp->DibArray[i].ObjIndex);
			OpcDioCreateInfoNode(pService, pSDp, &pSDp->DibArray[i], &uaIdObjObj);
		}
		if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_MULTIFUNC_AI)
		{
			//printf("create MAI index:%d, CONNECT\n", pSDp->DibArray[i].ObjIndex);
			OpcMAiCreateInfoNode(pService, pSDp, &pSDp->DibArray[i], &uaIdObjObj);
		}
	}
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
OpcMngDelSDpForConnect(
	PSDP_DEVICE					pSDp
	)
{
	POPC_SERVICE				pService = &OpcService;
	char						StringId[256] = {0};
	int							i = 0;
	UA_NodeId					uaIdObjRoot = UA_NODEID_NULL;
	UA_NodeId					uaIdDeviceInfo = UA_NODEID_NULL;
	UA_NodeId					uaIdObjObj = UA_NODEID_NULL;


	for (i = 0; i < pSDp->DpInfo.Channels; i++)
	{
		uaIdObjObj = UA_NODEID_NULL;


		if ((pSDp->DibArray[i].Info.Type != DLLP_CHL_TYPE_UART) &&
			(pSDp->DibArray[i].Info.Type != DLLP_CHL_TYPE_DIO) &&
			(pSDp->DibArray[i].Info.Type != DLLP_CHL_TYPE_MULTIFUNC_AI))
		{
			continue;
		}


		if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_UART)
		{
			//printf("delete SER index:%d, CONNECT\n", pSDp->DibArray[i].ObjIndex);
			OpcSerDeleteInfoNode(pService, pSDp, &pSDp->DibArray[i]);
		}
		if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_DIO)
		{
			//printf("delete DIO index:%d, CONNECT\n", pSDp->DibArray[i].ObjIndex);
			OpcDioDeleteInfoNode(pService, pSDp, &pSDp->DibArray[i]);
		}
		if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_MULTIFUNC_AI)
		{
			//printf("delete MAI index:%d, CONNECT\n", pSDp->DibArray[i].ObjIndex);
			OpcMAiDeleteInfoNode(pService, pSDp, &pSDp->DibArray[i]);
		}


		// Object object node
		memset(StringId, 0, sizeof(StringId));
		switch (pSDp->DibArray[i].Info.Type)
		{
			case DLLP_CHL_TYPE_UART :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_serial", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
			case DLLP_CHL_TYPE_DIO :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_dio", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
			case DLLP_CHL_TYPE_MULTIFUNC_AI :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_mfai", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
			default :
				sprintf(StringId, "%02x:%02x:%02x-(%d)Object_unknow", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], pSDp->DibArray[i].ObjIndex);
				break;
		}
		uaIdObjObj = UA_NODEID_STRING(1, StringId);
		UA_Server_deleteNode(pService->uaServerPtr, uaIdObjObj, true);
	}


	// Device_info object node
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Device_info", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
	uaIdDeviceInfo = UA_NODEID_STRING(1, StringId);
	UA_Server_deleteNode(pService->uaServerPtr, uaIdDeviceInfo, true);


	// Root object node
	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x:%02x:%02x:%02x", pSDp->DpInfo.Mac[0], pSDp->DpInfo.Mac[1], pSDp->DpInfo.Mac[2], pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);
	uaIdObjRoot = UA_NODEID_STRING(1, StringId);
	UA_Server_deleteNode(pService->uaServerPtr, uaIdObjRoot, true);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------------------------------------*/


void
OpcMngUpdateSDpDeviceName(
	PSDP_DEVICE					pSDp
	)
{
	POPC_SERVICE				pService = &OpcService;
	char						StringId[256] = {0};
	UA_String 					uaString = UA_STRING((char *)pSDp->DeviceName);
	UA_Variant					uaVariant;


	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Device_name", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Variant_init(&uaVariant);
	UA_Variant_setScalar(&uaVariant, &uaString, &UA_TYPES[UA_TYPES_STRING]);
	UA_Server_writeValue(pService->uaServerPtr, UA_NODEID_STRING(1, StringId), uaVariant);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
OpcMngUpdateSDpPassword(
	PSDP_DEVICE					pSDp
	)
{
	POPC_SERVICE				pService = &OpcService;
	char						StringId[256] = {0};
	UA_String 					uaString;
	UA_Variant					uaVariant;


	if (pSDp->PasswordLength > 0)
	{
		uaString = UA_STRING((char *)pSDp->Password);
	}
	else
	{
		uaString = UA_STRING("");
	}

	memset(StringId, 0, sizeof(StringId));
	sprintf(StringId, "%02x:%02x:%02x-Password", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	UA_Variant_init(&uaVariant);
	UA_Variant_setScalar(&uaVariant, &uaString, &UA_TYPES[UA_TYPES_STRING]);
	UA_Server_writeValue(pService->uaServerPtr, UA_NODEID_STRING(1, StringId), uaVariant);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------------------------------------*/


UA_StatusCode
OpcMngCallbackSearchSDp(
	UA_Server *					pServer,
	const UA_NodeId *			pSessionId,
	void *						pSessionHandle,
	const UA_NodeId *			pMethodId,
	void *						pMethodContext,
	const UA_NodeId *			pObjectId,
	void *						pObjectContext,
	size_t						inputSize,
	const UA_Variant *			pInput,
	size_t						outputSize,
	UA_Variant *				pOutput
	)
{
	UA_StatusCode				uaStatus = UA_STATUSCODE_GOOD;
	bool						bReturn = false;


	do
	{
		bReturn = IntMngSearchSDp();
		if (bReturn != true)
		{
			uaStatus = UA_STATUSCODE_BADUNEXPECTEDERROR;
			break;
		}

	} while (false);

	return uaStatus;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


UA_StatusCode
OpcMngCallbackSDpConnect(
	UA_Server *					pServer,
	const UA_NodeId *			pSessionId,
	void *						pSessionHandle,
	const UA_NodeId *			pMethodId,
	void *						pMethodContext,
	const UA_NodeId *			pObjectId,
	void *						pObjectContext,
	size_t						inputSize,
	const UA_Variant *			pInput,
	size_t						outputSize,
	UA_Variant *				pOutput
	)
{
	UA_StatusCode				uaStatus = UA_STATUSCODE_GOOD;
	bool						bReturn = false;
	PSDP_DEVICE					pSDp = (PSDP_DEVICE)pMethodContext;
	UA_String *					uaValue1Ptr = (UA_String *)pInput[0].data;
	bool						PasswdEnable = false;
	char						PasswdData[256] = {0};
	int							PasswdDataLength = 0;


	do
	{
		PasswdDataLength = uaValue1Ptr->length;
		if (PasswdDataLength > 0)
		{
			memcpy(PasswdData, uaValue1Ptr->data, PasswdDataLength);
			PasswdEnable = true;
		}

		bReturn = IntMngSDpConnect(pSDp, PasswdEnable, PasswdData, PasswdDataLength);
		if (bReturn != true)
		{
			uaStatus = UA_STATUSCODE_BADUNEXPECTEDERROR;
			break;
		}

	} while (false);

	return uaStatus;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


UA_StatusCode
OpcMngCallbackSDpDisconnect(
	UA_Server *					pServer,
	const UA_NodeId *			pSessionId,
	void *						pSessionHandle,
	const UA_NodeId *			pMethodId,
	void *						pMethodContext,
	const UA_NodeId *			pObjectId,
	void *						pObjectContext,
	size_t						inputSize,
	const UA_Variant *			pInput,
	size_t						outputSize,
	UA_Variant *				pOutput
	)
{
	UA_StatusCode				uaStatus = UA_STATUSCODE_GOOD;
	bool						bReturn = false;
	PSDP_DEVICE					pSDp = (PSDP_DEVICE)pMethodContext;


	do
	{
		bReturn = IntMngSDpDisconnect(pSDp);
		if (bReturn != true)
		{
			uaStatus = UA_STATUSCODE_BADUNEXPECTEDERROR;
			break;
		}

	} while (false);

	return uaStatus;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


UA_StatusCode
OpcMngCallbackSDpSetDeviceName(
	UA_Server *					pServer,
	const UA_NodeId *			pSessionId,
	void *						pSessionHandle,
	const UA_NodeId *			pMethodId,
	void *						pMethodContext,
	const UA_NodeId *			pObjectId,
	void *						pObjectContext,
	size_t						inputSize,
	const UA_Variant *			pInput,
	size_t						outputSize,
	UA_Variant *				pOutput
	)
{
	UA_StatusCode				uaStatus = UA_STATUSCODE_GOOD;
	bool						bReturn = false;
	PSDP_DEVICE					pSDp = (PSDP_DEVICE)pMethodContext;
	UA_String *					uaValue1Ptr = (UA_String *)pInput[0].data;
	char						DeviceName[256] = {0};
	int							DeviceNameLength = 0;


	do
	{
		DeviceNameLength = uaValue1Ptr->length;
		if ((DeviceNameLength <= 0) || (DeviceNameLength > 256))
		{
			uaStatus = UA_STATUSCODE_UNCERTAINENGINEERINGUNITSEXCEEDED;
			break;
		}

		memcpy(DeviceName, uaValue1Ptr->data, DeviceNameLength);

		bReturn = IntMngSDpSetDeviceName(pSDp, DeviceName);
		if (bReturn != true)
		{
			uaStatus = UA_STATUSCODE_BADUNEXPECTEDERROR;
			break;
		}

	} while (false);

	return uaStatus;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


UA_StatusCode
OpcMngCallbackSDpSetMac(
	UA_Server *					pServer,
	const UA_NodeId *			pSessionId,
	void *						pSessionHandle,
	const UA_NodeId *			pMethodId,
	void *						pMethodContext,
	const UA_NodeId *			pObjectId,
	void *						pObjectContext,
	size_t						inputSize,
	const UA_Variant *			pInput,
	size_t						outputSize,
	UA_Variant *				pOutput
	)
{
	UA_StatusCode				uaStatus = UA_STATUSCODE_GOOD;
	bool						bReturn = false;
	PSDP_DEVICE					pSDp = (PSDP_DEVICE)pMethodContext;
	UA_String *					uaValue1Ptr = (UA_String *)pInput[0].data;
	char						DeviceMacString[256] = {0};
	int							DeviceMacLength = 0;
	unsigned char				Mac[6] = {0};
	bool						bCheckData = true;
	int							i = 0;
	int							j = 0;
	unsigned int				HiByte = 0;
	unsigned int				LowByte = 0;


	do
	{
		DeviceMacLength = uaValue1Ptr->length;
		if (DeviceMacLength != 12)
		{
			uaStatus = UA_STATUSCODE_UNCERTAINENGINEERINGUNITSEXCEEDED;
			break;
		}

		memcpy(DeviceMacString, uaValue1Ptr->data, DeviceMacLength);

		for (i = 0; i < 12; i++)
		{
			if (((DeviceMacString[i] < '0') || (DeviceMacString[i] > '9')) && ((DeviceMacString[i] < 'a') || (DeviceMacString[i] > 'f')) && ((DeviceMacString[i] < 'A') || (DeviceMacString[i] > 'F')))
			{
				bCheckData = false;
				break;
			}
			DeviceMacString[i] = (char)tolower((int)DeviceMacString[i]);
		}

		if (bCheckData == false)
		{
			uaStatus = UA_STATUSCODE_UNCERTAINENGINEERINGUNITSEXCEEDED;
			break;
		}

		for (i = 0, j = 0; i < 12; i += 2, j++)
		{
			HiByte = DeviceMacString[i] > '9' ? DeviceMacString[i] - 'a' + 10 : DeviceMacString[i] - '0';
			LowByte = DeviceMacString[i + 1] > '9' ? DeviceMacString[i + 1] - 'a' + 10 : DeviceMacString[i + 1] - '0';

			Mac[j] = (HiByte << 4) | LowByte;
		}

		//printf("OPC (:%.2x:%.2x:%.2x), OpcMngCallbackSDpSetMac, MAC-%.2x:%.2x:%.2x:%.2x:%.2x:%.2x\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5], Mac[0], Mac[1], Mac[2], Mac[3], Mac[4], Mac[5]);

		bReturn = IntMngSDpSetDeviceMac(pSDp, Mac);
		if (bReturn != true)
		{
			uaStatus = UA_STATUSCODE_BADUNEXPECTEDERROR;
			break;
		}

	} while (false);

	return uaStatus;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


UA_StatusCode
OpcMngCallbackSDpSetPassword(
	UA_Server *					pServer,
	const UA_NodeId *			pSessionId,
	void *						pSessionHandle,
	const UA_NodeId *			pMethodId,
	void *						pMethodContext,
	const UA_NodeId *			pObjectId,
	void *						pObjectContext,
	size_t						inputSize,
	const UA_Variant *			pInput,
	size_t						outputSize,
	UA_Variant *				pOutput
	)
{
	UA_StatusCode				uaStatus = UA_STATUSCODE_GOOD;
	bool						bReturn = false;
	PSDP_DEVICE					pSDp = (PSDP_DEVICE)pMethodContext;
	UA_String *					uaValue1Ptr = (UA_String *)pInput[0].data;
	char						PasswdData[256] = {0};
	int							PasswdDataLength = 0;


	do
	{
		PasswdDataLength = uaValue1Ptr->length;
		if ((PasswdDataLength < 0) || (PasswdDataLength > 32))
		{
			uaStatus = UA_STATUSCODE_UNCERTAINENGINEERINGUNITSEXCEEDED;
			break;
		}

		memcpy(PasswdData, uaValue1Ptr->data, PasswdDataLength);

		bReturn = IntMngSDpSetPassword(pSDp, PasswdData, PasswdDataLength);
		if (bReturn != true)
		{
			uaStatus = UA_STATUSCODE_BADUNEXPECTEDERROR;
			break;
		}

	} while (false);

	return uaStatus;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


UA_StatusCode
OpcMngCallbackSDpClearPassword(
	UA_Server *					pServer,
	const UA_NodeId *			pSessionId,
	void *						pSessionHandle,
	const UA_NodeId *			pMethodId,
	void *						pMethodContext,
	const UA_NodeId *			pObjectId,
	void *						pObjectContext,
	size_t						inputSize,
	const UA_Variant *			pInput,
	size_t						outputSize,
	UA_Variant *				pOutput
	)
{
	UA_StatusCode				uaStatus = UA_STATUSCODE_GOOD;
	bool						bReturn = false;
	PSDP_DEVICE					pSDp = (PSDP_DEVICE)pMethodContext;


	do
	{
		bReturn = IntMngSDpClearPassword(pSDp);
		if (bReturn != true)
		{
			uaStatus = UA_STATUSCODE_BADUNEXPECTEDERROR;
			break;
		}

	} while (false);

	return uaStatus;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


