/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
OpcServiceCreateBasicNodeAndMethod(
	POPC_SERVICE				pService
	)
{
	UA_NodeId					uaId = UA_NODEID_NULL;
	UA_ObjectAttributes			uaAttrObj;
	UA_VariableAttributes		uaAttrVariable;
	UA_MethodAttributes			uaAttrMethod;
	UA_Boolean					uaBoolean;


	// DP root object node
	uaAttrObj = UA_ObjectAttributes_default;
	uaAttrObj.displayName = UA_LOCALIZEDTEXT("en-US", UAID_Device_port);

	UA_Server_addObjectNode(pService->uaServerPtr, UA_NODEID_STRING(1, UAID_Device_port),
		UA_NODEID_NUMERIC(0, UA_NS0ID_OBJECTSFOLDER),
		UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
		UA_QUALIFIEDNAME(1, UAID_Device_port),
		UA_NODEID_NUMERIC(0, UA_NS0ID_FOLDERTYPE), uaAttrObj, NULL, &uaId);


	// DP service start up variable node
	uaBoolean = SDpServiceIsStartUp?true:false;
	uaAttrVariable = UA_VariableAttributes_default;
	UA_Variant_setScalar(&uaAttrVariable.value, &uaBoolean, &UA_TYPES[UA_TYPES_BOOLEAN]);
	uaAttrVariable.displayName = UA_LOCALIZEDTEXT("en-US", UAID_Service_start_up);
	uaAttrVariable.dataType = UA_TYPES[UA_TYPES_BOOLEAN].typeId;
	uaAttrVariable.accessLevel = UA_ACCESSLEVELMASK_READ;

	UA_Server_addVariableNode(pService->uaServerPtr, UA_NODEID_STRING(1, UAID_Service_start_up), 
		uaId,
		UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
		UA_QUALIFIEDNAME(1, UAID_Service_start_up),
		UA_NODEID_NULL, uaAttrVariable, NULL, NULL);


	// search method
	uaAttrMethod = UA_MethodAttributes_default;
	uaAttrMethod.displayName = UA_LOCALIZEDTEXT("en-US", UAID_Search_available_device);
	uaAttrMethod.executable = true;
	uaAttrMethod.userExecutable = true;

	UA_Server_addMethodNode(pService->uaServerPtr, UA_NODEID_STRING(1, UAID_Search_available_device),
		uaId,
		UA_NODEID_NUMERIC(0, UA_NS0ID_HASORDEREDCOMPONENT),
		UA_QUALIFIEDNAME(1, UAID_Search_available_device),
		uaAttrMethod, &OpcMngCallbackSearchSDp,
		0, NULL, 0, NULL, NULL, NULL);


	// DP search list object node
	uaAttrObj = UA_ObjectAttributes_default;
	uaAttrObj.displayName = UA_LOCALIZEDTEXT("en-US", UAID_Search_device_list);

	UA_Server_addObjectNode(pService->uaServerPtr, UA_NODEID_STRING(1, UAID_Search_device_list),
		uaId,
		UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
		UA_QUALIFIEDNAME(1, UAID_Search_device_list),
		UA_NODEID_NUMERIC(0, UA_NS0ID_FOLDERTYPE), uaAttrObj, NULL, NULL);


	// DP connected list object node
	uaAttrObj = UA_ObjectAttributes_default;
	uaAttrObj.displayName = UA_LOCALIZEDTEXT("en-US", UAID_Connect_device_list);

	UA_Server_addObjectNode(pService->uaServerPtr, UA_NODEID_STRING(1, UAID_Connect_device_list),
		uaId,
		UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
		UA_QUALIFIEDNAME(1, UAID_Connect_device_list),
		UA_NODEID_NUMERIC(0, UA_NS0ID_FOLDERTYPE), uaAttrObj, NULL, NULL);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
OpcServiceDeleteBasicNodeAndMethod(
	POPC_SERVICE				pService
	)
{
	UA_NodeId					uaId = UA_NODEID_NULL;


	// DP connected list object node
	uaId = UA_NODEID_STRING(1, UAID_Connect_device_list);
	UA_Server_deleteNode(pService->uaServerPtr, uaId, true);


	// DP search list object node
	uaId = UA_NODEID_STRING(1, UAID_Search_device_list);
	UA_Server_deleteNode(pService->uaServerPtr, uaId, true);


	// DP root object node
	uaId = UA_NODEID_STRING(1, UAID_Device_port);
	UA_Server_deleteNode(pService->uaServerPtr, uaId, true);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
OpcServiceChangeDpServiceStartUpState(
	POPC_SERVICE				pService,
	bool						State
	)
{
	UA_Boolean 					uaBoolean = State?true:false;
	UA_Variant					uaVariant;


	UA_Variant_init(&uaVariant);
	UA_Variant_setScalar(&uaVariant, &uaBoolean, &UA_TYPES[UA_TYPES_BOOLEAN]);
	UA_Server_writeValue(pService->uaServerPtr, UA_NODEID_STRING(1, UAID_Service_start_up), uaVariant);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


