/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


static bool
_CreateSDpDevice(
	PSDP_DEVICE					pSDp,
	PDP_INFO					pDpInfo,
	bool						IsConnectedByHost,
	unsigned char *				pPassword,
	unsigned int				PasswordLength
	)
{
	bool						bSuccess = false;
	int							Index = 0;
	int							i = 0;
	unsigned int				ObjIndex = 1;


	do
	{
		SxxListInit(&pSDp->Entry);
		Index = SDpMainIndexAlloc();
		if (Index == -1)
		{
			break;
		}
		pSDp->Index = (unsigned char)Index;
		memcpy(&pSDp->DpInfo, pDpInfo, sizeof(DP_INFO));

		pSDp->HasDeviceName = false;
		memset(pSDp->DeviceName, 0, sizeof(pSDp->DeviceName));

		pSDp->IsConnectedByHost = IsConnectedByHost;
		
		memset(pSDp->Password, 0, sizeof(pSDp->Password));
		pSDp->PasswordLength = 0;
		if (PasswordLength > 0)
		{
			memcpy(pSDp->Password, pPassword, PasswordLength);
			pSDp->PasswordLength = PasswordLength;
		}

		for (i = 0; i < pSDp->DpInfo.Channels; i++)
		{
			memcpy(&pSDp->DibArray[i].Info, &pDpInfo->DibInfo[i], sizeof(DIB_INFO));
			pSDp->DibArray[i].SDpPtr = pSDp;

			if ((pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_CONTROL_UNIT) || (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_VIRTUAL))
			{
				//printf("DIB Type:x%02x, index = 0\n", pSDp->DibArray[i].Info.Type);
				pSDp->DibArray[i].ObjIndex = 0;

				if (pSDp->DibArray[i].Info.Type == DLLP_CHL_TYPE_VIRTUAL)
				{
					pSDp->HasDeviceName = true;
					memcpy(pSDp->DeviceName, pSDp->DibArray[i].Info.StrInfo.DeviceName, pSDp->DibArray[i].Info.StrInfo.DeviceNameLen);
				}
			}
			else
			{
				//printf("DIB Type:x%02x, index = %d\n", pSDp->DibArray[i].Info.Type, ObjIndex);
				pSDp->DibArray[i].ObjIndex = ObjIndex;
				ObjIndex++;
			}
		}

		bSuccess = true;

	} while (false);

	if (bSuccess != true)
	{

	}

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void
_FreeSDpDevice(
	PSDP_DEVICE					pSDp
	)
{
	do
	{

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------------------------------------*/


int
SDpMainIndexAlloc(
	void
	)
{
	int							index = 0;
	int							tryCnt = 0;


	index = 1;

	pthread_mutex_lock(&SDpIndexActiveMutex);
	while (1)
	{
		if (SDpIndexActive[index] == false)
		{
			SDpIndexActive[index] = true;
			break;
		}
	
		index++;
		if (index >= SDP_INDEX_NUMBER_MAX)
		{
			index = 1;
		}

		tryCnt++;
		if (tryCnt > SDP_INDEX_NUMBER_MAX)
		{
			index = -1;
			//printf("SDP can't get available index\n");
			break;
		}
	}

	pthread_mutex_unlock(&SDpIndexActiveMutex);

	return index;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
SDpMainIndexFree(
	unsigned char				Index
	)
{
	if (Index < 0x01)
	{
		return;
	}

	pthread_mutex_lock(&SDpIndexActiveMutex);
	SDpIndexActive[Index] = false;
	pthread_mutex_unlock(&SDpIndexActiveMutex);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


PSDP_DEVICE
SDpMainGetByMac(
	unsigned char *				pMac
	)
{
	PSDP_DEVICE					pSDp = NULL;
	DLIST *						pListHead = &SDpList;
	DLIST *						e = NULL;
	bool						bMatch = false;


	do
	{
		if (!SxxListEmpty(pListHead))
		{
			e = pListHead->Flink;
		}
		else
		{
			break;
		}


		while ((e != NULL) && (e != pListHead))
		{
			if (e != NULL)
			{
				pSDp = SDP_DEVICE_PTR(e);
				if (pSDp != NULL)
				{
					if ((pSDp->DpInfo.Mac[0] == pMac[0]) && 
						(pSDp->DpInfo.Mac[1] == pMac[1]) && 
						(pSDp->DpInfo.Mac[2] == pMac[2]) &&
						(pSDp->DpInfo.Mac[3] == pMac[3]) &&
						(pSDp->DpInfo.Mac[4] == pMac[4]) &&
						(pSDp->DpInfo.Mac[5] == pMac[5]))
					{
						bMatch = true;
						break;
					}
				}

				e = e->Flink;
			}
		}

	} while (false);

	if (bMatch == true)
	{

	}
	else
	{
		pSDp = NULL;
	}

	return pSDp;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


PSDP_DEVICE
SDpMainGetByIndex(
	unsigned char 				Index
	)
{
	PSDP_DEVICE					pSDp = NULL;
	DLIST *						pListHead = &SDpList;
	DLIST *						e = NULL;
	bool						bMatch = false;


	do
	{
		if (!SxxListEmpty(pListHead))
		{
			e = pListHead->Flink;
		}
		else
		{
			break;
		}


		while ((e != NULL) && (e != pListHead))
		{
			if (e != NULL)
			{
				pSDp = SDP_DEVICE_PTR(e);
				if (pSDp != NULL)
				{
					if (pSDp->Index == Index)
					{
						bMatch = true;
						break;
					}
				}

				e = e->Flink;
			}
		}

	} while (false);

	if (bMatch == true)
	{

	}
	else
	{
		pSDp = NULL;
	}

	return pSDp;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


bool
SDpMainAdd(
	PDP_INFO					pDpInfo,
	bool						IsConnectedByHost,
	unsigned char *				pPassword,
	unsigned int				PasswordLength
	)
{
	bool						bSuccess = false;
	bool						bReturn = false;
	PSDP_DEVICE					pSDp = NULL;


	do
	{
		pSDp = SDpMainGetByMac(pDpInfo->Mac);
		if (pSDp != NULL)
		{
			break;
		}


		pSDp = (PSDP_DEVICE)malloc(sizeof(SDP_DEVICE));
		if (pSDp == NULL)
		{
			break;
		}
		memset(pSDp, 0, sizeof(SDP_DEVICE));


		bReturn = _CreateSDpDevice(pSDp, pDpInfo, IsConnectedByHost, pPassword, PasswordLength);
		if (bReturn != true)
		{
			break;
		}


		pthread_mutex_lock(&SDpListMutex);
		SxxListInsertTail(&SDpList, &pSDp->Entry);
		SDpAmount++;
		pthread_mutex_unlock(&SDpListMutex);


		if (IsConnectedByHost == true)
		{
			OpcMngAddSDpForConnect(pSDp);
		}
		else
		{
			OpcMngAddSDpForSearch(pSDp);
		}


		bSuccess = true;

		//printf("SDP (:%.2x:%.2x:%.2x), add\n", pSDp->DpInfo.Mac[3], pSDp->DpInfo.Mac[4], pSDp->DpInfo.Mac[5]);

	} while (false);

	if (bSuccess != true)
	{
		free(pSDp);
		pSDp = NULL;
	}

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
SDpMainDel(
	unsigned char *				pMac
	)
{
	PSDP_DEVICE					pSDp = NULL;
	unsigned char				Mac[6] = {0};


	do
	{
		pthread_mutex_lock(&SDpListMutex);

		pSDp = SDpMainGetByMac(pMac);
		if (pSDp != NULL)
		{
			if (pSDp->IsConnectedByHost == true)
			{
				OpcMngDelSDpForConnect(pSDp);
				LibSioDisconnect(pSDp->DpInfo.Mac);
				LibCtuDisconnect(pSDp->DpInfo.Mac);
			}
			else
			{
				OpcMngDelSDpForSearch(pSDp);
			}

			memcpy(Mac, pSDp->DpInfo.Mac, 6);

			SxxListRemoveEntry(&pSDp->Entry);

			SDpMainIndexFree(pSDp->Index);

			_FreeSDpDevice(pSDp);

			//printf("SDP (:%.2x:%.2x:%.2x), del\n", Mac[3], Mac[4], Mac[5]);
			free(pSDp);
			SDpAmount--;
		}

		pthread_mutex_unlock(&SDpListMutex);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
SDpMainDelAllWithoutConnectedByHost(
	void
	)
{
	PSDP_DEVICE					pSDp = NULL;
	unsigned char				Mac[6] = {0};
	unsigned char				Index = 0;
	int							Amount = 0;


	do
	{
		pthread_mutex_lock(&SDpListMutex);

		Amount = SDpAmount;

		while (Amount > 0)
		{
			pSDp = SDpMainGetByIndex(Index);
			if (pSDp != NULL)
			{
				if (pSDp->IsConnectedByHost == false)
				{
					OpcMngDelSDpForSearch(pSDp);

					memcpy(Mac, pSDp->DpInfo.Mac, 6);

					SxxListRemoveEntry(&pSDp->Entry);

					SDpMainIndexFree(pSDp->Index);

					_FreeSDpDevice(pSDp);

					//printf("SDP (:%.2x:%.2x:%.2x), del\n", Mac[3], Mac[4], Mac[5]);
					free(pSDp);
					SDpAmount--;
				}
				Amount--;
			}

			Index++;
			if (Index > SDP_INDEX_NUMBER_MAX)
			{
				break;
			}
		}

		pthread_mutex_unlock(&SDpListMutex);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


void
SDpMainDelAll(
	void
	)
{
	PSDP_DEVICE					pSDp = NULL;
	DLIST *						pListHead = &SDpList;
	DLIST *						e = NULL;
	unsigned char				Mac[6] = {0};


	do
	{
		pthread_mutex_lock(&SDpListMutex);

		while (!SxxListEmpty(pListHead))
		{
			e = SxxListGetNext(pListHead);
			if (e != NULL)
			{
				pSDp = SDP_DEVICE_PTR(e);
				if (pSDp != NULL)
				{
					if (pSDp->IsConnectedByHost == true)
					{
						OpcMngDelSDpForConnect(pSDp);
						LibSioDisconnect(pSDp->DpInfo.Mac);
						LibCtuDisconnect(pSDp->DpInfo.Mac);
					}
					else
					{
						OpcMngDelSDpForSearch(pSDp);
					}

					memcpy(Mac, pSDp->DpInfo.Mac, 6);

					SxxListRemoveEntry(&pSDp->Entry);

					SDpMainIndexFree(pSDp->Index);

					_FreeSDpDevice(pSDp);

					//printf("SDP (:%.2x:%.2x:%.2x), del\n", Mac[3], Mac[4], Mac[5]);
					free(pSDp);
					SDpAmount--;
				}
			}
		}

		pthread_mutex_unlock(&SDpListMutex);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


