/***********************************************************************
 *
 *  Module      : 
 *  Description : 
 *
 *
 *  Author(s)   : Jason Lee
 *  Company     : SUNIX Co., Ltd.
 *  Maintain    : Jason Lee
 ***********************************************************************/


#include "sxx_precomp.h"


/*-----------------------------------------------------------------------------------------------------------------------------*/


pthread_mutex_t							DbgLock;
bool									DbgMngIpcProtocol;
bool									DbgSioIpcProtocol;
bool									PrintfEnable;
bool									LogEnable;
FILE *									LogNode;
char *									LogBuff;

bool									SDpIndexActive[SDP_INDEX_NUMBER_MAX];
pthread_mutex_t							SDpIndexActiveMutex;
int										SDpAmount;
DLIST									SDpList;
pthread_mutex_t							SDpListMutex;

bool									SDpServiceIsStartUp;

LIB_SERVICE								LibService;
OPC_SERVICE								OpcService;


/*-----------------------------------------------------------------------------------------------------------------------------*/


#if (BUILD_FOR_DAEMON != 1)
static void
_PrintInfo(
	void
	)
{
	printf("-----------------------------------------\n");
	printf(" Program Name      : %s\n", OBJ_NAME);
	printf(" Program Version   : %s\n", OBJ_VERSION);
	printf("-----------------------------------------\n\n");
	printf("======== Ctrl+C to exit program ========\n\n");
}
#endif


/*-----------------------------------------------------------------------------------------------------------------------------*/


static bool 
_InitSDpFromLibraryFunc(
	void
	)
{
	PDP_LIST					pDpList = NULL;
	bool						bReturn = false;
	int							i = 0;
	int							j = 0;
	unsigned char				Password[256];
	unsigned int				PasswordLength = 0;


	do
	{
		pDpList = (PDP_LIST)malloc(sizeof(DP_LIST));
		if (pDpList == NULL)
		{
			break;
		}
		memset(pDpList, 0, sizeof(DP_LIST));

		bReturn = LibMngGetAllDpConnectedInfo(pDpList);
		if (bReturn != true)
		{
			break;
		}

		for (i = 0 ; i < pDpList->DpAmount; i++)
		{
			bReturn = LibMngGetSpecificDpDetailInfo(pDpList->DpInfo[i].Mac, &pDpList->DpInfo[i]);
			if (bReturn != true)
			{
				//printf("SXX (:%.2x:%.2x:%.2x), InitFromLib, get detail info fail\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
				continue;
			}

			bReturn = LibMngSetDisconnectEventCallback(pDpList->DpInfo[i].Mac);
			if (bReturn != true)
			{
				//printf("SXX (:%.2x:%.2x:%.2x), InitFromLib, LibMngSetDisconnectEventCallback fail\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
				continue;
			}

			//usleep(200 * 1000);

			bReturn = LibCtuConnect(pDpList->DpInfo[i].Mac);
			if (bReturn != true)
			{
				//printf("SXX (:%.2x:%.2x:%.2x), InitFromLib, LibCtuConnect fail\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
				continue;
			}

			bReturn = LibSioConnect(pDpList->DpInfo[i].Mac, pDpList->DpInfo[i].SioTcpPort);
			if (bReturn != true)
			{
				//printf("SXX (:%.2x:%.2x:%.2x), InitFromLib, LibSioConnect fail\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
				continue;
			}


			memset(Password, 0, sizeof(Password));
			PasswordLength = 0;
			if (pDpList->DpInfo[i].PasswordEnable == 0x01)
			{
				bReturn = LibMngGetPassword(pDpList->DpInfo[i].Mac, Password, &PasswordLength);
				if (bReturn != true)
				{
					//printf("SXX (:%.2x:%.2x:%.2x), InitFromLib, get password fail\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
				}
			}


			for (j = 0; j < pDpList->DpInfo[i].Channels; j++)
			{
				if (pDpList->DpInfo[i].DibInfo[j].Type == DLLP_CHL_TYPE_UART)
				{
					//printf("SXX (:%.2x:%.2x:%.2x), InitFromLib, get SER info for opc\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
				}
				if (pDpList->DpInfo[i].DibInfo[j].Type == DLLP_CHL_TYPE_DIO)
				{
					bReturn = LibDioGetInfo(pDpList->DpInfo[i].Mac, pDpList->DpInfo[i].DibInfo[j].ChlNumber, &pDpList->DpInfo[i].DibInfo[j].DioInfo);
					//printf("SXX (:%.2x:%.2x:%.2x), InitFromLib, get DIO info for opc, bReturn:%s\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5], bReturn?"TRUE":"FALSE");
				}
				if (pDpList->DpInfo[i].DibInfo[j].Type == DLLP_CHL_TYPE_MULTIFUNC_AI)
				{
					bReturn = LibMAiGetInfo(pDpList->DpInfo[i].Mac, pDpList->DpInfo[i].DibInfo[j].ChlNumber, &pDpList->DpInfo[i].DibInfo[j].MaiInfo);
					//printf("SXX (:%.2x:%.2x:%.2x)(%d), InitFromLib, get MAi info for opc, bReturn:%s, MAI_Type:x%02x\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5], pDpList->DpInfo[i].DibInfo[j].ChlNumber, bReturn?"TRUE":"FALSE", pDpList->DpInfo[i].DibInfo[j].MaiInfo.MAI_Type);
				}
			}


			bReturn = SDpMainAdd(&pDpList->DpInfo[i], true, Password, PasswordLength);
			if (bReturn != true)
			{
				//printf("SXX (:%.2x:%.2x:%.2x), add fail\n", pDpList->DpInfo[i].Mac[3], pDpList->DpInfo[i].Mac[4], pDpList->DpInfo[i].Mac[5]);
			}

		}

	} while (false);

	if (pDpList != NULL)
	{
		free(pDpList);
		pDpList = NULL;
	}

	return true;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static bool 
_InitProgFunc(
	void
	)
{
	bool						bSuccess = false;
	int							nReturn = 0;
	bool						bReturn = false;
	int							nStatus = MNG_STATUS_SUCCESS;
	int							i = 0;
	bool						bStep_11 = false;
	bool						bStep_12 = false;
	bool						bStep_13 = false;

	bool						bStep_21 = false;
	bool						bStep_22 = false;
	bool						bStep_31 = false;
	bool						bStep_41 = false;
	bool						bStep_42 = false;
	bool						bStep_a1 = false;
	bool						bStep_b1 = false;


	do
	{
		nReturn = pthread_mutex_init(&DbgLock, NULL);
		if (nReturn != 0)
		{
			break;
		}
		bStep_11 = true;


		DbgMngIpcProtocol = false;
		DbgSioIpcProtocol = false;
		PrintfEnable = false;
		LogEnable = true;
		LogNode = NULL;
		LogBuff = NULL;
		SDpServiceIsStartUp = false;


		LogBuff = (char *)malloc(BUFF_SIZE);
		if (LogBuff == NULL)
		{
			break;
		}
		bStep_12 = true;


		memset(LogBuff, 0, BUFF_SIZE);
		bReturn = LogOpen();
		if (bReturn != true)
		{
			break;
		}
		bStep_13 = true;


		for (i = 0; i < SDP_INDEX_NUMBER_MAX; i++)
		{
			SDpIndexActive[i] = false;
		}
		nReturn = pthread_mutex_init(&SDpIndexActiveMutex, NULL);
		if (nReturn != 0)
		{
			break;
		}
		bStep_21 = true;


		SDpAmount = 0;
		SxxListInit(&SDpList);
		nReturn = pthread_mutex_init(&SDpListMutex, NULL);
		if (nReturn != 0)
		{
			break;
		}
		bStep_22 = true;


		memset(&LibService, 0, sizeof(LIB_SERVICE));
		nStatus = dp_lib_init();
		if (nStatus != MNG_STATUS_SUCCESS)
		{
			break;
		}
		bStep_31 = true;


		memset(&OpcService, 0, sizeof(OPC_SERVICE));
		OpcService.uaIsRunning = false;
		OpcService.uaServerConfigPtr = NULL;
		OpcService.uaServerPtr = NULL;

		OpcService.uaServerConfigPtr = UA_ServerConfig_new_default();
		if (OpcService.uaServerConfigPtr == NULL)
		{
			break;
		}
		bStep_41 = true;


		OpcService.uaServerPtr = UA_Server_new(OpcService.uaServerConfigPtr);
		if (OpcService.uaServerPtr == NULL)
		{
			break;
		}
		bStep_42 = true;

		bReturn = LibServiceStart(&LibService);
		if (bReturn != true)
		{
			break;
		}
		bStep_a1 = true;


		OpcServiceCreateBasicNodeAndMethod(&OpcService);


		bReturn = _InitSDpFromLibraryFunc();
		if (bReturn != true)
		{
			break;
		}
		bStep_b1 = true;


		bSuccess = true;

	} while (false);

	if (bSuccess != true)
	{
		if (bStep_b1 == true)
		{

		}

		if (bStep_a1 == true)
		{
			LibServiceStop(&LibService);
		}

		if (bStep_42 == true)
		{
			if (OpcService.uaServerPtr != NULL)
			{
				UA_Server_delete(OpcService.uaServerPtr);
				OpcService.uaServerPtr = NULL;
			}
		}

		if (bStep_41 == true)
		{
			if (OpcService.uaServerConfigPtr != NULL)
			{
				UA_ServerConfig_delete(OpcService.uaServerConfigPtr);
				OpcService.uaServerConfigPtr = NULL;
			}
		}

		if (bStep_31 == true)
		{
			dp_lib_free();
		}

		if (bStep_22 == true)
		{
			pthread_mutex_destroy(&SDpListMutex);
		}

		if (bStep_21 == true)
		{
			for (i = 0; i < SDP_INDEX_NUMBER_MAX; i++)
			{
				SDpIndexActive[i] = false;
			}
			pthread_mutex_destroy(&SDpIndexActiveMutex);
		}

		if (bStep_13 == true)
		{
			LogClose();
		}

		if (bStep_12 == true)
		{
			if (LogBuff != NULL)
			{
				free(LogBuff);
				LogBuff = NULL;
			}
		}

		if (bStep_11 == true)
		{
			pthread_mutex_destroy(&DbgLock);
		}
	}

	return bSuccess;
}


/*-----------------------------------------------------------------------------------------------------------------------------*/


static void
_FreeProgFunc(
	void
	)
{
	int							i = 0;


	do
	{
		LibServiceStop(&LibService);


		SDpMainDelAll();


		OpcServiceDeleteBasicNodeAndMethod(&OpcService);


		if (OpcService.uaServerPtr != NULL)
		{
			UA_Server_delete(OpcService.uaServerPtr);
			OpcService.uaServerPtr = NULL;
		}


		if (OpcService.uaServerConfigPtr != NULL)
		{
			UA_ServerConfig_delete(OpcService.uaServerConfigPtr);
			OpcService.uaServerConfigPtr = NULL;
		}


		LibMngFree();


		pthread_mutex_destroy(&SDpListMutex);
		for (i = 0; i < SDP_INDEX_NUMBER_MAX; i++)
		{
			SDpIndexActive[i] = false;
		}
		pthread_mutex_destroy(&SDpIndexActiveMutex);


		LogClose();
		if (LogBuff != NULL)
		{
			free(LogBuff);
			LogBuff = NULL;
		}
		pthread_mutex_destroy(&DbgLock);

	} while (false);
}


/*-----------------------------------------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------------------------------------*/


#if (BUILD_FOR_DAEMON != 1)
void
sig_handler(
	int							arg
	)
{
	OpcService.uaIsRunning = false;
}


int
main(
	int							argc,
	char *						argv[]
	)
{
	bool						bReturn = false;


	system("clear");
	_PrintInfo();

	bReturn = _InitProgFunc();
	if (bReturn != true)
	{
		printf("Init fail, exit program\n");
		SxxGetche();
		exit(-1);
	}


	signal(SIGINT, sig_handler);


	OpcService.uaIsRunning = true;
	UA_Server_run(OpcService.uaServerPtr, &OpcService.uaIsRunning);


	_FreeProgFunc();
	printf("\n\n======== Exit program ========\n\n");
	return 0;
}
#else
#define DAEMON_NAME "sdxopcua"
#define PIDFILE "/var/run/sdxopcua.pid"


void
sig_handler(
	int							arg
	)
{
	OpcService.uaIsRunning = false;
}


int
main(
	int							argc,
	char *						argv[]
	)
{
	bool						bReturn = false;
	pid_t						pid;
	int							pid_fd;
	int							fd = 0;


	openlog(DAEMON_NAME, LOG_PID, LOG_DAEMON);


	printf("SDXOPCUA : Version - %s\n", OBJ_VERSION);


	pid = fork();
	if (pid < 0)
	{
		syslog(LOG_INFO, "Fork off the parent process failure.");
		exit(EXIT_FAILURE);
	}
	if (pid > 0)
	{
		exit(EXIT_SUCCESS);
	}


	if (setsid() < 0)
	{
		syslog(LOG_INFO, "Create a new SID for the child process failure.");
		exit(EXIT_FAILURE);
	}


	signal(SIGINT, SIG_IGN);


	umask(0);


	if ((chdir("/")) < 0)
	{
		syslog(LOG_INFO, "Change the current working directory failure.");
		exit(EXIT_FAILURE);
	}


	for (fd = sysconf(_SC_OPEN_MAX); fd > 0; fd--)
	{
		close(fd);
	}


	// mark this, cause of daemon will not success, (opcua62541 ?)
	//close(STDIN_FILENO);
	close(STDOUT_FILENO);
	close(STDERR_FILENO);


	errno = 0;
	pid_fd = open(PIDFILE, O_RDWR | O_CREAT, 0666);
	if (pid_fd >= 0)
	{
		char str[256] = {0};
		sprintf(str, "%d\n", getpid());
		write(pid_fd, str, strlen(str));
		close(pid_fd);
	}


	signal(SIGTERM, sig_handler);


	bReturn = _InitProgFunc();
	if (bReturn != true)
	{
		syslog(LOG_INFO, "Init service failure.");
		exit(EXIT_FAILURE);
	}


	OpcService.uaIsRunning = true;
	UA_Server_run(OpcService.uaServerPtr, &OpcService.uaIsRunning);


	_FreeProgFunc();
	closelog();
	return EXIT_SUCCESS;
}
#endif


/*-----------------------------------------------------------------------------------------------------------------------------*/


